#!/bin/csh -f

#
# restore configuration registers of HIBs saved by saveconfig.csh,
# so that they can be used without rebooting the host after
# FPGA reconfiguration.
#

set cmdname  = $0:t;
set rootpath = $0:h;
if (${rootpath} == '.') then
    set rootpath = '..'
else
    set rootpath = ${rootpath:s/scripts//};
endif


set tmpconf = ".tmpconf"

echo ''
echo -n "Searching for HIB(s)... "

if (-x /sbin/lspci) then
    set lspci = /sbin/lspci
else
    if (-x `which lspci`) then
        set lspci = `which lspci`
    else
      echo command \'lspci\' not found.
      echo you may need to set PATH so that it includes
      echo the directory where lspci is located.
      goto LABEL_ERROR_EXIT
    endif
endif

(${lspci} -d '1556:0070' | wc | awk '{print $1}' > /tmp/tmp$$) >& /dev/null
set nxhib = `cat /tmp/tmp$$`
\rm -f /tmp/tmp$$
echo -n "Found ${nxhib} PCI-X HIB(s). "

(${lspci} -d '1556:0e70' | wc | awk '{print $1}' > /tmp/tmp$$) >& /dev/null
set nehib = `cat /tmp/tmp$$`
\rm -f /tmp/tmp$$
echo "Found ${nehib} PCIe HIB(s)."

set nhib = ${nxhib}
@ nhib += ${nehib}
echo "Found ${nhib} HIB(s) in total."
echo ''

#
# restore BARs of each card.
#
if (! -e ${tmpconf}) then
    echo cannot find ${tmpconf}, which should have values of configuration registers to be restored.
    exit 1
else

set nline = 0;
while ($nhib > 0)
    @ nhib--;
    echo "Restoring configuration registers of HIB[$nhib]..."
    @ nline++;
    set val = `awk "{if (NR == "$nline") {print $1}}" ${tmpconf}`
    echo $val
    ${rootpath}/hibutil/hibtest 5 0x00000010 $val $nhib;

    @ nline++;
    set val = `awk "{if (NR == "$nline") {print $1}}" ${tmpconf}`
    echo $val
    ${rootpath}/hibutil/hibtest 5 0x00000014 $val $nhib;

    @ nline++;
    set val = `awk "{if (NR == "$nline") {print $1}}" ${tmpconf}`
    echo $val
    ${rootpath}/hibutil/hibtest 5 0x00000018 $val $nhib;

end
echo "Done."
echo ''

exit 0

LABEL_ERROR_EXIT:

exit 1
