#!/bin/csh -f

#
# save configuration registers of HIBs so that they can be restored
# without rebooting the host after FPGA reconfiguration.
#

set cmdname  = $0:t;
set rootpath = $0:h;
if (${rootpath} == '.') then
    set rootpath = '..'
else
    set rootpath = ${rootpath:s/scripts//};
endif


set tmpconf = ".tmpconf"

echo ''
echo -n "Searching for HIB(s)... "

if (-x /sbin/lspci) then
    set lspci = /sbin/lspci
else
    if (-x `which lspci`) then
        set lspci = `which lspci`
    else
      echo command \'lspci\' not found.
      echo you may need to set PATH so that it includes
      echo the directory where lspci is located.
      goto LABEL_ERROR_EXIT
    endif
endif

(${lspci} -d '1556:0070' | wc | awk '{print $1}' > /tmp/tmp$$) >& /dev/null
set nxhib = `cat /tmp/tmp$$`
\rm -f /tmp/tmp$$
echo -n "Found ${nxhib} PCI-X HIB(s). "

(${lspci} -d '1556:0e70' | wc | awk '{print $1}' > /tmp/tmp$$) >& /dev/null
set nehib = `cat /tmp/tmp$$`
\rm -f /tmp/tmp$$
echo "Found ${nehib} PCIe HIB(s)."

set nhib = ${nxhib}
@ nhib += ${nehib}
echo "Found ${nhib} HIB(s) in total."
echo ''

#
# save BARs of each card.
#
rm ${tmpconf} >& /dev/null;
while ($nhib > 0)
    @ nhib--;
    echo "Saving configuration registers of HIB[$nhib]..."
    set base = `${rootpath}/hibutil/hibtest 0 $nhib | & awk '/0x00000010/{addr = $3; print $3}'`;
    echo "$base" >> ${tmpconf};
    set base = `${rootpath}/hibutil/hibtest 0 $nhib | & awk '/0x00000014/{addr = $3; print $3}'`;
    echo "$base" >> ${tmpconf};
    set base = `${rootpath}/hibutil/hibtest 0 $nhib | & awk '/0x00000018/{addr = $3; print $3}'`;
    echo "$base" >> ${tmpconf};
end
echo "Done."
echo ''

exit 0

LABEL_ERROR_EXIT:

exit 1
