/*
 * a sample program to show HIB user library usage.
 */

#include <stdio.h>
#include <stdlib.h>
#include "hibutil.h"


int
main(void)
{
    int i;
    int size = 10; // size in 8-byte word. size should not exceed HIBDRV_DMABUF_BYTES/8.
    Hib *h;
    static UINT64 piowbuf[1024]; // buffer for data to be sent to HIB(s).
    UINT64 *dmawbuf[NHIB];       // pointers to buffers for data to be received from HIB(s).
    int devid = 0;               // Assume only one HIB is installed, whose device id should be 0.
                                 // Note that "device id" used here is a number defined and used by
                                 // HIB user library only, and has nothing to do with "PCI Device
                                 // ID" defined by PCI Spec.

    h = hib_openMC(devid);
    dmawbuf[devid] = h->dmaw_buf;
    // As a receiving buffer for DMA wtite transfer (write from HIB to
    // host), a buffer allocated by hib_openMC() should be used.  A
    // buffer allocated by an usual method, such as malloc(), cannot
    // be used.

    for (i = 0; i < size; i++) {
	piowbuf[i] = 0x123456789abc0000LL|i; // data to be sent.
	dmawbuf[devid][i] = 0xfedcba9876540000LL|i; // dummy data to be overwritten.
    }

    hib_start_dmawMC(devid, size, dmawbuf[devid]); // DMA write kickoff may or may not precede PIO write.
    hib_piowMC(devid, size, piowbuf); // perform PIO write.
    hib_finish_dmawMC(devid); // wait completion of DMA write.

    // compare data sent and received.
    for (i = 0; i < size; i++) {
        printf("0x%04x  sent : 0x%016llx    received : 0x%016llx  ",
               i, piowbuf[i], dmawbuf[devid][i]);
        if (piowbuf[i] == dmawbuf[devid][i]) {
            printf("OK\n");
        }
        else {
            printf("NG\n");
        }
    }
    hib_closeMC(0);

    exit(0);
}
