library ieee;
use ieee.std_logic_1164.all;

entity alt2gxb_agx_2x4 is
  port (
    cal_blk_clk             : in std_logic ;
    fixedclk                : in std_logic ;
    gxb_powerdown           : in std_logic_vector (0 downto 0);
    pipe8b10binvpolarity    : in std_logic_vector (7 downto 0);
    pll_inclk               : in std_logic ;
    powerdn                 : in std_logic_vector (15 downto 0);
    rx_analogreset          : in std_logic_vector (0 downto 0);
    rx_coreclk              : in std_logic_vector (7 downto 0);
    rx_datain               : in std_logic_vector (7 downto 0);
    rx_digitalreset         : in std_logic_vector (0 downto 0);
    tx_coreclk              : in std_logic_vector (7 downto 0);
    tx_ctrlenable           : in std_logic_vector (15 downto 0);
    tx_datain               : in std_logic_vector (127 downto 0);
    tx_detectrxloop         : in std_logic_vector (7 downto 0);
    tx_digitalreset         : in std_logic_vector (0 downto 0);
    tx_forcedispcompliance  : in std_logic_vector (7 downto 0);
    tx_forceelecidle        : in std_logic_vector (7 downto 0);
    coreclkout              : out std_logic_vector (1 downto 0);
    pipedatavalid           : out std_logic_vector (7 downto 0);
    pipeelecidle            : out std_logic_vector (7 downto 0);
    pipephydonestatus       : out std_logic_vector (7 downto 0);
    pipestatus              : out std_logic_vector (23 downto 0);
    pll_locked              : out std_logic_vector (1 downto 0);
    rx_ctrldetect           : out std_logic_vector (15 downto 0);
    rx_dataout              : out std_logic_vector (127 downto 0);
    rx_disperr              : out std_logic_vector (15 downto 0);
    rx_errdetect            : out std_logic_vector (15 downto 0);
    rx_freqlocked           : out std_logic_vector (7 downto 0);
    rx_patterndetect        : out std_logic_vector (15 downto 0);
    rx_pll_locked           : out std_logic_vector (7 downto 0);
    rx_syncstatus           : out std_logic_vector (15 downto 0);
    tx_dataout              : out std_logic_vector (7 downto 0)
  );
end alt2gxb_agx_2x4;


architecture syn of alt2gxb_agx_2x4 is

  component myalt2gxb_agx_x4
    port (
      cal_blk_clk            : in  std_logic;
      fixedclk               : in  std_logic;
      pll_inclk              : in  std_logic;
      powerdn                : in  std_logic_vector(4*2-1 downto 0);

      pll_locked             : out std_logic_vector((4-1)/4 downto 0);
      gxb_powerdown          : in  std_logic_vector(0 downto 0);
      tx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_analogreset         : in  std_logic_vector(0 downto 0);
      rx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_pll_locked          : out std_logic_vector(4-1 downto 0);
      rx_freqlocked          : out std_logic_vector(4-1 downto 0);

      coreclkout             : out std_logic_vector((4-1)/4 downto 0);
--      tx_clkout              : out std_logic_vector(0 downto 0);  -- x1 only

      tx_coreclk             : in  std_logic_vector(4-1 downto 0);
      rx_coreclk             : in  std_logic_vector(4-1 downto 0);

      tx_dataout             : out std_logic_vector(4-1 downto 0);
      tx_datain              : in  std_logic_vector(4*16-1 downto 0);
      tx_ctrlenable          : in  std_logic_vector(4*2-1 downto 0);
      tx_forceelecidle       : in  std_logic_vector(4-1 downto 0);
      tx_detectrxloop        : in  std_logic_vector(4-1 downto 0);
      tx_forcedispcompliance : in  std_logic_vector(4-1 downto 0);
      pipe8b10binvpolarity   : in  std_logic_vector(4-1 downto 0);

      rx_datain              : in  std_logic_vector(4-1 downto 0);
      rx_dataout             : out std_logic_vector(4*16-1 downto 0);
      rx_ctrldetect          : out std_logic_vector(4*2-1 downto 0);
      rx_patterndetect       : out std_logic_vector(4*2-1 downto 0);
      rx_disperr             : out std_logic_vector(4*2-1 downto 0);
      rx_errdetect           : out std_logic_vector(4*2-1 downto 0);
      rx_syncstatus          : out std_logic_vector(4*2-1 downto 0);

      pipedatavalid          : out std_logic_vector(4-1 downto 0);
      pipeelecidle           : out std_logic_vector(4-1 downto 0);
      pipephydonestatus      : out std_logic_vector(4-1 downto 0);
      pipestatus             : out std_logic_vector(4*3-1 downto 0)
    );
  end component;

begin

  --
  -- combine two alt2gxb (x4 PIPE, bonded mode) to form x8 PIPE.
  --
  x4gen : for i in 0 to 1 generate
    x4 : myalt2gxb_agx_x4
      port map (
        cal_blk_clk            => cal_blk_clk,
        fixedclk               => fixedclk,
        pll_inclk              => pll_inclk,
        powerdn                => powerdn(4*2*(i+1)-1 downto 4*2*i),

        pll_locked             => pll_locked(i downto i),
        gxb_powerdown          => gxb_powerdown,
        tx_digitalreset        => tx_digitalreset,
        rx_analogreset         => rx_analogreset,
        rx_digitalreset        => rx_digitalreset,
        rx_pll_locked          => rx_pll_locked(4*(i+1)-1 downto 4*i),
        rx_freqlocked          => rx_freqlocked(4*(i+1)-1 downto 4*i),

        -- in bonded mode, one clk per GXB quad
        -- is provided by each central clock divider block.
        coreclkout             => coreclkout(i downto i),
        rx_coreclk             => rx_coreclk(4*(i+1)-1 downto 4*i),
        tx_coreclk             => tx_coreclk(4*(i+1)-1 downto 4*i),
        tx_dataout             => tx_dataout(4*(i+1)-1 downto 4*i),
        tx_datain              => tx_datain(4*16*(i+1)-1 downto 4*16*i),
        tx_ctrlenable          => tx_ctrlenable(4*2*(i+1)-1 downto 4*2*i),
        tx_forceelecidle       => tx_forceelecidle(4*(i+1)-1 downto 4*i),
        tx_detectrxloop        => tx_detectrxloop(4*(i+1)-1 downto 4*i),
        tx_forcedispcompliance => tx_forcedispcompliance(4*(i+1)-1 downto 4*i),
        
        rx_datain              => rx_datain(4*(i+1)-1 downto 4*i),
        rx_dataout             => rx_dataout(4*16*(i+1)-1 downto 4*16*i),
        rx_ctrldetect          => rx_ctrldetect(4*2*(i+1)-1 downto 4*2*i),
        rx_patterndetect       => rx_patterndetect(4*2*(i+1)-1 downto 4*2*i),
        rx_disperr             => rx_disperr(4*2*(i+1)-1 downto 4*2*i),
        rx_errdetect           => rx_errdetect(4*2*(i+1)-1 downto 4*2*i),
        rx_syncstatus          => rx_syncstatus(4*2*(i+1)-1 downto 4*2*i),

        pipe8b10binvpolarity   => pipe8b10binvpolarity(4*(i+1)-1 downto 4*i),
        pipedatavalid          => pipedatavalid(4*(i+1)-1 downto 4*i),
        pipeelecidle           => pipeelecidle(4*(i+1)-1 downto 4*i),
        pipephydonestatus      => pipephydonestatus(4*(i+1)-1 downto 4*i),
        pipestatus             => pipestatus(4*3*(i+1)-1 downto 4*3*i)
      );
  end generate x4gen;
  
end syn;


library ieee;
use ieee.std_logic_1164.all;

entity myalt2gxb_agx_x4 is
  port
    (
      cal_blk_clk             : in std_logic ;
      fixedclk                : in std_logic ;
      pll_inclk               : in std_logic ;
      powerdn                 : in std_logic_vector(7 downto 0);

      pll_locked              : out std_logic_vector(0 downto 0);
      gxb_powerdown           : in std_logic_vector(0 downto 0);
      tx_digitalreset         : in std_logic_vector(0 downto 0);
      rx_analogreset          : in std_logic_vector(0 downto 0);
      rx_digitalreset         : in std_logic_vector(0 downto 0);
      rx_pll_locked           : out std_logic_vector(3 downto 0);
      rx_freqlocked           : out std_logic_vector(3 downto 0);

      coreclkout              : out std_logic_vector(0 downto 0);
      tx_coreclk              : in std_logic_vector(3 downto 0);
      rx_coreclk              : in std_logic_vector(3 downto 0);

      tx_dataout              : out std_logic_vector(3 downto 0);
      tx_datain               : in std_logic_vector(63 downto 0);
      tx_ctrlenable           : in std_logic_vector(7 downto 0);
      tx_forceelecidle        : in std_logic_vector(3 downto 0);
      tx_detectrxloop         : in std_logic_vector(3 downto 0);
      tx_forcedispcompliance  : in std_logic_vector(3 downto 0);
      pipe8b10binvpolarity    : in std_logic_vector(3 downto 0);

      rx_datain               : in std_logic_vector(3 downto 0);
      rx_dataout              : out std_logic_vector(63 downto 0);
      rx_ctrldetect           : out std_logic_vector(7 downto 0);
      rx_patterndetect        : out std_logic_vector(7 downto 0);
      rx_disperr              : out std_logic_vector(7 downto 0);
      rx_errdetect            : out std_logic_vector(7 downto 0);
      rx_syncstatus           : out std_logic_vector(7 downto 0);

      pipedatavalid           : out std_logic_vector(3 downto 0);
      pipeelecidle            : out std_logic_vector(3 downto 0);
      pipephydonestatus       : out std_logic_vector(3 downto 0);
      pipestatus              : out std_logic_vector(11 downto 0)
    );
end myalt2gxb_agx_x4;


architecture syn of myalt2gxb_agx_x4 is

  signal sub_wire0    : std_logic_vector (7 downto 0);
  signal sub_wire1    : std_logic_vector (7 downto 0);
  signal sub_wire2    : std_logic_vector (0 downto 0);
  signal sub_wire3    : std_logic_vector (7 downto 0);
  signal sub_wire4    : std_logic_vector (3 downto 0);
  signal sub_wire5    : std_logic_vector (7 downto 0);
  signal sub_wire6    : std_logic_vector (3 downto 0);
  signal sub_wire7    : std_logic_vector (3 downto 0);
  signal sub_wire8    : std_logic_vector (3 downto 0);
  signal sub_wire9    : std_logic_vector (3 downto 0);
  signal sub_wire10    : std_logic_vector (3 downto 0);
  signal sub_wire11    : std_logic_vector (11 downto 0);
  signal sub_wire12    : std_logic_vector (7 downto 0);
  signal sub_wire13    : std_logic_vector (0 downto 0);
  signal sub_wire14    : std_logic_vector (63 downto 0);



  component alt2gxb
    generic (
      cmu_pll_inclock_period        : natural;
      cmu_pll_loop_filter_resistor_control        : natural;
      digitalreset_port_width        : natural;
      enable_fast_recovery_pci_mode        : string;
      enable_pll_inclk_alt_drive_rx_cru        : string;
      enable_pll_inclk_drive_rx_cru        : string;
      en_local_clk_div_ctrl        : string;
      equalizer_ctrl_a_setting        : natural;
      equalizer_ctrl_b_setting        : natural;
      equalizer_ctrl_c_setting        : natural;
      equalizer_ctrl_d_setting        : natural;
      equalizer_ctrl_v_setting        : natural;
      equalizer_dcgain_setting        : natural;
      intended_device_family        : string;
      loopback_mode        : string;
      lpm_type        : string;
      number_of_channels        : natural;
      operation_mode        : string;
      preemphasis_ctrl_1stposttap_setting        : natural;
      preemphasis_ctrl_2ndposttap_inv_setting        : string;
      preemphasis_ctrl_2ndposttap_setting        : natural;
      preemphasis_ctrl_pretap_inv_setting        : string;
      preemphasis_ctrl_pretap_setting        : natural;
      protocol        : string;
      receiver_termination        : string;
      reconfig_dprio_mode        : natural;
      reverse_loopback_mode        : string;
      rx_8b_10b_compatibility_mode        : string;
      rx_8b_10b_mode        : string;
      rx_align_pattern        : string;
      rx_align_pattern_length        : natural;
      rx_allow_align_polarity_inversion        : string;
      rx_allow_pipe_polarity_inversion        : string;
      rx_bandwidth_mode        : natural;
      rx_bitslip_enable        : string;
      rx_byte_ordering_mode        : string;
      rx_channel_bonding        : string;
      rx_channel_width        : natural;
      rx_common_mode        : string;
      rx_cru_pre_divide_by        : natural;
      rx_datapath_protocol        : string;
      rx_data_rate        : natural;
      rx_data_rate_remainder        : natural;
      rx_disable_auto_idle_insertion        : string;
      rx_enable_bit_reversal        : string;
      rx_enable_lock_to_data_sig        : string;
      rx_enable_lock_to_refclk_sig        : string;
      rx_enable_self_test_mode        : string;
      rx_enable_true_complement_match_in_word_align        : string;
      rx_force_signal_detect        : string;
      rx_ppmselect        : natural;
      rx_rate_match_back_to_back        : string;
      rx_rate_match_fifo_mode        : string;
      rx_rate_match_ordered_set_based        : string;
      rx_rate_match_pattern1        : string;
      rx_rate_match_pattern2        : string;
      rx_rate_match_pattern_size        : natural;
      rx_rate_match_skip_set_based        : string;
      rx_run_length_enable        : string;
      rx_signal_detect_threshold        : natural;
      rx_use_align_state_machine        : string;
      rx_use_clkout        : string;
      rx_use_coreclk        : string;
      rx_use_deserializer_double_data_mode        : string;
      rx_use_deskew_fifo        : string;
      rx_use_double_data_mode        : string;
      rx_use_pipe8b10binvpolarity        : string;
      rx_use_rate_match_pattern1_only        : string;
      transmitter_termination        : string;
      tx_8b_10b_compatibility_mode        : string;
      tx_8b_10b_mode        : string;
      tx_allow_polarity_inversion        : string;
      tx_analog_power        : string;
      tx_channel_bonding        : string;
      tx_channel_width        : natural;
      tx_common_mode        : string;
      tx_data_rate        : natural;
      tx_data_rate_remainder        : natural;
      tx_enable_bit_reversal        : string;
      tx_enable_idle_selection        : string;
      tx_enable_self_test_mode        : string;
      tx_refclk_divide_by        : natural;
      tx_transmit_protocol        : string;
      tx_use_coreclk        : string;
      tx_use_double_data_mode        : string;
      tx_use_serializer_double_data_mode        : string;
      use_calibration_block        : string;
      vod_ctrl_setting        : natural
      );
    port (
      tx_forceelecidle    : in std_logic_vector (3 downto 0);
      rx_disperr    : out std_logic_vector (7 downto 0);
      rx_patterndetect    : out std_logic_vector (7 downto 0);
      rx_coreclk    : in std_logic_vector (3 downto 0);
      coreclkout    : out std_logic_vector (0 downto 0);
      rx_ctrldetect    : out std_logic_vector (7 downto 0);
      pll_inclk    : in std_logic ;
      gxb_powerdown    : in std_logic_vector (0 downto 0);
      pipedatavalid    : out std_logic_vector (3 downto 0);
      tx_datain    : in std_logic_vector (63 downto 0);
      rx_errdetect    : out std_logic_vector (7 downto 0);
      pipephydonestatus    : out std_logic_vector (3 downto 0);
      rx_pll_locked    : out std_logic_vector (3 downto 0);
      rx_freqlocked    : out std_logic_vector (3 downto 0);
      cal_blk_clk    : in std_logic ;
      powerdn    : in std_logic_vector (7 downto 0);
      tx_dataout    : out std_logic_vector (3 downto 0);
      fixedclk    : in std_logic ;
      rx_datain    : in std_logic_vector (3 downto 0);
      pipeelecidle    : out std_logic_vector (3 downto 0);
      tx_coreclk    : in std_logic_vector (3 downto 0);
      tx_ctrlenable    : in std_logic_vector (7 downto 0);
      pipestatus    : out std_logic_vector (11 downto 0);
      rx_analogreset    : in std_logic_vector (0 downto 0);
      pipe8b10binvpolarity    : in std_logic_vector (3 downto 0);
      rx_syncstatus    : out std_logic_vector (7 downto 0);
      rx_digitalreset    : in std_logic_vector (0 downto 0);
      tx_digitalreset    : in std_logic_vector (0 downto 0);
      tx_forcedispcompliance    : in std_logic_vector (3 downto 0);
      pll_locked    : out std_logic_vector (0 downto 0);
      rx_dataout    : out std_logic_vector (63 downto 0);
      tx_detectrxloop    : in std_logic_vector (3 downto 0)
      );
  end component;

begin
  rx_disperr    <= sub_wire0(7 downto 0);
  rx_patterndetect    <= sub_wire1(7 downto 0);
  coreclkout    <= sub_wire2(0 downto 0);
  rx_ctrldetect    <= sub_wire3(7 downto 0);
  pipedatavalid    <= sub_wire4(3 downto 0);
  rx_errdetect    <= sub_wire5(7 downto 0);
  pipephydonestatus    <= sub_wire6(3 downto 0);
  rx_pll_locked    <= sub_wire7(3 downto 0);
  rx_freqlocked    <= sub_wire8(3 downto 0);
  tx_dataout    <= sub_wire9(3 downto 0);
  pipeelecidle    <= sub_wire10(3 downto 0);
  pipestatus    <= sub_wire11(11 downto 0);
  rx_syncstatus    <= sub_wire12(7 downto 0);
  pll_locked    <= sub_wire13(0 downto 0);
  rx_dataout    <= sub_wire14(63 downto 0);

  alt2gxb_component : alt2gxb
    generic map (
      cmu_pll_inclock_period => 10000,
      cmu_pll_loop_filter_resistor_control => 3,
      digitalreset_port_width => 1,
      enable_fast_recovery_pci_mode => "false",
      enable_pll_inclk_alt_drive_rx_cru => "true",
      enable_pll_inclk_drive_rx_cru => "true",
      en_local_clk_div_ctrl => "true",
      equalizer_ctrl_a_setting => 0,
      equalizer_ctrl_b_setting => 0,
      equalizer_ctrl_c_setting => 0,
      equalizer_ctrl_d_setting => 0,
      equalizer_ctrl_v_setting => 0,
      equalizer_dcgain_setting => 1,
      intended_device_family => "arria gx",
      loopback_mode => "none",
      lpm_type => "alt2gxb",
      number_of_channels => 4,
      operation_mode => "duplex",
      preemphasis_ctrl_1stposttap_setting => 5,
      preemphasis_ctrl_2ndposttap_inv_setting => "false",
      preemphasis_ctrl_2ndposttap_setting => 0,
      preemphasis_ctrl_pretap_inv_setting => "false",
      preemphasis_ctrl_pretap_setting => 0,
      protocol => "pipe",
      receiver_termination => "oct_100_ohms",
      reconfig_dprio_mode => 0,
      reverse_loopback_mode => "rplb",
      rx_8b_10b_compatibility_mode => "true",
      rx_8b_10b_mode => "normal",
      rx_align_pattern => "0101111100",
      rx_align_pattern_length => 10,
      rx_allow_align_polarity_inversion => "false",
      rx_allow_pipe_polarity_inversion => "true",
      rx_bandwidth_mode => 1,
      rx_bitslip_enable => "false",
      rx_byte_ordering_mode => "none",
      rx_channel_bonding => "x4",
      rx_channel_width => 16,
      rx_common_mode => "0.9v",
      rx_cru_pre_divide_by => 1,
      rx_datapath_protocol => "pipe",
      rx_data_rate => 2500,
      rx_data_rate_remainder => 0,
      rx_disable_auto_idle_insertion => "false",
      rx_enable_bit_reversal => "false",
      rx_enable_lock_to_data_sig => "false",
      rx_enable_lock_to_refclk_sig => "false",
      rx_enable_self_test_mode => "false",
      rx_enable_true_complement_match_in_word_align => "false",
      rx_force_signal_detect => "false",
      rx_ppmselect => 8,
      rx_rate_match_back_to_back => "false",
      rx_rate_match_fifo_mode => "normal",
      rx_rate_match_ordered_set_based => "false",
      rx_rate_match_pattern1 => "11010000111010000011",
      rx_rate_match_pattern2 => "00101111000101111100",
      rx_rate_match_pattern_size => 20,
      rx_rate_match_skip_set_based => "false",
      rx_run_length_enable => "false",
      rx_signal_detect_threshold => 2,
      rx_use_align_state_machine => "true",
      rx_use_clkout => "false",
      rx_use_coreclk => "true",
      rx_use_deserializer_double_data_mode => "false",
      rx_use_deskew_fifo => "false",
      rx_use_double_data_mode => "true",
      rx_use_pipe8b10binvpolarity => "true",
      rx_use_rate_match_pattern1_only => "false",
      transmitter_termination => "oct_100_ohms",
      tx_8b_10b_compatibility_mode => "true",
      tx_8b_10b_mode => "normal",
      tx_allow_polarity_inversion => "false",
      tx_analog_power => "1.2v",
      tx_channel_bonding => "x4",
      tx_channel_width => 16,
      tx_common_mode => "0.6v",
      tx_data_rate => 2500,
      tx_data_rate_remainder => 0,
      tx_enable_bit_reversal => "false",
      tx_enable_idle_selection => "false",
      tx_enable_self_test_mode => "false",
      tx_refclk_divide_by => 1,
      tx_transmit_protocol => "pipe",
      tx_use_coreclk => "true",
      tx_use_double_data_mode => "true",
      tx_use_serializer_double_data_mode => "false",
      use_calibration_block => "true",
      vod_ctrl_setting => 4
      )
    port map (
      tx_forceelecidle => tx_forceelecidle,
      rx_coreclk => rx_coreclk,
      pll_inclk => pll_inclk,
      gxb_powerdown => gxb_powerdown,
      tx_datain => tx_datain,
      cal_blk_clk => cal_blk_clk,
      powerdn => powerdn,
      fixedclk => fixedclk,
      rx_datain => rx_datain,
      tx_coreclk => tx_coreclk,
      tx_ctrlenable => tx_ctrlenable,
      rx_analogreset => rx_analogreset,
      pipe8b10binvpolarity => pipe8b10binvpolarity,
      rx_digitalreset => rx_digitalreset,
      tx_digitalreset => tx_digitalreset,
      tx_forcedispcompliance => tx_forcedispcompliance,
      tx_detectrxloop => tx_detectrxloop,
      rx_disperr => sub_wire0,
      rx_patterndetect => sub_wire1,
      coreclkout => sub_wire2,
      rx_ctrldetect => sub_wire3,
      pipedatavalid => sub_wire4,
      rx_errdetect => sub_wire5,
      pipephydonestatus => sub_wire6,
      rx_pll_locked => sub_wire7,
      rx_freqlocked => sub_wire8,
      tx_dataout => sub_wire9,
      pipeelecidle => sub_wire10,
      pipestatus => sub_wire11,
      rx_syncstatus => sub_wire12,
      pll_locked => sub_wire13,
      rx_dataout => sub_wire14
      );
end syn;
