library ieee;
use ieee.std_logic_1164.all;

entity alt2gxb_agx_8x1 is
  port (
    cal_blk_clk             : in std_logic ;
    fixedclk                : in std_logic ;
    gxb_powerdown           : in std_logic_vector (0 downto 0);
    pipe8b10binvpolarity    : in std_logic_vector (7 downto 0);
    pll_inclk               : in std_logic ;
    powerdn                 : in std_logic_vector (15 downto 0);
    rx_analogreset          : in std_logic_vector (0 downto 0);
    rx_coreclk              : in std_logic_vector (7 downto 0);
    rx_datain               : in std_logic_vector (7 downto 0);
    rx_digitalreset         : in std_logic_vector (0 downto 0);
    tx_coreclk              : in std_logic_vector (7 downto 0);
    tx_ctrlenable           : in std_logic_vector (15 downto 0);
    tx_datain               : in std_logic_vector (127 downto 0);
    tx_detectrxloop         : in std_logic_vector (7 downto 0);
    tx_digitalreset         : in std_logic_vector (0 downto 0);
    tx_forcedispcompliance  : in std_logic_vector (7 downto 0);
    tx_forceelecidle        : in std_logic_vector (7 downto 0);
    coreclkout              : out std_logic_vector (1 downto 0);
    pipedatavalid           : out std_logic_vector (7 downto 0);
    pipeelecidle            : out std_logic_vector (7 downto 0);
    pipephydonestatus       : out std_logic_vector (7 downto 0);
    pipestatus              : out std_logic_vector (23 downto 0);
    pll_locked              : out std_logic_vector (1 downto 0);
    rx_ctrldetect           : out std_logic_vector (15 downto 0);
    rx_dataout              : out std_logic_vector (127 downto 0);
    rx_disperr              : out std_logic_vector (15 downto 0);
    rx_errdetect            : out std_logic_vector (15 downto 0);
    rx_freqlocked           : out std_logic_vector (7 downto 0);
    rx_patterndetect        : out std_logic_vector (15 downto 0);
    rx_pll_locked           : out std_logic_vector (7 downto 0);
    rx_syncstatus           : out std_logic_vector (15 downto 0);
    tx_dataout              : out std_logic_vector (7 downto 0)
  );
end alt2gxb_agx_8x1;


architecture syn of alt2gxb_agx_8x1 is

  component myalt2gxb_agx_x1
    port (
      cal_blk_clk            : in  std_logic;
      fixedclk               : in  std_logic;
      pll_inclk              : in  std_logic;
      powerdn                : in  std_logic_vector(1*2-1 downto 0);

      pll_locked             : out std_logic_vector((1-1)/4 downto 0);
      gxb_powerdown          : in  std_logic_vector(0 downto 0);
      tx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_analogreset         : in  std_logic_vector(0 downto 0);
      rx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_pll_locked          : out std_logic_vector(1-1 downto 0);
      rx_freqlocked          : out std_logic_vector(1-1 downto 0);

--      coreclkout             : out std_logic_vector((1-1)/4 downto 0);
      tx_clkout              : out std_logic_vector(0 downto 0);  -- x1 only
      rx_coreclk             : in  std_logic_vector(1-1 downto 0);
      tx_coreclk             : in  std_logic_vector(1-1 downto 0);
      tx_dataout             : out std_logic_vector(1-1 downto 0);
      tx_datain              : in  std_logic_vector(1*16-1 downto 0);
      tx_ctrlenable          : in  std_logic_vector(1*2-1 downto 0);
      tx_forceelecidle       : in  std_logic_vector(1-1 downto 0);
      tx_detectrxloop        : in  std_logic_vector(1-1 downto 0);
      tx_forcedispcompliance : in  std_logic_vector(1-1 downto 0);
      pipe8b10binvpolarity   : in  std_logic_vector(1-1 downto 0);

      rx_datain              : in  std_logic_vector(1-1 downto 0);
      rx_dataout             : out std_logic_vector(1*16-1 downto 0);
      rx_ctrldetect          : out std_logic_vector(1*2-1 downto 0);
      rx_patterndetect       : out std_logic_vector(1*2-1 downto 0);
      rx_disperr             : out std_logic_vector(1*2-1 downto 0);
      rx_errdetect           : out std_logic_vector(1*2-1 downto 0);
      rx_syncstatus          : out std_logic_vector(1*2-1 downto 0);

      pipedatavalid          : out std_logic_vector(1-1 downto 0);
      pipeelecidle           : out std_logic_vector(1-1 downto 0);
      pipephydonestatus      : out std_logic_vector(1-1 downto 0);
      pipestatus             : out std_logic_vector(1*3-1 downto 0)
    );
  end component;

  signal pll_locked0         : std_logic_vector(7 downto 0);
  signal coreclkout0         : std_logic_vector(7 downto 0);

begin
  --
  -- alt2gxb (x1 PIPE, individual mode) generated with MegaWizard.
  --

  x1gen : for i in 0 to 7 generate
    x1 : myalt2gxb_agx_x1
      port map (
        cal_blk_clk            => cal_blk_clk,
        fixedclk               => fixedclk,
        pll_inclk              => pll_inclk,
        powerdn                => powerdn(2*(i+1)-1 downto 2*i),

        pll_locked             => pll_locked0(i downto i),
        gxb_powerdown          => gxb_powerdown,
        tx_digitalreset        => tx_digitalreset,
        rx_analogreset         => rx_analogreset,
        rx_digitalreset        => rx_digitalreset,
        rx_pll_locked          => rx_pll_locked(i downto i),
        rx_freqlocked          => rx_freqlocked(i downto i),

        -- in individual (non-bonded) mode, one clk per channel
        -- is provided by each local clock divider block.
        tx_clkout              => coreclkout0(i downto i),
        rx_coreclk             => rx_coreclk(i downto i),
        tx_coreclk             => tx_coreclk(i downto i),
        tx_dataout             => tx_dataout(i downto i),
        tx_datain              => tx_datain(16*(i+1)-1 downto 16*i),
        tx_ctrlenable          => tx_ctrlenable(2*(i+1)-1 downto 2*i),
        tx_forceelecidle       => tx_forceelecidle(i downto i),
        tx_detectrxloop        => tx_detectrxloop(i downto i),
        tx_forcedispcompliance => tx_forcedispcompliance(i downto i),
        
        rx_datain              => rx_datain(i downto i),
        rx_dataout             => rx_dataout(16*(i+1)-1 downto 16*i),
        rx_ctrldetect          => rx_ctrldetect(2*(i+1)-1 downto 2*i),
        rx_patterndetect       => rx_patterndetect(2*(i+1)-1 downto 2*i),
        rx_disperr             => rx_disperr(2*(i+1)-1 downto 2*i),
        rx_errdetect           => rx_errdetect(2*(i+1)-1 downto 2*i),
        rx_syncstatus          => rx_syncstatus(2*(i+1)-1 downto 2*i),

        pipe8b10binvpolarity   => pipe8b10binvpolarity(i downto i),
        pipedatavalid          => pipedatavalid(i downto i),
        pipeelecidle           => pipeelecidle(i downto i),
        pipephydonestatus      => pipephydonestatus(i downto i),
        pipestatus             => pipestatus(3*(i+1)-1 downto 3*i)
      );
  end generate x1gen;

  -- assign coreclkout for each GXB block.
  coreclkout(0) <= coreclkout0(0);
  coreclkout(1) <= coreclkout0(4);

  -- check if lanes in a single GXB blocks locked to their own PLLs.
  process (pll_locked0)
    variable alllockedtmp : std_logic;
  begin
    for quad in 0 to 1 loop
      alllockedtmp := '1';
      for i in 0 to 3 loop
        if (pll_locked0(quad * 4 + i) = '0') then
          alllockedtmp := '0';
        end if;
      end loop;
      pll_locked(quad) <= alllockedtmp;
    end loop;
  end process;

  
end syn;


LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY myalt2gxb_agx_x1 IS
	PORT
	(
		cal_blk_clk		: IN STD_LOGIC ;
		fixedclk		: IN STD_LOGIC ;
		gxb_powerdown		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		pipe8b10binvpolarity		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		pll_inclk		: IN STD_LOGIC ;
		powerdn		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_analogreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_coreclk		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_datain		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_coreclk		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_ctrlenable		: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
		tx_datain		: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
		tx_detectrxloop		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_digitalreset		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_forcedispcompliance		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_forceelecidle		: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
		pipedatavalid		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		pipeelecidle		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		pipephydonestatus		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		pipestatus		: OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
		pll_locked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_ctrldetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_dataout		: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
		rx_disperr		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_errdetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_freqlocked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_patterndetect		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		rx_pll_locked		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		rx_syncstatus		: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
		coreclkout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
		tx_dataout		: OUT STD_LOGIC_VECTOR (0 DOWNTO 0)
	);
END myalt2gxb_agx_x1;


ARCHITECTURE SYN OF myalt2gxb_agx_x1 IS

	SIGNAL sub_wire0	: STD_LOGIC_VECTOR (1 DOWNTO 0);
	SIGNAL sub_wire1	: STD_LOGIC_VECTOR (1 DOWNTO 0);
	SIGNAL sub_wire2	: STD_LOGIC_VECTOR (1 DOWNTO 0);
	SIGNAL sub_wire3	: STD_LOGIC_VECTOR (0 DOWNTO 0);
	SIGNAL sub_wire4	: STD_LOGIC_VECTOR (1 DOWNTO 0);
	SIGNAL sub_wire5	: STD_LOGIC_VECTOR (0 DOWNTO 0);
	SIGNAL sub_wire6	: STD_LOGIC_VECTOR (0 DOWNTO 0);
	SIGNAL sub_wire7	: STD_LOGIC_VECTOR (0 DOWNTO 0);
	SIGNAL sub_wire8	: STD_LOGIC_VECTOR (0 DOWNTO 0);
	SIGNAL sub_wire9	: STD_LOGIC_VECTOR (0 DOWNTO 0);
	SIGNAL sub_wire10	: STD_LOGIC_VECTOR (2 DOWNTO 0);
	SIGNAL sub_wire11	: STD_LOGIC_VECTOR (1 DOWNTO 0);
	SIGNAL sub_wire12	: STD_LOGIC_VECTOR (0 DOWNTO 0);
	SIGNAL sub_wire13	: STD_LOGIC_VECTOR (0 DOWNTO 0);
	SIGNAL sub_wire14	: STD_LOGIC_VECTOR (15 DOWNTO 0);



	COMPONENT alt2gxb
	GENERIC (
		cmu_pll_inclock_period		: NATURAL;
		cmu_pll_loop_filter_resistor_control		: NATURAL;
		digitalreset_port_width		: NATURAL;
		enable_fast_recovery_pci_mode		: STRING;
		enable_pll_inclk_alt_drive_rx_cru		: STRING;
		enable_pll_inclk_drive_rx_cru		: STRING;
		en_local_clk_div_ctrl		: STRING;
		equalizer_ctrl_a_setting		: NATURAL;
		equalizer_ctrl_b_setting		: NATURAL;
		equalizer_ctrl_c_setting		: NATURAL;
		equalizer_ctrl_d_setting		: NATURAL;
		equalizer_ctrl_v_setting		: NATURAL;
		equalizer_dcgain_setting		: NATURAL;
		intended_device_family		: STRING;
		loopback_mode		: STRING;
		lpm_type		: STRING;
		number_of_channels		: NATURAL;
		operation_mode		: STRING;
		preemphasis_ctrl_1stposttap_setting		: NATURAL;
		preemphasis_ctrl_2ndposttap_inv_setting		: STRING;
		preemphasis_ctrl_2ndposttap_setting		: NATURAL;
		preemphasis_ctrl_pretap_inv_setting		: STRING;
		preemphasis_ctrl_pretap_setting		: NATURAL;
		protocol		: STRING;
		receiver_termination		: STRING;
		reconfig_dprio_mode		: NATURAL;
		reverse_loopback_mode		: STRING;
		rx_8b_10b_compatibility_mode		: STRING;
		rx_8b_10b_mode		: STRING;
		rx_align_pattern		: STRING;
		rx_align_pattern_length		: NATURAL;
		rx_allow_align_polarity_inversion		: STRING;
		rx_allow_pipe_polarity_inversion		: STRING;
		rx_bandwidth_mode		: NATURAL;
		rx_bitslip_enable		: STRING;
		rx_byte_ordering_mode		: STRING;
		rx_channel_bonding		: STRING;
		rx_channel_width		: NATURAL;
		rx_common_mode		: STRING;
		rx_cru_pre_divide_by		: NATURAL;
		rx_datapath_protocol		: STRING;
		rx_data_rate		: NATURAL;
		rx_data_rate_remainder		: NATURAL;
		rx_disable_auto_idle_insertion		: STRING;
		rx_enable_bit_reversal		: STRING;
		rx_enable_lock_to_data_sig		: STRING;
		rx_enable_lock_to_refclk_sig		: STRING;
		rx_enable_self_test_mode		: STRING;
		rx_enable_true_complement_match_in_word_align		: STRING;
		rx_force_signal_detect		: STRING;
		rx_ppmselect		: NATURAL;
		rx_rate_match_back_to_back		: STRING;
		rx_rate_match_fifo_mode		: STRING;
		rx_rate_match_ordered_set_based		: STRING;
		rx_rate_match_pattern1		: STRING;
		rx_rate_match_pattern2		: STRING;
		rx_rate_match_pattern_size		: NATURAL;
		rx_rate_match_skip_set_based		: STRING;
		rx_run_length_enable		: STRING;
		rx_signal_detect_threshold		: NATURAL;
		rx_use_align_state_machine		: STRING;
		rx_use_clkout		: STRING;
		rx_use_coreclk		: STRING;
		rx_use_deserializer_double_data_mode		: STRING;
		rx_use_deskew_fifo		: STRING;
		rx_use_double_data_mode		: STRING;
		rx_use_pipe8b10binvpolarity		: STRING;
		rx_use_rate_match_pattern1_only		: STRING;
		transmitter_termination		: STRING;
		tx_8b_10b_compatibility_mode		: STRING;
		tx_8b_10b_mode		: STRING;
		tx_allow_polarity_inversion		: STRING;
		tx_analog_power		: STRING;
		tx_channel_bonding		: STRING;
		tx_channel_width		: NATURAL;
		tx_common_mode		: STRING;
		tx_data_rate		: NATURAL;
		tx_data_rate_remainder		: NATURAL;
		tx_enable_bit_reversal		: STRING;
		tx_enable_idle_selection		: STRING;
		tx_enable_self_test_mode		: STRING;
		tx_refclk_divide_by		: NATURAL;
		tx_transmit_protocol		: STRING;
		tx_use_coreclk		: STRING;
		tx_use_double_data_mode		: STRING;
		tx_use_serializer_double_data_mode		: STRING;
		use_calibration_block		: STRING;
		vod_ctrl_setting		: NATURAL
	);
	PORT (
			tx_forceelecidle	: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
			rx_disperr	: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
			rx_patterndetect	: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
			rx_coreclk	: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
			rx_ctrldetect	: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
			pll_inclk	: IN STD_LOGIC ;
			gxb_powerdown	: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
			pipedatavalid	: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
			tx_datain	: IN STD_LOGIC_VECTOR (15 DOWNTO 0);
			rx_errdetect	: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
			pipephydonestatus	: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
			rx_pll_locked	: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
			rx_freqlocked	: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
			cal_blk_clk	: IN STD_LOGIC ;
			powerdn	: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
			tx_dataout	: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
			fixedclk	: IN STD_LOGIC ;
			rx_datain	: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
			pipeelecidle	: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
			tx_coreclk	: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
			tx_ctrlenable	: IN STD_LOGIC_VECTOR (1 DOWNTO 0);
			pipestatus	: OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
			rx_analogreset	: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
			pipe8b10binvpolarity	: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
			rx_syncstatus	: OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
			rx_digitalreset	: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
			tx_digitalreset	: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
			tx_clkout	: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
			tx_forcedispcompliance	: IN STD_LOGIC_VECTOR (0 DOWNTO 0);
			pll_locked	: OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
			rx_dataout	: OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
			tx_detectrxloop	: IN STD_LOGIC_VECTOR (0 DOWNTO 0)
	);
	END COMPONENT;

BEGIN
	rx_disperr    <= sub_wire0(1 DOWNTO 0);
	rx_patterndetect    <= sub_wire1(1 DOWNTO 0);
	rx_ctrldetect    <= sub_wire2(1 DOWNTO 0);
	pipedatavalid    <= sub_wire3(0 DOWNTO 0);
	rx_errdetect    <= sub_wire4(1 DOWNTO 0);
	pipephydonestatus    <= sub_wire5(0 DOWNTO 0);
	rx_pll_locked    <= sub_wire6(0 DOWNTO 0);
	rx_freqlocked    <= sub_wire7(0 DOWNTO 0);
	tx_dataout    <= sub_wire8(0 DOWNTO 0);
	pipeelecidle    <= sub_wire9(0 DOWNTO 0);
	pipestatus    <= sub_wire10(2 DOWNTO 0);
	rx_syncstatus    <= sub_wire11(1 DOWNTO 0);
	tx_clkout     <= sub_wire12(0 DOWNTO 0);
	pll_locked    <= sub_wire13(0 DOWNTO 0);
	rx_dataout    <= sub_wire14(15 DOWNTO 0);

	alt2gxb_component : alt2gxb
	GENERIC MAP (
		cmu_pll_inclock_period => 10000,
		cmu_pll_loop_filter_resistor_control => 3,
		digitalreset_port_width => 1,
		enable_fast_recovery_pci_mode => "false",
		enable_pll_inclk_alt_drive_rx_cru => "true",
		enable_pll_inclk_drive_rx_cru => "true",
		en_local_clk_div_ctrl => "true",
		equalizer_ctrl_a_setting => 0,
		equalizer_ctrl_b_setting => 0,
		equalizer_ctrl_c_setting => 0,
		equalizer_ctrl_d_setting => 0,
		equalizer_ctrl_v_setting => 0,
		equalizer_dcgain_setting => 1,
		intended_device_family => "Arria GX",
		loopback_mode => "none",
		lpm_type => "alt2gxb",
		number_of_channels => 1,
		operation_mode => "duplex",
		preemphasis_ctrl_1stposttap_setting => 5,
		preemphasis_ctrl_2ndposttap_inv_setting => "false",
		preemphasis_ctrl_2ndposttap_setting => 0,
		preemphasis_ctrl_pretap_inv_setting => "false",
		preemphasis_ctrl_pretap_setting => 0,
		protocol => "pipe",
		receiver_termination => "oct_100_ohms",
		reconfig_dprio_mode => 0,
		reverse_loopback_mode => "rplb",
		rx_8b_10b_compatibility_mode => "true",
		rx_8b_10b_mode => "normal",
		rx_align_pattern => "0101111100",
		rx_align_pattern_length => 10,
		rx_allow_align_polarity_inversion => "false",
		rx_allow_pipe_polarity_inversion => "true",
		rx_bandwidth_mode => 1,
		rx_bitslip_enable => "false",
		rx_byte_ordering_mode => "none",
		rx_channel_bonding => "indv",
		rx_channel_width => 16,
		rx_common_mode => "0.9v",
		rx_cru_pre_divide_by => 1,
		rx_datapath_protocol => "pipe",
		rx_data_rate => 2500,
		rx_data_rate_remainder => 0,
		rx_disable_auto_idle_insertion => "false",
		rx_enable_bit_reversal => "false",
		rx_enable_lock_to_data_sig => "false",
		rx_enable_lock_to_refclk_sig => "false",
		rx_enable_self_test_mode => "false",
		rx_enable_true_complement_match_in_word_align => "false",
		rx_force_signal_detect => "false",
		rx_ppmselect => 8,
		rx_rate_match_back_to_back => "false",
		rx_rate_match_fifo_mode => "normal",
		rx_rate_match_ordered_set_based => "false",
		rx_rate_match_pattern1 => "11010000111010000011",
		rx_rate_match_pattern2 => "00101111000101111100",
		rx_rate_match_pattern_size => 20,
		rx_rate_match_skip_set_based => "false",
		rx_run_length_enable => "false",
		rx_signal_detect_threshold => 2,
		rx_use_align_state_machine => "true",
		rx_use_clkout => "false",
		rx_use_coreclk => "true",
		rx_use_deserializer_double_data_mode => "false",
		rx_use_deskew_fifo => "false",
		rx_use_double_data_mode => "true",
		rx_use_pipe8b10binvpolarity => "true",
		rx_use_rate_match_pattern1_only => "false",
		transmitter_termination => "oct_100_ohms",
		tx_8b_10b_compatibility_mode => "true",
		tx_8b_10b_mode => "normal",
		tx_allow_polarity_inversion => "false",
		tx_analog_power => "1.2v",
		tx_channel_bonding => "indv",
		tx_channel_width => 16,
		tx_common_mode => "0.6v",
		tx_data_rate => 2500,
		tx_data_rate_remainder => 0,
		tx_enable_bit_reversal => "false",
		tx_enable_idle_selection => "false",
		tx_enable_self_test_mode => "false",
		tx_refclk_divide_by => 1,
		tx_transmit_protocol => "pipe",
		tx_use_coreclk => "true",
		tx_use_double_data_mode => "true",
		tx_use_serializer_double_data_mode => "false",
		use_calibration_block => "true",
		vod_ctrl_setting => 4
	)
	PORT MAP (
		tx_forceelecidle => tx_forceelecidle,
		rx_coreclk => rx_coreclk,
		pll_inclk => pll_inclk,
		gxb_powerdown => gxb_powerdown,
		tx_datain => tx_datain,
		cal_blk_clk => cal_blk_clk,
		powerdn => powerdn,
		fixedclk => fixedclk,
		rx_datain => rx_datain,
		tx_coreclk => tx_coreclk,
		tx_ctrlenable => tx_ctrlenable,
		rx_analogreset => rx_analogreset,
		pipe8b10binvpolarity => pipe8b10binvpolarity,
		rx_digitalreset => rx_digitalreset,
		tx_digitalreset => tx_digitalreset,
		tx_forcedispcompliance => tx_forcedispcompliance,
		tx_detectrxloop => tx_detectrxloop,
		rx_disperr => sub_wire0,
		rx_patterndetect => sub_wire1,
		rx_ctrldetect => sub_wire2,
		pipedatavalid => sub_wire3,
		rx_errdetect => sub_wire4,
		pipephydonestatus => sub_wire5,
		rx_pll_locked => sub_wire6,
		rx_freqlocked => sub_wire7,
		tx_dataout => sub_wire8,
		pipeelecidle => sub_wire9,
		pipestatus => sub_wire10,
		rx_syncstatus => sub_wire11,
		tx_clkout => sub_wire12,
		pll_locked => sub_wire13,
		rx_dataout => sub_wire14
	);



END SYN;
