library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity phy125 is
  generic (
    DEVICE       : string := "Arria GX";
    USE_CLK32    : integer := 1;        -- use clock input pin 'clk32'.
    NLANE        : integer;
    SIMULATION   : integer
  );
  port (
    cal_blk_clk             : in    std_logic;  -- used for GXB calibration.
    clk125plllock           : out   std_logic;
    clk32                   : in    std_logic;  -- used only for powerup timing control.
    clk100                  : in    std_logic;  -- dedicated to GXB refclk.
    clk125out               : out   std_logic;
    rstn                    : in    std_logic;

    rx_in                   : in    std_logic_vector(NLANE-1 downto 0);
    tx_out                  : out   std_logic_vector(NLANE-1 downto 0);

    wake                    : out   std_logic;
    phystatus               : out   std_logic;
    powerdown               : in    std_logic_vector(1 downto 0);
    txdetectrx              : in    std_logic;

    txdata                  : in    std_logic_vector(NLANE*16-1 downto 0);
    txdatak                 : in    std_logic_vector(NLANE*2-1 downto 0);
    txelecidle              : in    std_logic_vector(NLANE-1 downto 0);
    txcompl                 : in    std_logic_vector(NLANE-1 downto 0);
    rxpolarity              : in    std_logic_vector(NLANE-1 downto 0);
    rxdata                  : out   std_logic_vector(NLANE*16-1 downto 0);
    rxdatak                 : out   std_logic_vector(NLANE*2-1 downto 0);
    rxvalid                 : out   std_logic_vector(NLANE-1 downto 0);
    rxelecidle              : out   std_logic_vector(NLANE-1 downto 0);
    rxstatus                : out   std_logic_vector(NLANE*3-1 downto 0)
  );
end phy125;

architecture default of phy125 is

  component alt2gxb_agx_x1
    port (
      cal_blk_clk            : in  std_logic;
      fixedclk               : in  std_logic;
      pll_inclk              : in  std_logic;
      powerdn                : in  std_logic_vector(NLANE*2-1 downto 0);

      pll_locked             : out std_logic_vector((NLANE-1)/4 downto 0);
      gxb_powerdown          : in  std_logic_vector(0 downto 0);
      tx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_analogreset         : in  std_logic_vector(0 downto 0);
      rx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_pll_locked          : out std_logic_vector(NLANE-1 downto 0);
      rx_freqlocked          : out std_logic_vector(NLANE-1 downto 0);

      coreclkout             : out std_logic_vector((NLANE-1)/4 downto 0);
      tx_coreclk             : in  std_logic_vector(NLANE-1 downto 0);
      rx_coreclk             : in  std_logic_vector(NLANE-1 downto 0);
      tx_dataout             : out std_logic_vector(NLANE-1 downto 0);
      tx_datain              : in  std_logic_vector(NLANE*16-1 downto 0);
      tx_ctrlenable          : in  std_logic_vector(NLANE*2-1 downto 0);
      tx_forceelecidle       : in  std_logic_vector(NLANE-1 downto 0);
      tx_detectrxloop        : in  std_logic_vector(NLANE-1 downto 0);
      tx_forcedispcompliance : in  std_logic_vector(NLANE-1 downto 0);
      pipe8b10binvpolarity   : in  std_logic_vector(NLANE-1 downto 0);

      rx_datain              : in  std_logic_vector(NLANE-1 downto 0);
      rx_dataout             : out std_logic_vector(NLANE*16-1 downto 0);
      rx_ctrldetect          : out std_logic_vector(NLANE*2-1 downto 0);
      rx_patterndetect       : out std_logic_vector(NLANE*2-1 downto 0);
      rx_disperr             : out std_logic_vector(NLANE*2-1 downto 0);
      rx_errdetect           : out std_logic_vector(NLANE*2-1 downto 0);
      rx_syncstatus          : out std_logic_vector(NLANE*2-1 downto 0);

      pipedatavalid          : out std_logic_vector(NLANE-1 downto 0);
      pipeelecidle           : out std_logic_vector(NLANE-1 downto 0);
      pipephydonestatus      : out std_logic_vector(NLANE-1 downto 0);
      pipestatus             : out std_logic_vector(NLANE*3-1 downto 0)
    );
  end component;

  component alt2gxb_agx_x4
    port (
      cal_blk_clk            : in  std_logic;
      fixedclk               : in  std_logic;
      pll_inclk              : in  std_logic;
      powerdn                : in  std_logic_vector(NLANE*2-1 downto 0);

      pll_locked             : out std_logic_vector((NLANE-1)/4 downto 0);
      gxb_powerdown          : in  std_logic_vector(0 downto 0);
      tx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_analogreset         : in  std_logic_vector(0 downto 0);
      rx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_pll_locked          : out std_logic_vector(NLANE-1 downto 0);
      rx_freqlocked          : out std_logic_vector(NLANE-1 downto 0);

      coreclkout             : out std_logic_vector((NLANE-1)/4 downto 0);
      tx_coreclk             : in  std_logic_vector(NLANE-1 downto 0);
      rx_coreclk             : in  std_logic_vector(NLANE-1 downto 0);
      tx_dataout             : out std_logic_vector(NLANE-1 downto 0);
      tx_datain              : in  std_logic_vector(NLANE*16-1 downto 0);
      tx_ctrlenable          : in  std_logic_vector(NLANE*2-1 downto 0);
      tx_forceelecidle       : in  std_logic_vector(NLANE-1 downto 0);
      tx_detectrxloop        : in  std_logic_vector(NLANE-1 downto 0);
      tx_forcedispcompliance : in  std_logic_vector(NLANE-1 downto 0);
      pipe8b10binvpolarity   : in  std_logic_vector(NLANE-1 downto 0);

      rx_datain              : in  std_logic_vector(NLANE-1 downto 0);
      rx_dataout             : out std_logic_vector(NLANE*16-1 downto 0);
      rx_ctrldetect          : out std_logic_vector(NLANE*2-1 downto 0);
      rx_patterndetect       : out std_logic_vector(NLANE*2-1 downto 0);
      rx_disperr             : out std_logic_vector(NLANE*2-1 downto 0);
      rx_errdetect           : out std_logic_vector(NLANE*2-1 downto 0);
      rx_syncstatus          : out std_logic_vector(NLANE*2-1 downto 0);

      pipedatavalid          : out std_logic_vector(NLANE-1 downto 0);
      pipeelecidle           : out std_logic_vector(NLANE-1 downto 0);
      pipephydonestatus      : out std_logic_vector(NLANE-1 downto 0);
      pipestatus             : out std_logic_vector(NLANE*3-1 downto 0)
    );
  end component;

  component alt2gxb_agx_2x4
    port (
      cal_blk_clk            : in  std_logic;
      fixedclk               : in  std_logic;
      pll_inclk              : in  std_logic;
      powerdn                : in  std_logic_vector(NLANE*2-1 downto 0);

      pll_locked             : out std_logic_vector((NLANE-1)/4 downto 0);
      gxb_powerdown          : in  std_logic_vector(0 downto 0);
      tx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_analogreset         : in  std_logic_vector(0 downto 0);
      rx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_pll_locked          : out std_logic_vector(NLANE-1 downto 0);
      rx_freqlocked          : out std_logic_vector(NLANE-1 downto 0);

      coreclkout             : out std_logic_vector((NLANE-1)/4 downto 0);
      tx_coreclk             : in  std_logic_vector(NLANE-1 downto 0);
      rx_coreclk             : in  std_logic_vector(NLANE-1 downto 0);
      tx_dataout             : out std_logic_vector(NLANE-1 downto 0);
      tx_datain              : in  std_logic_vector(NLANE*16-1 downto 0);
      tx_ctrlenable          : in  std_logic_vector(NLANE*2-1 downto 0);
      tx_forceelecidle       : in  std_logic_vector(NLANE-1 downto 0);
      tx_detectrxloop        : in  std_logic_vector(NLANE-1 downto 0);
      tx_forcedispcompliance : in  std_logic_vector(NLANE-1 downto 0);
      pipe8b10binvpolarity   : in  std_logic_vector(NLANE-1 downto 0);

      rx_datain              : in  std_logic_vector(NLANE-1 downto 0);
      rx_dataout             : out std_logic_vector(NLANE*16-1 downto 0);
      rx_ctrldetect          : out std_logic_vector(NLANE*2-1 downto 0);
      rx_patterndetect       : out std_logic_vector(NLANE*2-1 downto 0);
      rx_disperr             : out std_logic_vector(NLANE*2-1 downto 0);
      rx_errdetect           : out std_logic_vector(NLANE*2-1 downto 0);
      rx_syncstatus          : out std_logic_vector(NLANE*2-1 downto 0);

      pipedatavalid          : out std_logic_vector(NLANE-1 downto 0);
      pipeelecidle           : out std_logic_vector(NLANE-1 downto 0);
      pipephydonestatus      : out std_logic_vector(NLANE-1 downto 0);
      pipestatus             : out std_logic_vector(NLANE*3-1 downto 0)
    );
  end component;

  component alt2gxb_s2gx_x1
    port (
      cal_blk_clk            : in  std_logic;
      fixedclk               : in  std_logic;
      pll_inclk              : in  std_logic;
      powerdn                : in  std_logic_vector(NLANE*2-1 downto 0);

      pll_locked             : out std_logic_vector((NLANE-1)/4 downto 0);
      gxb_powerdown          : in  std_logic_vector(0 downto 0);
      tx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_analogreset         : in  std_logic_vector(0 downto 0);
      rx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_pll_locked          : out std_logic_vector(NLANE-1 downto 0);
      rx_freqlocked          : out std_logic_vector(NLANE-1 downto 0);

      coreclkout             : out std_logic_vector((NLANE-1)/4 downto 0);
      tx_coreclk             : in  std_logic_vector(NLANE-1 downto 0);
      rx_coreclk             : in  std_logic_vector(NLANE-1 downto 0);
      tx_dataout             : out std_logic_vector(NLANE-1 downto 0);
      tx_datain              : in  std_logic_vector(NLANE*16-1 downto 0);
      tx_ctrlenable          : in  std_logic_vector(NLANE*2-1 downto 0);
      tx_forceelecidle       : in  std_logic_vector(NLANE-1 downto 0);
      tx_detectrxloop        : in  std_logic_vector(NLANE-1 downto 0);
      tx_forcedispcompliance : in  std_logic_vector(NLANE-1 downto 0);
      pipe8b10binvpolarity   : in  std_logic_vector(NLANE-1 downto 0);

      rx_datain              : in  std_logic_vector(NLANE-1 downto 0);
      rx_dataout             : out std_logic_vector(NLANE*16-1 downto 0);
      rx_ctrldetect          : out std_logic_vector(NLANE*2-1 downto 0);
      rx_patterndetect       : out std_logic_vector(NLANE*2-1 downto 0);
      rx_disperr             : out std_logic_vector(NLANE*2-1 downto 0);
      rx_errdetect           : out std_logic_vector(NLANE*2-1 downto 0);
      rx_syncstatus          : out std_logic_vector(NLANE*2-1 downto 0);

      pipedatavalid          : out std_logic_vector(NLANE-1 downto 0);
      pipeelecidle           : out std_logic_vector(NLANE-1 downto 0);
      pipephydonestatus      : out std_logic_vector(NLANE-1 downto 0);
      pipestatus             : out std_logic_vector(NLANE*3-1 downto 0)
    );
  end component;

  component alt2gxb_s2gx_x4
    port (
      cal_blk_clk            : in  std_logic;
      fixedclk               : in  std_logic;
      pll_inclk              : in  std_logic;
      powerdn                : in  std_logic_vector(NLANE*2-1 downto 0);

      pll_locked             : out std_logic_vector((NLANE-1)/4 downto 0);
      gxb_powerdown          : in  std_logic_vector(0 downto 0);
      tx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_analogreset         : in  std_logic_vector(0 downto 0);
      rx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_pll_locked          : out std_logic_vector(NLANE-1 downto 0);
      rx_freqlocked          : out std_logic_vector(NLANE-1 downto 0);

      coreclkout             : out std_logic_vector((NLANE-1)/4 downto 0);
      tx_coreclk             : in  std_logic_vector(NLANE-1 downto 0);
      rx_coreclk             : in  std_logic_vector(NLANE-1 downto 0);
      tx_dataout             : out std_logic_vector(NLANE-1 downto 0);
      tx_datain              : in  std_logic_vector(NLANE*16-1 downto 0);
      tx_ctrlenable          : in  std_logic_vector(NLANE*2-1 downto 0);
      tx_forceelecidle       : in  std_logic_vector(NLANE-1 downto 0);
      tx_detectrxloop        : in  std_logic_vector(NLANE-1 downto 0);
      tx_forcedispcompliance : in  std_logic_vector(NLANE-1 downto 0);
      pipe8b10binvpolarity   : in  std_logic_vector(NLANE-1 downto 0);

      rx_datain              : in  std_logic_vector(NLANE-1 downto 0);
      rx_dataout             : out std_logic_vector(NLANE*16-1 downto 0);
      rx_ctrldetect          : out std_logic_vector(NLANE*2-1 downto 0);
      rx_patterndetect       : out std_logic_vector(NLANE*2-1 downto 0);
      rx_disperr             : out std_logic_vector(NLANE*2-1 downto 0);
      rx_errdetect           : out std_logic_vector(NLANE*2-1 downto 0);
      rx_syncstatus          : out std_logic_vector(NLANE*2-1 downto 0);

      pipedatavalid          : out std_logic_vector(NLANE-1 downto 0);
      pipeelecidle           : out std_logic_vector(NLANE-1 downto 0);
      pipephydonestatus      : out std_logic_vector(NLANE-1 downto 0);
      pipestatus             : out std_logic_vector(NLANE*3-1 downto 0)
    );
  end component;

  component alt2gxb_s2gx_x8
    port (
      cal_blk_clk            : in  std_logic;
      fixedclk               : in  std_logic;
      pll_inclk              : in  std_logic;
      powerdn                : in  std_logic_vector(NLANE*2-1 downto 0);

      pll_locked             : out std_logic_vector((NLANE-1)/4 downto 0);
      gxb_powerdown          : in  std_logic_vector(0 downto 0);
      tx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_analogreset         : in  std_logic_vector(0 downto 0);
      rx_digitalreset        : in  std_logic_vector(0 downto 0);
      rx_pll_locked          : out std_logic_vector(NLANE-1 downto 0);
      rx_freqlocked          : out std_logic_vector(NLANE-1 downto 0);

      coreclkout             : out std_logic_vector((NLANE-1)/4 downto 0);
      tx_coreclk             : in  std_logic_vector(NLANE-1 downto 0);
      rx_coreclk             : in  std_logic_vector(NLANE-1 downto 0);
      tx_dataout             : out std_logic_vector(NLANE-1 downto 0);
      tx_datain              : in  std_logic_vector(NLANE*16-1 downto 0);
      tx_ctrlenable          : in  std_logic_vector(NLANE*2-1 downto 0);
      tx_forceelecidle       : in  std_logic_vector(NLANE-1 downto 0);
      tx_detectrxloop        : in  std_logic_vector(NLANE-1 downto 0);
      tx_forcedispcompliance : in  std_logic_vector(NLANE-1 downto 0);
      pipe8b10binvpolarity   : in  std_logic_vector(NLANE-1 downto 0);

      rx_datain              : in  std_logic_vector(NLANE-1 downto 0);
      rx_dataout             : out std_logic_vector(NLANE*16-1 downto 0);
      rx_ctrldetect          : out std_logic_vector(NLANE*2-1 downto 0);
      rx_patterndetect       : out std_logic_vector(NLANE*2-1 downto 0);
      rx_disperr             : out std_logic_vector(NLANE*2-1 downto 0);
      rx_errdetect           : out std_logic_vector(NLANE*2-1 downto 0);
      rx_syncstatus          : out std_logic_vector(NLANE*2-1 downto 0);

      pipedatavalid          : out std_logic_vector(NLANE-1 downto 0);
      pipeelecidle           : out std_logic_vector(NLANE-1 downto 0);
      pipephydonestatus      : out std_logic_vector(NLANE-1 downto 0);
      pipestatus             : out std_logic_vector(NLANE*3-1 downto 0)
    );
  end component;

  component resetctrl
    generic (
      USE_CLK32    : integer := USE_CLK32;
      NLANE        : integer := NLANE;
      SIMULATION   : integer := SIMULATION
    );
    port (
      rstn         : in  std_logic;
      pwrup        : in  std_logic;
      clk32        : in  std_logic;
      clk125       : in  std_logic;
      txplllocked  : in  std_logic;
      rxfreqlocked : in  std_logic;
      rxplllocked  : in  std_logic;
      pipestatus   : in  std_logic_vector(NLANE*3-1 downto 0);
      txdrst       : out std_logic;
      rxarst       : out std_logic;
      rxdrst       : out std_logic
    );
  end component;

  component global
    port (
      a_in  : in  std_logic;
      a_out : out std_logic
    );
  end component;

  --
  -- connection between PIPE and alt2gxb.
  --
  signal clk125                 : std_logic;
  signal powerdn                : std_logic_vector(NLANE*2-1 downto 0);
  signal pipestatus             : std_logic_vector(NLANE*3-1 downto 0);
  signal tx_detectrxloop        : std_logic_vector(NLANE-1 downto 0);
  signal pll_locked             : std_logic_vector((NLANE-1)/4 downto 0);
  signal coreclkout             : std_logic_vector((NLANE-1)/4 downto 0);
  signal rx_coreclk             : std_logic_vector(NLANE-1 downto 0);
  signal tx_coreclk             : std_logic_vector(NLANE-1 downto 0);
  signal gxb_powerdown          : std_logic_vector(0 downto 0);
  signal rx_pll_locked          : std_logic_vector(NLANE-1 downto 0);
  signal rx_freqlocked          : std_logic_vector(NLANE-1 downto 0);
  signal txdrst                 : std_logic;
  signal rxarst                 : std_logic;
  signal rxdrst                 : std_logic;

  --
  -- internal use
  --
  signal gxb_pwd_cnt            : std_logic_vector(15 downto 0);
  signal gxb_pwd_cnt_max        : std_logic_vector(15 downto 0);
  signal gxb_pwd_done           : std_logic;
  signal pwrup                  : std_logic;
  signal alltxplllocked         : std_logic;
  signal allrxplllocked         : std_logic;
  signal allrxfreqlocked        : std_logic;
  
begin

  -- share coreclkout(0) among all Tx/Rx PLD logics and MAC layers logics.
  -- for x1 PIPE, use tx_clkout(0) instead.
  gclk0 : global
    port map (
      coreclkout(0),
      clk125
    );
  clk125out <= clk125;

  -- clk125plllock is asserted when all pll_locked(i) are asserted.
  clk125plllock <= alltxplllocked;
  wake          <= '0' when (powerdown = "11" and txelecidle(0) = '0') else 'Z';

  sig_lane : for i in 0 to NLANE-1 generate
    powerdn(2*(i+1)-1 downto 2*i) <= powerdown;
    tx_detectrxloop(i) <= txdetectrx;
    tx_coreclk(i) <= clk125;
    rx_coreclk(i) <= clk125;
  end generate sig_lane;

  -- check if Tx of all GXB blocks locked to their own PLLs.
  process (pll_locked)
    variable alllocked : std_logic;
  begin
    alllocked := '1';
    for i in 0 to (NLANE-1)/4 loop
      if (pll_locked(i) = '0') then
        alllocked := '0';
      end if;
    end loop;
    alltxplllocked <= alllocked;
  end process;

  -- check if Rx of all lanes locked to their own PLLs.
  process (rx_pll_locked)
    variable alllocked : std_logic;
  begin
    alllocked := '1';
    for i in 0 to NLANE-1 loop
      if (rx_pll_locked(i) = '0') then
        alllocked := '0';
      end if;
    end loop;
    allrxplllocked <= alllocked;
  end process;

  -- check if Rx of all lanes locked to in coming data.
  process (rx_freqlocked)
    variable alllocked : std_logic;
  begin
    alllocked := '1';
    for i in 0 to NLANE-1 loop
      if (rx_freqlocked(i) = '0') then
        alllocked := '0';
      end if;
    end loop;
    allrxfreqlocked <= alllocked;
  end process;

  rxstatus <= pipestatus;

  --
  -- alt2gxb generated with MegaWizard.
  --
  genagx1 : if (DEVICE = "Arria GX" and NLANE = 1) generate
    ugxb0 : alt2gxb_agx_x1
      port map (
        cal_blk_clk            => cal_blk_clk,
        fixedclk               => coreclkout(0),
        pll_inclk              => clk100,
        powerdn                => powerdn,

        pll_locked             => pll_locked,
        gxb_powerdown          => gxb_powerdown,
        tx_digitalreset(0)     => txdrst,
        rx_analogreset(0)      => rxarst,
        rx_digitalreset(0)     => rxdrst,
        rx_pll_locked          => rx_pll_locked,
        rx_freqlocked          => rx_freqlocked,

        coreclkout             => coreclkout,
        rx_coreclk             => rx_coreclk,
        tx_coreclk             => tx_coreclk,
        tx_dataout             => tx_out,
        tx_datain              => txdata,
        tx_ctrlenable          => txdatak,
        tx_forceelecidle       => txelecidle,
        tx_detectrxloop        => tx_detectrxloop,
        tx_forcedispcompliance => txcompl,
        
        rx_datain              => rx_in,
        rx_dataout             => rxdata,
        rx_ctrldetect          => rxdatak,
        rx_patterndetect       => open,
        rx_disperr             => open,
        rx_errdetect           => open,
        rx_syncstatus          => open,

        pipe8b10binvpolarity   => rxpolarity,
        pipedatavalid          => rxvalid,
        pipeelecidle           => rxelecidle,
        pipephydonestatus(0)   => phystatus,
        pipestatus             => pipestatus
      );
    end generate genagx1;

  genagx4 : if (DEVICE = "Arria GX" and NLANE = 4) generate
    ugxb0 : alt2gxb_agx_x4
      port map (
        cal_blk_clk            => cal_blk_clk,
        fixedclk               => coreclkout(0),
        pll_inclk              => clk100,
        powerdn                => powerdn,

        pll_locked             => pll_locked,
        gxb_powerdown          => gxb_powerdown,
        tx_digitalreset(0)     => txdrst,
        rx_analogreset(0)      => rxarst,
        rx_digitalreset(0)     => rxdrst,
        rx_pll_locked          => rx_pll_locked,
        rx_freqlocked          => rx_freqlocked,

        coreclkout             => coreclkout,
        rx_coreclk             => rx_coreclk,
        tx_coreclk             => tx_coreclk,
        tx_dataout             => tx_out,
        tx_datain              => txdata,
        tx_ctrlenable          => txdatak,
        tx_forceelecidle       => txelecidle,
        tx_detectrxloop        => tx_detectrxloop,
        tx_forcedispcompliance => txcompl,
        
        rx_datain              => rx_in,
        rx_dataout             => rxdata,
        rx_ctrldetect          => rxdatak,
        rx_patterndetect       => open,
        rx_disperr             => open,
        rx_errdetect           => open,
        rx_syncstatus          => open,

        pipe8b10binvpolarity   => rxpolarity,
        pipedatavalid          => rxvalid,
        pipeelecidle           => rxelecidle,
        pipephydonestatus(0)   => phystatus,
        pipestatus             => pipestatus
      );
    end generate genagx4;

  genagx8 : if (DEVICE = "Arria GX" and NLANE = 8) generate
    ugxb0 : alt2gxb_agx_2x4
      port map (
        cal_blk_clk            => cal_blk_clk,
        fixedclk               => coreclkout(0),
        pll_inclk              => clk100,
        powerdn                => powerdn,

        pll_locked             => pll_locked,
        gxb_powerdown          => gxb_powerdown,
        tx_digitalreset(0)     => txdrst,
        rx_analogreset(0)      => rxarst,
        rx_digitalreset(0)     => rxdrst,
        rx_pll_locked          => rx_pll_locked,
        rx_freqlocked          => rx_freqlocked,

        coreclkout             => coreclkout,
        rx_coreclk             => rx_coreclk,
        tx_coreclk             => tx_coreclk,
        tx_dataout             => tx_out,
        tx_datain              => txdata,
        tx_ctrlenable          => txdatak,
        tx_forceelecidle       => txelecidle,
        tx_detectrxloop        => tx_detectrxloop,
        tx_forcedispcompliance => txcompl,
        
        rx_datain              => rx_in,
        rx_dataout             => rxdata,
        rx_ctrldetect          => rxdatak,
        rx_patterndetect       => open,
        rx_disperr             => open,
        rx_errdetect           => open,
        rx_syncstatus          => open,

        pipe8b10binvpolarity   => rxpolarity,
        pipedatavalid          => rxvalid,
        pipeelecidle           => rxelecidle,
        pipephydonestatus(0)   => phystatus,
        pipestatus             => pipestatus
      );
    end generate genagx8;

  gens2gx1 : if (DEVICE = "Stratix II GX" and NLANE = 1) generate
    ugxb0 : alt2gxb_s2gx_x1
      port map (
        cal_blk_clk            => cal_blk_clk,
        fixedclk               => coreclkout(0),
        pll_inclk              => clk100,
        powerdn                => powerdn,

        pll_locked             => pll_locked,
        gxb_powerdown          => gxb_powerdown,
        tx_digitalreset(0)     => txdrst,
        rx_analogreset(0)      => rxarst,
        rx_digitalreset(0)     => rxdrst,
        rx_pll_locked          => rx_pll_locked,
        rx_freqlocked          => rx_freqlocked,

        coreclkout             => coreclkout,
        rx_coreclk             => rx_coreclk,
        tx_coreclk             => tx_coreclk,
        tx_dataout             => tx_out,
        tx_datain              => txdata,
        tx_ctrlenable          => txdatak,
        tx_forceelecidle       => txelecidle,
        tx_detectrxloop        => tx_detectrxloop,
        tx_forcedispcompliance => txcompl,
        
        rx_datain              => rx_in,
        rx_dataout             => rxdata,
        rx_ctrldetect          => rxdatak,
        rx_patterndetect       => open,
        rx_disperr             => open,
        rx_errdetect           => open,
        rx_syncstatus          => open,

        pipe8b10binvpolarity   => rxpolarity,
        pipedatavalid          => rxvalid,
        pipeelecidle           => rxelecidle,
        pipephydonestatus(0)   => phystatus,
        pipestatus             => pipestatus
      );
    end generate gens2gx1;

  gens2gx4 : if (DEVICE = "Stratix II GX" and NLANE = 4) generate
    ugxb0 : alt2gxb_s2gx_x4
      port map (
        cal_blk_clk            => cal_blk_clk,
        fixedclk               => coreclkout(0),
        pll_inclk              => clk100,
        powerdn                => powerdn,

        pll_locked             => pll_locked,
        gxb_powerdown          => gxb_powerdown,
        tx_digitalreset(0)     => txdrst,
        rx_analogreset(0)      => rxarst,
        rx_digitalreset(0)     => rxdrst,
        rx_pll_locked          => rx_pll_locked,
        rx_freqlocked          => rx_freqlocked,

        coreclkout             => coreclkout,
        rx_coreclk             => rx_coreclk,
        tx_coreclk             => tx_coreclk,
        tx_dataout             => tx_out,
        tx_datain              => txdata,
        tx_ctrlenable          => txdatak,
        tx_forceelecidle       => txelecidle,
        tx_detectrxloop        => tx_detectrxloop,
        tx_forcedispcompliance => txcompl,
        
        rx_datain              => rx_in,
        rx_dataout             => rxdata,
        rx_ctrldetect          => rxdatak,
        rx_patterndetect       => open,
        rx_disperr             => open,
        rx_errdetect           => open,
        rx_syncstatus          => open,

        pipe8b10binvpolarity   => rxpolarity,
        pipedatavalid          => rxvalid,
        pipeelecidle           => rxelecidle,
        pipephydonestatus(0)   => phystatus,
        pipestatus             => pipestatus
      );
    end generate gens2gx4;

  gens2gx8 : if (DEVICE = "Stratix II GX" and NLANE = 8) generate
    ugxb0 : alt2gxb_s2gx_x8
      port map (
        cal_blk_clk            => cal_blk_clk,
        fixedclk               => coreclkout(0),
        pll_inclk              => clk100,
        powerdn                => powerdn,

        pll_locked             => pll_locked,
        gxb_powerdown          => gxb_powerdown,
        tx_digitalreset(0)     => txdrst,
        rx_analogreset(0)      => rxarst,
        rx_digitalreset(0)     => rxdrst,
        rx_pll_locked          => rx_pll_locked,
        rx_freqlocked          => rx_freqlocked,

        coreclkout             => coreclkout,
        rx_coreclk             => rx_coreclk,
        tx_coreclk             => tx_coreclk,
        tx_dataout             => tx_out,
        tx_datain              => txdata,
        tx_ctrlenable          => txdatak,
        tx_forceelecidle       => txelecidle,
        tx_detectrxloop        => tx_detectrxloop,
        tx_forcedispcompliance => txcompl,
        
        rx_datain              => rx_in,
        rx_dataout             => rxdata,
        rx_ctrldetect          => rxdatak,
        rx_patterndetect       => open,
        rx_disperr             => open,
        rx_errdetect           => open,
        rx_syncstatus          => open,

        pipe8b10binvpolarity   => rxpolarity,
        pipedatavalid          => rxvalid,
        pipeelecidle           => rxelecidle,
        pipephydonestatus(0)   => phystatus,
        pipestatus             => pipestatus
      );
    end generate gens2gx8;

  --
  -- manage reset signals.
  --
  uresetctrl0 : resetctrl
    port map (
      rstn            => rstn,
      pwrup           => pwrup,
      clk32           => clk32,
      clk125          => clk125,
      txplllocked     => alltxplllocked,
      rxfreqlocked    => allrxfreqlocked,
      rxplllocked     => allrxplllocked,
      pipestatus      => pipestatus,
      txdrst          => txdrst,
      rxarst          => rxarst,
      rxdrst          => rxdrst
    );
  
  --
  -- control logic for gxb_powerdown.
  --
  gxb_powerdown(0) <= not pwrup;

  -- assert powerdown for more than 100ns (for PCIe compliance phase).
  -- any value longer than 100ns will do.
  ifsim0: if (SIMULATION = 0) generate
    gxb_pwd_cnt_max <= "0000000100000000";
  end generate;
  -- 16 clk is enough for simulation.
  ifsim1: if (SIMULATION = 1) generate
    gxb_pwd_cnt_max <= "0000000000010000";
  end generate;

  ifclk32 : if (USE_CLK32 = 1) generate
    process (rstn, clk32)
    begin
      if rstn = '0' then
        gxb_pwd_cnt   <= "0000000000000000";
        pwrup         <= '0';
      elsif rising_edge(clk32) then
        if (gxb_pwd_cnt = gxb_pwd_cnt_max) then
          pwrup <= '1';
        else
          gxb_pwd_cnt <= unsigned(gxb_pwd_cnt) + '1' ;
        end if;
      end if;
    end process;
  end generate ifclk32;

  ifnoclk32 : if (USE_CLK32 = 0) generate
    process (rstn)
    begin
      if rstn = '0' then
        pwrup <= '0';
      else
        pwrup <= '1';
      end if;
    end process;
  end generate ifnoclk32;

end default;


--
-- reset controller
--
-- controls tx_digitalreset, rx_analogreset, and rx_digitalreset.
-- 

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity resetctrl is
  generic (
    USE_CLK32    : integer := 1;
    NLANE        : integer;
    SIMULATION   : integer
  );
  port (
    rstn            : in  std_logic;
    pwrup           : in  std_logic;
    clk32           : in  std_logic;
    clk125          : in  std_logic;
    txplllocked     : in  std_logic;
    rxfreqlocked    : in  std_logic;
    rxplllocked     : in  std_logic;
    pipestatus      : in  std_logic_vector(NLANE*3-1 downto 0);
    txdrst          : out std_logic;
    rxarst          : out std_logic;
    rxdrst          : out std_logic
  );
end resetctrl;
    
architecture default of resetctrl is

  signal rxdrst0       : std_logic;
  signal rxdrst1       : std_logic;
  signal rxfrequp      : std_logic;
  signal rxfreqlocked1 : std_logic;
  signal rmerr         : std_logic;  -- rate match FIFO overflow/underflow

begin

  ifclk32 : if (USE_CLK32 = 1) generate

    process (rstn, clk32)
    begin
      if rstn = '0' then
        txdrst   <= '0';
        rxarst   <= '0';
      elsif rising_edge(clk32) then
        if (pwrup = '0') then
          txdrst <= '1';
          rxarst <= '1';
        elsif (txplllocked = '1') then
          txdrst <= '0';
          rxarst <= '0';
        end if;
      end if;
    end process;
  end generate ifclk32;

  ifnoclk32 : if (USE_CLK32 = 0) generate
    process (rstn)
    begin
      if rstn = '0' then
        txdrst   <= '0';
        rxarst   <= '0';
      else
        if (pwrup = '0') then
          txdrst <= '1';
          rxarst <= '1';
        elsif (txplllocked = '1') then
          txdrst <= '0';
          rxarst <= '0';
        end if;
      end if;
    end process;
  end generate ifnoclk32;

  rxdrst  <= rxdrst0;

  process (txplllocked, clk125)
    variable any_rmerr : std_logic;
  begin
    if (txplllocked = '0') then

      rxdrst0       <= '1';
      rxdrst1       <= '0';
      rxfreqlocked1 <= '0';
      rxfrequp      <= '0';
      rmerr         <= '0';      

    elsif rising_edge(clk125) then

      -- rx_freqlocked status.
      rxfreqlocked1 <= rxfreqlocked;
      if (rxfreqlocked = '1' and rxfreqlocked1 = '0') then
        rxfrequp <= '1';    -- rx_freqlocked for all channels up just now.
      else
        rxfrequp <= '0';
      end if;

      -- rate match FIFO overflow/underflow on any lane.
      any_rmerr := '0';
      for i in 0 to NLANE-1 loop
        if (pipestatus(3*(i+1)-1 downto 3*i) = "101" or
            pipestatus(3*(i+1)-1 downto 3*i) = "110") then
          any_rmerr := '1';
        end if;
      end loop;
      rmerr <= any_rmerr;

      -- during initialization phase, assert rx_digitalreset until
      -- rx_pll_locked is asserted for all channels.
      if rxfreqlocked = '0' then
        if rxplllocked = '1' then
          rxdrst0 <= '0';
        end if;
      else
        -- during normal operation phase, if rx_freqlocked
        -- accidentally go down and up again, or if rate match FIFO
        -- overflow/underflow, assert rx_drst for 2clks.
        if (rxdrst1 = '1') then
          rxdrst0 <= '0';
        elsif (rxfrequp = '1' or rmerr = '1') then
          rxdrst0 <= '1';
        end if;
      end if;

      rxdrst1 <= rxdrst0;

    end if;
  end process;

end default;
