

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity phy125 is
  generic (
    DEVICE : string := "Arria GX";
    USE_CLK32 : integer := 1; -- use clock input pin 'clk32'.
    NLANE : integer;
    SIMULATION : integer
  );
  port (
    cal_blk_clk : in std_logic; -- used for GXB calibration.
    clk125plllock : out std_logic;
    clk32 : in std_logic; -- used only for powerup timing control.
    clk100 : in std_logic; -- dedicated to GXB refclk.
    clk125out : out std_logic;
    rstn : in std_logic;

    rx_in : in std_logic_vector(NLANE-1 downto 0);
    tx_out : out std_logic_vector(NLANE-1 downto 0);

    wake : out std_logic;
    phystatus : out std_logic;
    powerdown : in std_logic_vector(1 downto 0);
    txdetectrx : in std_logic;

    txdata : in std_logic_vector(NLANE*16-1 downto 0);
    txdatak : in std_logic_vector(NLANE*2-1 downto 0);
    txelecidle : in std_logic_vector(NLANE-1 downto 0);
    txcompl : in std_logic_vector(NLANE-1 downto 0);
    rxpolarity : in std_logic_vector(NLANE-1 downto 0);
    rxdata : out std_logic_vector(NLANE*16-1 downto 0);
    rxdatak : out std_logic_vector(NLANE*2-1 downto 0);
    rxvalid : out std_logic_vector(NLANE-1 downto 0);
    rxelecidle : out std_logic_vector(NLANE-1 downto 0);
    rxstatus : out std_logic_vector(NLANE*3-1 downto 0)
  );
end phy125;

architecture default of phy125 is

  component alt2gxb_agx_x1
    port (
      cal_blk_clk : in std_logic;
      fixedclk : in std_logic;
      pll_inclk : in std_logic;
      powerdn : in std_logic_vector(NLANE*2-1 downto 0);

      pll_locked : out std_logic_vector((NLANE-1)/4 downto 0);
      gxb_powerdown : in std_logic_vector(0 downto 0);
      tx_digitalreset : in std_logic_vector(0 downto 0);
      rx_analogreset : in std_logic_vector(0 downto 0);
      rx_digitalreset : in std_logic_vector(0 downto 0);
      rx_pll_locked : out std_logic_vector(NLANE-1 downto 0);
      rx_freqlocked : out std_logic_vector(NLANE-1 downto 0);

      coreclkout : out std_logic_vector((NLANE-1)/4 downto 0);
      tx_coreclk : in std_logic_vector(NLANE-1 downto 0);
      rx_coreclk : in std_logic_vector(NLANE-1 downto 0);
      tx_dataout : out std_logic_vector(NLANE-1 downto 0);
      tx_datain : in std_logic_vector(NLANE*16-1 downto 0);
      tx_ctrlenable : in std_logic_vector(NLANE*2-1 downto 0);
      tx_forceelecidle : in std_logic_vector(NLANE-1 downto 0);
      tx_detectrxloop : in std_logic_vector(NLANE-1 downto 0);
      tx_forcedispcompliance : in std_logic_vector(NLANE-1 downto 0);
      pipe8b10binvpolarity : in std_logic_vector(NLANE-1 downto 0);

      rx_datain : in std_logic_vector(NLANE-1 downto 0);
      rx_dataout : out std_logic_vector(NLANE*16-1 downto 0);
      rx_ctrldetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_patterndetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_disperr : out std_logic_vector(NLANE*2-1 downto 0);
      rx_errdetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_syncstatus : out std_logic_vector(NLANE*2-1 downto 0);

      pipedatavalid : out std_logic_vector(NLANE-1 downto 0);
      pipeelecidle : out std_logic_vector(NLANE-1 downto 0);
      pipephydonestatus : out std_logic_vector(NLANE-1 downto 0);
      pipestatus : out std_logic_vector(NLANE*3-1 downto 0)
    );
  end component;

  component alt2gxb_agx_x4
    port (
      cal_blk_clk : in std_logic;
      fixedclk : in std_logic;
      pll_inclk : in std_logic;
      powerdn : in std_logic_vector(NLANE*2-1 downto 0);

      pll_locked : out std_logic_vector((NLANE-1)/4 downto 0);
      gxb_powerdown : in std_logic_vector(0 downto 0);
      tx_digitalreset : in std_logic_vector(0 downto 0);
      rx_analogreset : in std_logic_vector(0 downto 0);
      rx_digitalreset : in std_logic_vector(0 downto 0);
      rx_pll_locked : out std_logic_vector(NLANE-1 downto 0);
      rx_freqlocked : out std_logic_vector(NLANE-1 downto 0);

      coreclkout : out std_logic_vector((NLANE-1)/4 downto 0);
      tx_coreclk : in std_logic_vector(NLANE-1 downto 0);
      rx_coreclk : in std_logic_vector(NLANE-1 downto 0);
      tx_dataout : out std_logic_vector(NLANE-1 downto 0);
      tx_datain : in std_logic_vector(NLANE*16-1 downto 0);
      tx_ctrlenable : in std_logic_vector(NLANE*2-1 downto 0);
      tx_forceelecidle : in std_logic_vector(NLANE-1 downto 0);
      tx_detectrxloop : in std_logic_vector(NLANE-1 downto 0);
      tx_forcedispcompliance : in std_logic_vector(NLANE-1 downto 0);
      pipe8b10binvpolarity : in std_logic_vector(NLANE-1 downto 0);

      rx_datain : in std_logic_vector(NLANE-1 downto 0);
      rx_dataout : out std_logic_vector(NLANE*16-1 downto 0);
      rx_ctrldetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_patterndetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_disperr : out std_logic_vector(NLANE*2-1 downto 0);
      rx_errdetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_syncstatus : out std_logic_vector(NLANE*2-1 downto 0);

      pipedatavalid : out std_logic_vector(NLANE-1 downto 0);
      pipeelecidle : out std_logic_vector(NLANE-1 downto 0);
      pipephydonestatus : out std_logic_vector(NLANE-1 downto 0);
      pipestatus : out std_logic_vector(NLANE*3-1 downto 0)
    );
  end component;

  component alt2gxb_agx_2x4
    port (
      cal_blk_clk : in std_logic;
      fixedclk : in std_logic;
      pll_inclk : in std_logic;
      powerdn : in std_logic_vector(NLANE*2-1 downto 0);

      pll_locked : out std_logic_vector((NLANE-1)/4 downto 0);
      gxb_powerdown : in std_logic_vector(0 downto 0);
      tx_digitalreset : in std_logic_vector(0 downto 0);
      rx_analogreset : in std_logic_vector(0 downto 0);
      rx_digitalreset : in std_logic_vector(0 downto 0);
      rx_pll_locked : out std_logic_vector(NLANE-1 downto 0);
      rx_freqlocked : out std_logic_vector(NLANE-1 downto 0);

      coreclkout : out std_logic_vector((NLANE-1)/4 downto 0);
      tx_coreclk : in std_logic_vector(NLANE-1 downto 0);
      rx_coreclk : in std_logic_vector(NLANE-1 downto 0);
      tx_dataout : out std_logic_vector(NLANE-1 downto 0);
      tx_datain : in std_logic_vector(NLANE*16-1 downto 0);
      tx_ctrlenable : in std_logic_vector(NLANE*2-1 downto 0);
      tx_forceelecidle : in std_logic_vector(NLANE-1 downto 0);
      tx_detectrxloop : in std_logic_vector(NLANE-1 downto 0);
      tx_forcedispcompliance : in std_logic_vector(NLANE-1 downto 0);
      pipe8b10binvpolarity : in std_logic_vector(NLANE-1 downto 0);

      rx_datain : in std_logic_vector(NLANE-1 downto 0);
      rx_dataout : out std_logic_vector(NLANE*16-1 downto 0);
      rx_ctrldetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_patterndetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_disperr : out std_logic_vector(NLANE*2-1 downto 0);
      rx_errdetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_syncstatus : out std_logic_vector(NLANE*2-1 downto 0);

      pipedatavalid : out std_logic_vector(NLANE-1 downto 0);
      pipeelecidle : out std_logic_vector(NLANE-1 downto 0);
      pipephydonestatus : out std_logic_vector(NLANE-1 downto 0);
      pipestatus : out std_logic_vector(NLANE*3-1 downto 0)
    );
  end component;

  component alt2gxb_s2gx_x1
    port (
      cal_blk_clk : in std_logic;
      fixedclk : in std_logic;
      pll_inclk : in std_logic;
      powerdn : in std_logic_vector(NLANE*2-1 downto 0);

      pll_locked : out std_logic_vector((NLANE-1)/4 downto 0);
      gxb_powerdown : in std_logic_vector(0 downto 0);
      tx_digitalreset : in std_logic_vector(0 downto 0);
      rx_analogreset : in std_logic_vector(0 downto 0);
      rx_digitalreset : in std_logic_vector(0 downto 0);
      rx_pll_locked : out std_logic_vector(NLANE-1 downto 0);
      rx_freqlocked : out std_logic_vector(NLANE-1 downto 0);

      coreclkout : out std_logic_vector((NLANE-1)/4 downto 0);
      tx_coreclk : in std_logic_vector(NLANE-1 downto 0);
      rx_coreclk : in std_logic_vector(NLANE-1 downto 0);
      tx_dataout : out std_logic_vector(NLANE-1 downto 0);
      tx_datain : in std_logic_vector(NLANE*16-1 downto 0);
      tx_ctrlenable : in std_logic_vector(NLANE*2-1 downto 0);
      tx_forceelecidle : in std_logic_vector(NLANE-1 downto 0);
      tx_detectrxloop : in std_logic_vector(NLANE-1 downto 0);
      tx_forcedispcompliance : in std_logic_vector(NLANE-1 downto 0);
      pipe8b10binvpolarity : in std_logic_vector(NLANE-1 downto 0);

      rx_datain : in std_logic_vector(NLANE-1 downto 0);
      rx_dataout : out std_logic_vector(NLANE*16-1 downto 0);
      rx_ctrldetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_patterndetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_disperr : out std_logic_vector(NLANE*2-1 downto 0);
      rx_errdetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_syncstatus : out std_logic_vector(NLANE*2-1 downto 0);

      pipedatavalid : out std_logic_vector(NLANE-1 downto 0);
      pipeelecidle : out std_logic_vector(NLANE-1 downto 0);
      pipephydonestatus : out std_logic_vector(NLANE-1 downto 0);
      pipestatus : out std_logic_vector(NLANE*3-1 downto 0)
    );
  end component;

  component alt2gxb_s2gx_x4
    port (
      cal_blk_clk : in std_logic;
      fixedclk : in std_logic;
      pll_inclk : in std_logic;
      powerdn : in std_logic_vector(NLANE*2-1 downto 0);

      pll_locked : out std_logic_vector((NLANE-1)/4 downto 0);
      gxb_powerdown : in std_logic_vector(0 downto 0);
      tx_digitalreset : in std_logic_vector(0 downto 0);
      rx_analogreset : in std_logic_vector(0 downto 0);
      rx_digitalreset : in std_logic_vector(0 downto 0);
      rx_pll_locked : out std_logic_vector(NLANE-1 downto 0);
      rx_freqlocked : out std_logic_vector(NLANE-1 downto 0);

      coreclkout : out std_logic_vector((NLANE-1)/4 downto 0);
      tx_coreclk : in std_logic_vector(NLANE-1 downto 0);
      rx_coreclk : in std_logic_vector(NLANE-1 downto 0);
      tx_dataout : out std_logic_vector(NLANE-1 downto 0);
      tx_datain : in std_logic_vector(NLANE*16-1 downto 0);
      tx_ctrlenable : in std_logic_vector(NLANE*2-1 downto 0);
      tx_forceelecidle : in std_logic_vector(NLANE-1 downto 0);
      tx_detectrxloop : in std_logic_vector(NLANE-1 downto 0);
      tx_forcedispcompliance : in std_logic_vector(NLANE-1 downto 0);
      pipe8b10binvpolarity : in std_logic_vector(NLANE-1 downto 0);

      rx_datain : in std_logic_vector(NLANE-1 downto 0);
      rx_dataout : out std_logic_vector(NLANE*16-1 downto 0);
      rx_ctrldetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_patterndetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_disperr : out std_logic_vector(NLANE*2-1 downto 0);
      rx_errdetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_syncstatus : out std_logic_vector(NLANE*2-1 downto 0);

      pipedatavalid : out std_logic_vector(NLANE-1 downto 0);
      pipeelecidle : out std_logic_vector(NLANE-1 downto 0);
      pipephydonestatus : out std_logic_vector(NLANE-1 downto 0);
      pipestatus : out std_logic_vector(NLANE*3-1 downto 0)
    );
  end component;

  component alt2gxb_s2gx_x8
    port (
      cal_blk_clk : in std_logic;
      fixedclk : in std_logic;
      pll_inclk : in std_logic;
      powerdn : in std_logic_vector(NLANE*2-1 downto 0);

      pll_locked : out std_logic_vector((NLANE-1)/4 downto 0);
      gxb_powerdown : in std_logic_vector(0 downto 0);
      tx_digitalreset : in std_logic_vector(0 downto 0);
      rx_analogreset : in std_logic_vector(0 downto 0);
      rx_digitalreset : in std_logic_vector(0 downto 0);
      rx_pll_locked : out std_logic_vector(NLANE-1 downto 0);
      rx_freqlocked : out std_logic_vector(NLANE-1 downto 0);

      coreclkout : out std_logic_vector((NLANE-1)/4 downto 0);
      tx_coreclk : in std_logic_vector(NLANE-1 downto 0);
      rx_coreclk : in std_logic_vector(NLANE-1 downto 0);
      tx_dataout : out std_logic_vector(NLANE-1 downto 0);
      tx_datain : in std_logic_vector(NLANE*16-1 downto 0);
      tx_ctrlenable : in std_logic_vector(NLANE*2-1 downto 0);
      tx_forceelecidle : in std_logic_vector(NLANE-1 downto 0);
      tx_detectrxloop : in std_logic_vector(NLANE-1 downto 0);
      tx_forcedispcompliance : in std_logic_vector(NLANE-1 downto 0);
      pipe8b10binvpolarity : in std_logic_vector(NLANE-1 downto 0);

      rx_datain : in std_logic_vector(NLANE-1 downto 0);
      rx_dataout : out std_logic_vector(NLANE*16-1 downto 0);
      rx_ctrldetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_patterndetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_disperr : out std_logic_vector(NLANE*2-1 downto 0);
      rx_errdetect : out std_logic_vector(NLANE*2-1 downto 0);
      rx_syncstatus : out std_logic_vector(NLANE*2-1 downto 0);

      pipedatavalid : out std_logic_vector(NLANE-1 downto 0);
      pipeelecidle : out std_logic_vector(NLANE-1 downto 0);
      pipephydonestatus : out std_logic_vector(NLANE-1 downto 0);
      pipestatus : out std_logic_vector(NLANE*3-1 downto 0)
    );
  end component;

  component resetctrl
    generic (
      USE_CLK32 : integer := USE_CLK32;
      NLANE : integer := NLANE;
      SIMULATION : integer := SIMULATION
    );
    port (
      rstn : in std_logic;
      pwrup : in std_logic;
      clk32 : in std_logic;
      clk125 : in std_logic;
      txplllocked : in std_logic;
      rxfreqlocked : in std_logic;
      rxplllocked : in std_logic;
      pipestatus : in std_logic_vector(NLANE*3-1 downto 0);
      txdrst : out std_logic;
      rxarst : out std_logic;
      rxdrst : out std_logic
    );
  end component;

  component global
    port (
      a_in : in std_logic;
      a_out : out std_logic
    );
  end component;

  --
  -- connection between PIPE and alt2gxb.
  --
  signal clk125 : std_logic;
  signal powerdn : std_logic_vector(NLANE*2-1 downto 0);
  signal pipestatus : std_logic_vector(NLANE*3-1 downto 0);
  signal tx_detectrxloop : std_logic_vector(NLANE-1 downto 0);
  signal pll_locked : std_logic_vector((NLANE-1)/4 downto 0);
  signal coreclkout : std_logic_vector((NLANE-1)/4 downto 0);
  signal rx_coreclk : std_logic_vector(NLANE-1 downto 0);
  signal tx_coreclk : std_logic_vector(NLANE-1 downto 0);
  signal gxb_powerdown : std_logic_vector(0 downto 0);
  signal rx_pll_locked : std_logic_vector(NLANE-1 downto 0);
  signal rx_freqlocked : std_logic_vector(NLANE-1 downto 0);
  signal txdrst : std_logic;
  signal rxarst : std_logic;
  signal rxdrst : std_logic;

  --
  -- internal use
  --
  signal gxb_pwd_cnt : std_logic_vector(15 downto 0);
  signal gxb_pwd_cnt_max : std_logic_vector(15 downto 0);
  signal gxb_pwd_done : std_logic;
  signal pwrup : std_logic;
  signal alltxplllocked : std_logic;
  signal allrxplllocked : std_logic;
  signal allrxfreqlocked : std_logic;

begin

  -- share coreclkout(0) among all Tx/Rx PLD logics and MAC layers logics.
  -- for x1 PIPE, use tx_clkout(0) instead.
  gclk0 : global
    port map (
      coreclkout(0),
      clk125
    );
  clk125out <= clk125;

  -- clk125plllock is asserted when all pll_locked(i) are asserted.
  clk125plllock <= alltxplllocked;
  wake <= '0' when (powerdown = "11" and txelecidle(0) = '0') else 'Z';

  sig_lane : for i in 0 to NLANE-1 generate
    powerdn(2*(i+1)-1 downto 2*i) <= powerdown;
    tx_detectrxloop(i) <= txdetectrx;
    tx_coreclk(i) <= clk125;
    rx_coreclk(i) <= clk125;
  end generate sig_lane;

  -- check if Tx of all GXB blocks locked to their own PLLs.
  process (pll_locked)
    variable alllocked : std_logic;
  begin
    alllocked := '1';
    for i in 0 to (NLANE-1)/4 loop
      if (pll_locked(i) = '0') then
        alllocked := '0';
      end if;
    end loop;
    alltxplllocked <= alllocked;
  end process;

  -- check if Rx of all lanes locked to their own PLLs.
  process (rx_pll_locked)
    variable alllocked : std_logic;
  begin
    alllocked := '1';
    for i in 0 to NLANE-1 loop
      if (rx_pll_locked(i) = '0') then
        alllocked := '0';
      end if;
    end loop;
    allrxplllocked <= alllocked;
  end process;

  -- check if Rx of all lanes locked to in coming data.
  process (rx_freqlocked)
    variable alllocked : std_logic;
  begin
    alllocked := '1';
    for i in 0 to NLANE-1 loop
      if (rx_freqlocked(i) = '0') then
        alllocked := '0';
      end if;
    end loop;
    allrxfreqlocked <= alllocked;
  end process;

  rxstatus <= pipestatus;

  --
  -- alt2gxb generated with MegaWizard.
  --
  genagx1 : if (DEVICE = "Arria GX" and NLANE = 1) generate
    ugxb0 : alt2gxb_agx_x1
      port map (
        cal_blk_clk => cal_blk_clk,
        fixedclk => coreclkout(0),
        pll_inclk => clk100,
        powerdn => powerdn,

        pll_locked => pll_locked,
        gxb_powerdown => gxb_powerdown,
        tx_digitalreset(0) => txdrst,
        rx_analogreset(0) => rxarst,
        rx_digitalreset(0) => rxdrst,
        rx_pll_locked => rx_pll_locked,
        rx_freqlocked => rx_freqlocked,

        coreclkout => coreclkout,
        rx_coreclk => rx_coreclk,
        tx_coreclk => tx_coreclk,
        tx_dataout => tx_out,
        tx_datain => txdata,
        tx_ctrlenable => txdatak,
        tx_forceelecidle => txelecidle,
        tx_detectrxloop => tx_detectrxloop,
        tx_forcedispcompliance => txcompl,

        rx_datain => rx_in,
        rx_dataout => rxdata,
        rx_ctrldetect => rxdatak,
        rx_patterndetect => open,
        rx_disperr => open,
        rx_errdetect => open,
        rx_syncstatus => open,

        pipe8b10binvpolarity => rxpolarity,
        pipedatavalid => rxvalid,
        pipeelecidle => rxelecidle,
        pipephydonestatus(0) => phystatus,
        pipestatus => pipestatus
      );
    end generate genagx1;

  genagx4 : if (DEVICE = "Arria GX" and NLANE = 4) generate
    ugxb0 : alt2gxb_agx_x4
      port map (
        cal_blk_clk => cal_blk_clk,
        fixedclk => coreclkout(0),
        pll_inclk => clk100,
        powerdn => powerdn,

        pll_locked => pll_locked,
        gxb_powerdown => gxb_powerdown,
        tx_digitalreset(0) => txdrst,
        rx_analogreset(0) => rxarst,
        rx_digitalreset(0) => rxdrst,
        rx_pll_locked => rx_pll_locked,
        rx_freqlocked => rx_freqlocked,

        coreclkout => coreclkout,
        rx_coreclk => rx_coreclk,
        tx_coreclk => tx_coreclk,
        tx_dataout => tx_out,
        tx_datain => txdata,
        tx_ctrlenable => txdatak,
        tx_forceelecidle => txelecidle,
        tx_detectrxloop => tx_detectrxloop,
        tx_forcedispcompliance => txcompl,

        rx_datain => rx_in,
        rx_dataout => rxdata,
        rx_ctrldetect => rxdatak,
        rx_patterndetect => open,
        rx_disperr => open,
        rx_errdetect => open,
        rx_syncstatus => open,

        pipe8b10binvpolarity => rxpolarity,
        pipedatavalid => rxvalid,
        pipeelecidle => rxelecidle,
        pipephydonestatus(0) => phystatus,
        pipestatus => pipestatus
      );
    end generate genagx4;

  genagx8 : if (DEVICE = "Arria GX" and NLANE = 8) generate
    ugxb0 : alt2gxb_agx_2x4
      port map (
        cal_blk_clk => cal_blk_clk,
        fixedclk => coreclkout(0),
        pll_inclk => clk100,
        powerdn => powerdn,

        pll_locked => pll_locked,
        gxb_powerdown => gxb_powerdown,
        tx_digitalreset(0) => txdrst,
        rx_analogreset(0) => rxarst,
        rx_digitalreset(0) => rxdrst,
        rx_pll_locked => rx_pll_locked,
        rx_freqlocked => rx_freqlocked,

        coreclkout => coreclkout,
        rx_coreclk => rx_coreclk,
        tx_coreclk => tx_coreclk,
        tx_dataout => tx_out,
        tx_datain => txdata,
        tx_ctrlenable => txdatak,
        tx_forceelecidle => txelecidle,
        tx_detectrxloop => tx_detectrxloop,
        tx_forcedispcompliance => txcompl,

        rx_datain => rx_in,
        rx_dataout => rxdata,
        rx_ctrldetect => rxdatak,
        rx_patterndetect => open,
        rx_disperr => open,
        rx_errdetect => open,
        rx_syncstatus => open,

        pipe8b10binvpolarity => rxpolarity,
        pipedatavalid => rxvalid,
        pipeelecidle => rxelecidle,
        pipephydonestatus(0) => phystatus,
        pipestatus => pipestatus
      );
    end generate genagx8;

  gens2gx1 : if (DEVICE = "Stratix II GX" and NLANE = 1) generate
    ugxb0 : alt2gxb_s2gx_x1
      port map (
        cal_blk_clk => cal_blk_clk,
        fixedclk => coreclkout(0),
        pll_inclk => clk100,
        powerdn => powerdn,

        pll_locked => pll_locked,
        gxb_powerdown => gxb_powerdown,
        tx_digitalreset(0) => txdrst,
        rx_analogreset(0) => rxarst,
        rx_digitalreset(0) => rxdrst,
        rx_pll_locked => rx_pll_locked,
        rx_freqlocked => rx_freqlocked,

        coreclkout => coreclkout,
        rx_coreclk => rx_coreclk,
        tx_coreclk => tx_coreclk,
        tx_dataout => tx_out,
        tx_datain => txdata,
        tx_ctrlenable => txdatak,
        tx_forceelecidle => txelecidle,
        tx_detectrxloop => tx_detectrxloop,
        tx_forcedispcompliance => txcompl,

        rx_datain => rx_in,
        rx_dataout => rxdata,
        rx_ctrldetect => rxdatak,
        rx_patterndetect => open,
        rx_disperr => open,
        rx_errdetect => open,
        rx_syncstatus => open,

        pipe8b10binvpolarity => rxpolarity,
        pipedatavalid => rxvalid,
        pipeelecidle => rxelecidle,
        pipephydonestatus(0) => phystatus,
        pipestatus => pipestatus
      );
    end generate gens2gx1;

  gens2gx4 : if (DEVICE = "Stratix II GX" and NLANE = 4) generate
    ugxb0 : alt2gxb_s2gx_x4
      port map (
        cal_blk_clk => cal_blk_clk,
        fixedclk => coreclkout(0),
        pll_inclk => clk100,
        powerdn => powerdn,

        pll_locked => pll_locked,
        gxb_powerdown => gxb_powerdown,
        tx_digitalreset(0) => txdrst,
        rx_analogreset(0) => rxarst,
        rx_digitalreset(0) => rxdrst,
        rx_pll_locked => rx_pll_locked,
        rx_freqlocked => rx_freqlocked,

        coreclkout => coreclkout,
        rx_coreclk => rx_coreclk,
        tx_coreclk => tx_coreclk,
        tx_dataout => tx_out,
        tx_datain => txdata,
        tx_ctrlenable => txdatak,
        tx_forceelecidle => txelecidle,
        tx_detectrxloop => tx_detectrxloop,
        tx_forcedispcompliance => txcompl,

        rx_datain => rx_in,
        rx_dataout => rxdata,
        rx_ctrldetect => rxdatak,
        rx_patterndetect => open,
        rx_disperr => open,
        rx_errdetect => open,
        rx_syncstatus => open,

        pipe8b10binvpolarity => rxpolarity,
        pipedatavalid => rxvalid,
        pipeelecidle => rxelecidle,
        pipephydonestatus(0) => phystatus,
        pipestatus => pipestatus
      );
    end generate gens2gx4;

  gens2gx8 : if (DEVICE = "Stratix II GX" and NLANE = 8) generate
    ugxb0 : alt2gxb_s2gx_x8
      port map (
        cal_blk_clk => cal_blk_clk,
        fixedclk => coreclkout(0),
        pll_inclk => clk100,
        powerdn => powerdn,

        pll_locked => pll_locked,
        gxb_powerdown => gxb_powerdown,
        tx_digitalreset(0) => txdrst,
        rx_analogreset(0) => rxarst,
        rx_digitalreset(0) => rxdrst,
        rx_pll_locked => rx_pll_locked,
        rx_freqlocked => rx_freqlocked,

        coreclkout => coreclkout,
        rx_coreclk => rx_coreclk,
        tx_coreclk => tx_coreclk,
        tx_dataout => tx_out,
        tx_datain => txdata,
        tx_ctrlenable => txdatak,
        tx_forceelecidle => txelecidle,
        tx_detectrxloop => tx_detectrxloop,
        tx_forcedispcompliance => txcompl,

        rx_datain => rx_in,
        rx_dataout => rxdata,
        rx_ctrldetect => rxdatak,
        rx_patterndetect => open,
        rx_disperr => open,
        rx_errdetect => open,
        rx_syncstatus => open,

        pipe8b10binvpolarity => rxpolarity,
        pipedatavalid => rxvalid,
        pipeelecidle => rxelecidle,
        pipephydonestatus(0) => phystatus,
        pipestatus => pipestatus
      );
    end generate gens2gx8;

  --
  -- manage reset signals.
  --
  uresetctrl0 : resetctrl
    port map (
      rstn => rstn,
      pwrup => pwrup,
      clk32 => clk32,
      clk125 => clk125,
      txplllocked => alltxplllocked,
      rxfreqlocked => allrxfreqlocked,
      rxplllocked => allrxplllocked,
      pipestatus => pipestatus,
      txdrst => txdrst,
      rxarst => rxarst,
      rxdrst => rxdrst
    );

  --
  -- control logic for gxb_powerdown.
  --
  gxb_powerdown(0) <= not pwrup;

  -- assert powerdown for more than 100ns (for PCIe compliance phase).
  -- any value longer than 100ns will do.
  ifsim0: if (SIMULATION = 0) generate
    gxb_pwd_cnt_max <= "0000000100000000";
  end generate;
  -- 16 clk is enough for simulation.
  ifsim1: if (SIMULATION = 1) generate
    gxb_pwd_cnt_max <= "0000000000010000";
  end generate;

  ifclk32 : if (USE_CLK32 = 1) generate
    process (rstn, clk32)
    begin
      if rstn = '0' then
        gxb_pwd_cnt <= "0000000000000000";
        pwrup <= '0';
      elsif rising_edge(clk32) then
        if (gxb_pwd_cnt = gxb_pwd_cnt_max) then
          pwrup <= '1';
        else
          gxb_pwd_cnt <= unsigned(gxb_pwd_cnt) + '1' ;
        end if;
      end if;
    end process;
  end generate ifclk32;

  ifnoclk32 : if (USE_CLK32 = 0) generate
    process (rstn)
    begin
      if rstn = '0' then
        pwrup <= '0';
      else
        pwrup <= '1';
      end if;
    end process;
  end generate ifnoclk32;

end default;


--
-- reset controller
--
-- controls tx_digitalreset, rx_analogreset, and rx_digitalreset.
--

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

entity resetctrl is
  generic (
    USE_CLK32 : integer := 1;
    NLANE : integer;
    SIMULATION : integer
  );
  port (
    rstn : in std_logic;
    pwrup : in std_logic;
    clk32 : in std_logic;
    clk125 : in std_logic;
    txplllocked : in std_logic;
    rxfreqlocked : in std_logic;
    rxplllocked : in std_logic;
    pipestatus : in std_logic_vector(NLANE*3-1 downto 0);
    txdrst : out std_logic;
    rxarst : out std_logic;
    rxdrst : out std_logic
  );
end resetctrl;

architecture default of resetctrl is

  signal rxdrst0 : std_logic;
  signal rxdrst1 : std_logic;
  signal rxfrequp : std_logic;
  signal rxfreqlocked1 : std_logic;
  signal rmerr : std_logic; -- rate match FIFO overflow/underflow

begin

  ifclk32 : if (USE_CLK32 = 1) generate

    process (rstn, clk32)
    begin
      if rstn = '0' then
        txdrst <= '0';
        rxarst <= '0';
      elsif rising_edge(clk32) then
        if (pwrup = '0') then
          txdrst <= '1';
          rxarst <= '1';
        elsif (txplllocked = '1') then
          txdrst <= '0';
          rxarst <= '0';
        end if;
      end if;
    end process;
  end generate ifclk32;

  ifnoclk32 : if (USE_CLK32 = 0) generate
    process (rstn)
    begin
      if rstn = '0' then
        txdrst <= '0';
        rxarst <= '0';
      else
        if (pwrup = '0') then
          txdrst <= '1';
          rxarst <= '1';
        elsif (txplllocked = '1') then
          txdrst <= '0';
          rxarst <= '0';
        end if;
      end if;
    end process;
  end generate ifnoclk32;

  rxdrst <= rxdrst0;

  process (txplllocked, clk125)
    variable any_rmerr : std_logic;
  begin
    if (txplllocked = '0') then

      rxdrst0 <= '1';
      rxdrst1 <= '0';
      rxfreqlocked1 <= '0';
      rxfrequp <= '0';
      rmerr <= '0';

    elsif rising_edge(clk125) then

      -- rx_freqlocked status.
      rxfreqlocked1 <= rxfreqlocked;
      if (rxfreqlocked = '1' and rxfreqlocked1 = '0') then
        rxfrequp <= '1'; -- rx_freqlocked for all channels up just now.
      else
        rxfrequp <= '0';
      end if;

      -- rate match FIFO overflow/underflow on any lane.
      any_rmerr := '0';
      for i in 0 to NLANE-1 loop
        if (pipestatus(3*(i+1)-1 downto 3*i) = "101" or
            pipestatus(3*(i+1)-1 downto 3*i) = "110") then
          any_rmerr := '1';
        end if;
      end loop;
      rmerr <= any_rmerr;

      -- during initialization phase, assert rx_digitalreset until
      -- rx_pll_locked is asserted for all channels.
      if rxfreqlocked = '0' then
        if rxplllocked = '1' then
          rxdrst0 <= '0';
        end if;
      else
        -- during normal operation phase, if rx_freqlocked
        -- accidentally go down and up again, or if rate match FIFO
        -- overflow/underflow, assert rx_drst for 2clks.
        if (rxdrst1 = '1') then
          rxdrst0 <= '0';
        elsif (rxfrequp = '1' or rmerr = '1') then
          rxdrst0 <= '1';
        end if;
      end if;

      rxdrst1 <= rxdrst0;

    end if;
  end process;

end default;
-- megafunction wizard: %ALT2GXB%
-- GENERATION: STANDARD
-- VERSION: WM1.0
-- MODULE: alt2gxb

-- ============================================================
-- File Name: alt2gxb_agx.vhd
-- Megafunction Name(s):
-- alt2gxb
--
-- Simulation Library Files(s):
--
-- ============================================================
-- ************************************************************
-- THIS IS A WIZARD-GENERATED FILE. DO NOT EDIT THIS FILE!
--
-- 7.1 Build 178 06/25/2007 SP 1 SJ Full Version
-- ************************************************************


--Copyright (C) 1991-2007 Altera Corporation
--Your use of Altera Corporation's design tools, logic functions 
--and other software and tools, and its AMPP partner logic
--functions, and any output files from any of the foregoing
--(including device programming or simulation files), and any
--associated documentation or information are expressly subject
--to the terms and conditions of the Altera Program License
--Subscription Agreement, Altera MegaCore Function License
--Agreement, or other applicable license agreement, including,
--without limitation, that your use is for the sole purpose of
--programming logic devices manufactured by Altera and sold by
--Altera or its authorized distributors. Please refer to the
--applicable agreement for further details.


LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY alt2gxb_agx_x1 IS
 PORT
 (
  cal_blk_clk : IN STD_LOGIC ;
  fixedclk : IN STD_LOGIC ;
  gxb_powerdown : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipe8b10binvpolarity : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  pll_inclk : IN STD_LOGIC ;
  powerdn : IN STD_LOGIC_VECTOR (1 DOWNTO 0);
  rx_analogreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_coreclk : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_datain : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_coreclk : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_ctrlenable : IN STD_LOGIC_VECTOR (1 DOWNTO 0);
  tx_datain : IN STD_LOGIC_VECTOR (15 DOWNTO 0);
  tx_detectrxloop : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_forcedispcompliance : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_forceelecidle : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipedatavalid : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipeelecidle : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipephydonestatus : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipestatus : OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
  pll_locked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_ctrldetect : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
  rx_dataout : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
  rx_disperr : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
  rx_errdetect : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
  rx_freqlocked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_patterndetect : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
  rx_pll_locked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_syncstatus : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
  coreclkout : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_dataout : OUT STD_LOGIC_VECTOR (0 DOWNTO 0)
 );
END alt2gxb_agx_x1;


ARCHITECTURE SYN OF alt2gxb_agx_x1 IS

 SIGNAL sub_wire0 : STD_LOGIC_VECTOR (1 DOWNTO 0);
 SIGNAL sub_wire1 : STD_LOGIC_VECTOR (1 DOWNTO 0);
 SIGNAL sub_wire2 : STD_LOGIC_VECTOR (1 DOWNTO 0);
 SIGNAL sub_wire3 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire4 : STD_LOGIC_VECTOR (1 DOWNTO 0);
 SIGNAL sub_wire5 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire6 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire7 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire8 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire9 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire10 : STD_LOGIC_VECTOR (2 DOWNTO 0);
 SIGNAL sub_wire11 : STD_LOGIC_VECTOR (1 DOWNTO 0);
 SIGNAL sub_wire12 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire13 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire14 : STD_LOGIC_VECTOR (15 DOWNTO 0);



 COMPONENT alt2gxb
 GENERIC (
  cmu_pll_inclock_period : NATURAL;
  cmu_pll_loop_filter_resistor_control : NATURAL;
  digitalreset_port_width : NATURAL;
  enable_fast_recovery_pci_mode : STRING;
  enable_pll_inclk_alt_drive_rx_cru : STRING;
  enable_pll_inclk_drive_rx_cru : STRING;
  en_local_clk_div_ctrl : STRING;
  equalizer_ctrl_a_setting : NATURAL;
  equalizer_ctrl_b_setting : NATURAL;
  equalizer_ctrl_c_setting : NATURAL;
  equalizer_ctrl_d_setting : NATURAL;
  equalizer_ctrl_v_setting : NATURAL;
  equalizer_dcgain_setting : NATURAL;
  intended_device_family : STRING;
  loopback_mode : STRING;
  lpm_type : STRING;
  number_of_channels : NATURAL;
  operation_mode : STRING;
  preemphasis_ctrl_1stposttap_setting : NATURAL;
  preemphasis_ctrl_2ndposttap_inv_setting : STRING;
  preemphasis_ctrl_2ndposttap_setting : NATURAL;
  preemphasis_ctrl_pretap_inv_setting : STRING;
  preemphasis_ctrl_pretap_setting : NATURAL;
  protocol : STRING;
  receiver_termination : STRING;
  reconfig_dprio_mode : NATURAL;
  reverse_loopback_mode : STRING;
  rx_8b_10b_compatibility_mode : STRING;
  rx_8b_10b_mode : STRING;
  rx_align_pattern : STRING;
  rx_align_pattern_length : NATURAL;
  rx_allow_align_polarity_inversion : STRING;
  rx_allow_pipe_polarity_inversion : STRING;
  rx_bandwidth_mode : NATURAL;
  rx_bitslip_enable : STRING;
  rx_byte_ordering_mode : STRING;
  rx_channel_bonding : STRING;
  rx_channel_width : NATURAL;
  rx_common_mode : STRING;
  rx_cru_pre_divide_by : NATURAL;
  rx_datapath_protocol : STRING;
  rx_data_rate : NATURAL;
  rx_data_rate_remainder : NATURAL;
  rx_disable_auto_idle_insertion : STRING;
  rx_enable_bit_reversal : STRING;
  rx_enable_lock_to_data_sig : STRING;
  rx_enable_lock_to_refclk_sig : STRING;
  rx_enable_self_test_mode : STRING;
  rx_enable_true_complement_match_in_word_align : STRING;
  rx_force_signal_detect : STRING;
  rx_ppmselect : NATURAL;
  rx_rate_match_back_to_back : STRING;
  rx_rate_match_fifo_mode : STRING;
  rx_rate_match_ordered_set_based : STRING;
  rx_rate_match_pattern1 : STRING;
  rx_rate_match_pattern2 : STRING;
  rx_rate_match_pattern_size : NATURAL;
  rx_rate_match_skip_set_based : STRING;
  rx_run_length_enable : STRING;
  rx_signal_detect_threshold : NATURAL;
  rx_use_align_state_machine : STRING;
  rx_use_clkout : STRING;
  rx_use_coreclk : STRING;
  rx_use_deserializer_double_data_mode : STRING;
  rx_use_deskew_fifo : STRING;
  rx_use_double_data_mode : STRING;
  rx_use_pipe8b10binvpolarity : STRING;
  rx_use_rate_match_pattern1_only : STRING;
  transmitter_termination : STRING;
  tx_8b_10b_compatibility_mode : STRING;
  tx_8b_10b_mode : STRING;
  tx_allow_polarity_inversion : STRING;
  tx_analog_power : STRING;
  tx_channel_bonding : STRING;
  tx_channel_width : NATURAL;
  tx_common_mode : STRING;
  tx_data_rate : NATURAL;
  tx_data_rate_remainder : NATURAL;
  tx_enable_bit_reversal : STRING;
  tx_enable_idle_selection : STRING;
  tx_enable_self_test_mode : STRING;
  tx_refclk_divide_by : NATURAL;
  tx_transmit_protocol : STRING;
  tx_use_coreclk : STRING;
  tx_use_double_data_mode : STRING;
  tx_use_serializer_double_data_mode : STRING;
  use_calibration_block : STRING;
  vod_ctrl_setting : NATURAL
 );
 PORT (
   tx_forceelecidle : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_disperr : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   rx_patterndetect : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   rx_coreclk : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_ctrldetect : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   pll_inclk : IN STD_LOGIC ;
   gxb_powerdown : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pipedatavalid : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_datain : IN STD_LOGIC_VECTOR (15 DOWNTO 0);
   rx_errdetect : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   pipephydonestatus : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_pll_locked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_freqlocked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   cal_blk_clk : IN STD_LOGIC ;
   powerdn : IN STD_LOGIC_VECTOR (1 DOWNTO 0);
   tx_dataout : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   fixedclk : IN STD_LOGIC ;
   rx_datain : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pipeelecidle : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_coreclk : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_ctrlenable : IN STD_LOGIC_VECTOR (1 DOWNTO 0);
   pipestatus : OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
   rx_analogreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pipe8b10binvpolarity : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_syncstatus : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   rx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_clkout : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_forcedispcompliance : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pll_locked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_dataout : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
   tx_detectrxloop : IN STD_LOGIC_VECTOR (0 DOWNTO 0)
 );
 END COMPONENT;

BEGIN
 rx_disperr <= sub_wire0(1 DOWNTO 0);
 rx_patterndetect <= sub_wire1(1 DOWNTO 0);
 rx_ctrldetect <= sub_wire2(1 DOWNTO 0);
 pipedatavalid <= sub_wire3(0 DOWNTO 0);
 rx_errdetect <= sub_wire4(1 DOWNTO 0);
 pipephydonestatus <= sub_wire5(0 DOWNTO 0);
 rx_pll_locked <= sub_wire6(0 DOWNTO 0);
 rx_freqlocked <= sub_wire7(0 DOWNTO 0);
 tx_dataout <= sub_wire8(0 DOWNTO 0);
 pipeelecidle <= sub_wire9(0 DOWNTO 0);
 pipestatus <= sub_wire10(2 DOWNTO 0);
 rx_syncstatus <= sub_wire11(1 DOWNTO 0);
 coreclkout <= sub_wire12(0 DOWNTO 0);
 pll_locked <= sub_wire13(0 DOWNTO 0);
 rx_dataout <= sub_wire14(15 DOWNTO 0);

 alt2gxb_component : alt2gxb
 GENERIC MAP (
  cmu_pll_inclock_period => 10000,
  cmu_pll_loop_filter_resistor_control => 3,
  digitalreset_port_width => 1,
  enable_fast_recovery_pci_mode => "false",
  enable_pll_inclk_alt_drive_rx_cru => "true",
  enable_pll_inclk_drive_rx_cru => "true",
  en_local_clk_div_ctrl => "true",
  equalizer_ctrl_a_setting => 0,
  equalizer_ctrl_b_setting => 0,
  equalizer_ctrl_c_setting => 0,
  equalizer_ctrl_d_setting => 0,
  equalizer_ctrl_v_setting => 0,
  equalizer_dcgain_setting => 1,
  intended_device_family => "Arria GX",
  loopback_mode => "none",
  lpm_type => "alt2gxb",
  number_of_channels => 1,
  operation_mode => "duplex",
  preemphasis_ctrl_1stposttap_setting => 5,
  preemphasis_ctrl_2ndposttap_inv_setting => "false",
  preemphasis_ctrl_2ndposttap_setting => 0,
  preemphasis_ctrl_pretap_inv_setting => "false",
  preemphasis_ctrl_pretap_setting => 0,
  protocol => "pipe",
  receiver_termination => "oct_100_ohms",
  reconfig_dprio_mode => 0,
  reverse_loopback_mode => "rplb",
  rx_8b_10b_compatibility_mode => "true",
  rx_8b_10b_mode => "normal",
  rx_align_pattern => "0101111100",
  rx_align_pattern_length => 10,
  rx_allow_align_polarity_inversion => "false",
  rx_allow_pipe_polarity_inversion => "true",
  rx_bandwidth_mode => 1,
  rx_bitslip_enable => "false",
  rx_byte_ordering_mode => "none",
  rx_channel_bonding => "indv",
  rx_channel_width => 16,
  rx_common_mode => "0.9v",
  rx_cru_pre_divide_by => 1,
  rx_datapath_protocol => "pipe",
  rx_data_rate => 2500,
  rx_data_rate_remainder => 0,
  rx_disable_auto_idle_insertion => "false",
  rx_enable_bit_reversal => "false",
  rx_enable_lock_to_data_sig => "false",
  rx_enable_lock_to_refclk_sig => "false",
  rx_enable_self_test_mode => "false",
  rx_enable_true_complement_match_in_word_align => "false",
  rx_force_signal_detect => "false",
  rx_ppmselect => 8,
  rx_rate_match_back_to_back => "false",
  rx_rate_match_fifo_mode => "normal",
  rx_rate_match_ordered_set_based => "false",
  rx_rate_match_pattern1 => "11010000111010000011",
  rx_rate_match_pattern2 => "00101111000101111100",
  rx_rate_match_pattern_size => 20,
  rx_rate_match_skip_set_based => "false",
  rx_run_length_enable => "false",
  rx_signal_detect_threshold => 2,
  rx_use_align_state_machine => "true",
  rx_use_clkout => "false",
  rx_use_coreclk => "true",
  rx_use_deserializer_double_data_mode => "false",
  rx_use_deskew_fifo => "false",
  rx_use_double_data_mode => "true",
  rx_use_pipe8b10binvpolarity => "true",
  rx_use_rate_match_pattern1_only => "false",
  transmitter_termination => "oct_100_ohms",
  tx_8b_10b_compatibility_mode => "true",
  tx_8b_10b_mode => "normal",
  tx_allow_polarity_inversion => "false",
  tx_analog_power => "1.2v",
  tx_channel_bonding => "indv",
  tx_channel_width => 16,
  tx_common_mode => "0.6v",
  tx_data_rate => 2500,
  tx_data_rate_remainder => 0,
  tx_enable_bit_reversal => "false",
  tx_enable_idle_selection => "false",
  tx_enable_self_test_mode => "false",
  tx_refclk_divide_by => 1,
  tx_transmit_protocol => "pipe",
  tx_use_coreclk => "true",
  tx_use_double_data_mode => "true",
  tx_use_serializer_double_data_mode => "false",
  use_calibration_block => "true",
  vod_ctrl_setting => 4
 )
 PORT MAP (
  tx_forceelecidle => tx_forceelecidle,
  rx_coreclk => rx_coreclk,
  pll_inclk => pll_inclk,
  gxb_powerdown => gxb_powerdown,
  tx_datain => tx_datain,
  cal_blk_clk => cal_blk_clk,
  powerdn => powerdn,
  fixedclk => fixedclk,
  rx_datain => rx_datain,
  tx_coreclk => tx_coreclk,
  tx_ctrlenable => tx_ctrlenable,
  rx_analogreset => rx_analogreset,
  pipe8b10binvpolarity => pipe8b10binvpolarity,
  rx_digitalreset => rx_digitalreset,
  tx_digitalreset => tx_digitalreset,
  tx_forcedispcompliance => tx_forcedispcompliance,
  tx_detectrxloop => tx_detectrxloop,
  rx_disperr => sub_wire0,
  rx_patterndetect => sub_wire1,
  rx_ctrldetect => sub_wire2,
  pipedatavalid => sub_wire3,
  rx_errdetect => sub_wire4,
  pipephydonestatus => sub_wire5,
  rx_pll_locked => sub_wire6,
  rx_freqlocked => sub_wire7,
  tx_dataout => sub_wire8,
  pipeelecidle => sub_wire9,
  pipestatus => sub_wire10,
  rx_syncstatus => sub_wire11,
  tx_clkout => sub_wire12,
  pll_locked => sub_wire13,
  rx_dataout => sub_wire14
 );



END SYN;

-- ============================================================
-- CNX file retrieval info
-- ============================================================
-- Retrieval info: PRIVATE: ALT_SIMLIB_GEN STRING "0"
-- Retrieval info: PRIVATE: INTENDED_DEVICE_FAMILY STRING "Arria GX"
-- Retrieval info: PRIVATE: RECONFIG_PROTOCOL STRING "BASIC"
-- Retrieval info: PRIVATE: RECONFIG_SUBPROTOCOL STRING "none"
-- Retrieval info: PRIVATE: RX_ENABLE_DC_COUPLING STRING "false"
-- Retrieval info: PRIVATE: RX_FORCE_SIGNAL_DETECT STRING "false"
-- Retrieval info: PRIVATE: RX_SIGNAL_DETECT_THRESHOLD NUMERIC "2"
-- Retrieval info: PRIVATE: SYNTH_WRAPPER_GEN_POSTFIX STRING "0"
-- Retrieval info: PRIVATE: VOD_CTRL_SETTING NUMERIC "800"
-- Retrieval info: PRIVATE: WIZ_DATA_RATE STRING "2500"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INCLK_FREQ_ARRAY STRING "100.0 125.0 200.0 250.0 400.0 500.0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_A STRING "2000"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_A_UNIT STRING "Mbps"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_B STRING "100.0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_B_UNIT STRING "MHz"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_SELECTION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK0_FREQ STRING "100"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK0_PROTOCOL STRING "PCI Express (PIPE)"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK1_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK1_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK2_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK2_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK3_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK3_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK4_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK4_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK5_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK5_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK6_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK6_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_ENABLE_EQUALIZER_CTRL NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EQUALIZER_CTRL_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EXTERNAL_RX_TERMINATION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EXTERNAL_TX_TERMINATION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_FORCE_DEFAULT_SETTINGS NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_INCLK_FREQ STRING "100"
-- Retrieval info: PRIVATE: WIZ_INCLK_FREQ_ARRAY STRING "100"
-- Retrieval info: PRIVATE: WIZ_INPUT_A STRING "2500"
-- Retrieval info: PRIVATE: WIZ_INPUT_A_UNIT STRING "Mbps"
-- Retrieval info: PRIVATE: WIZ_INPUT_B STRING "100"
-- Retrieval info: PRIVATE: WIZ_INPUT_B_UNIT STRING "MHz"
-- Retrieval info: PRIVATE: WIZ_INPUT_SELECTION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_1STPOSTTAP_SETTING NUMERIC "5"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_2NDPOSTTAP_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_PRETAP_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PROTOCOL STRING "PCI Express (PIPE)"
-- Retrieval info: PRIVATE: WIZ_RX_TERMINATION NUMERIC "100"
-- Retrieval info: PRIVATE: WIZ_RX_VCM STRING "0.85"
-- Retrieval info: PRIVATE: WIZ_SUBPROTOCOL STRING "x1"
-- Retrieval info: PRIVATE: WIZ_TX_TERMINATION NUMERIC "100"
-- Retrieval info: PRIVATE: WIZ_TX_VCM STRING "0.6"
-- Retrieval info: PRIVATE: WIZ_VCCHTX STRING "1.2"
-- Retrieval info: PRIVATE: WIZ_WORD_ALIGN_FLIP_PATTERN STRING "0"
-- Retrieval info: CONSTANT: CMU_PLL_INCLOCK_PERIOD NUMERIC "10000"
-- Retrieval info: CONSTANT: CMU_PLL_LOOP_FILTER_RESISTOR_CONTROL NUMERIC "3"
-- Retrieval info: CONSTANT: DIGITALRESET_PORT_WIDTH NUMERIC "1"
-- Retrieval info: CONSTANT: ENABLE_FAST_RECOVERY_PCI_MODE STRING "false"
-- Retrieval info: CONSTANT: ENABLE_PLL_INCLK_ALT_DRIVE_RX_CRU STRING "true"
-- Retrieval info: CONSTANT: ENABLE_PLL_INCLK_DRIVE_RX_CRU STRING "true"
-- Retrieval info: CONSTANT: EN_LOCAL_CLK_DIV_CTRL STRING "true"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_A_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_B_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_C_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_D_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_V_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_DCGAIN_SETTING NUMERIC "1"
-- Retrieval info: CONSTANT: INTENDED_DEVICE_FAMILY STRING "Arria GX"
-- Retrieval info: CONSTANT: LOOPBACK_MODE STRING "none"
-- Retrieval info: CONSTANT: LPM_TYPE STRING "alt2gxb"
-- Retrieval info: CONSTANT: NUMBER_OF_CHANNELS NUMERIC "1"
-- Retrieval info: CONSTANT: OPERATION_MODE STRING "duplex"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_1STPOSTTAP_SETTING NUMERIC "5"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_2NDPOSTTAP_INV_SETTING STRING "false"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_2NDPOSTTAP_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_PRETAP_INV_SETTING STRING "false"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_PRETAP_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: RECEIVER_TERMINATION STRING "oct_100_ohms"
-- Retrieval info: CONSTANT: RECONFIG_DPRIO_MODE NUMERIC "0"
-- Retrieval info: CONSTANT: REVERSE_LOOPBACK_MODE STRING "rplb"
-- Retrieval info: CONSTANT: RX_8B_10B_COMPATIBILITY_MODE STRING "true"
-- Retrieval info: CONSTANT: RX_8B_10B_MODE STRING "normal"
-- Retrieval info: CONSTANT: RX_ALIGN_PATTERN STRING "0101111100"
-- Retrieval info: CONSTANT: RX_ALIGN_PATTERN_LENGTH NUMERIC "10"
-- Retrieval info: CONSTANT: RX_ALLOW_ALIGN_POLARITY_INVERSION STRING "false"
-- Retrieval info: CONSTANT: RX_ALLOW_PIPE_POLARITY_INVERSION STRING "true"
-- Retrieval info: CONSTANT: RX_BANDWIDTH_MODE NUMERIC "1"
-- Retrieval info: CONSTANT: RX_BITSLIP_ENABLE STRING "false"
-- Retrieval info: CONSTANT: RX_BYTE_ORDERING_MODE STRING "none"
-- Retrieval info: CONSTANT: RX_CHANNEL_BONDING STRING "indv"
-- Retrieval info: CONSTANT: RX_CHANNEL_WIDTH NUMERIC "16"
-- Retrieval info: CONSTANT: RX_COMMON_MODE STRING "0.9v"
-- Retrieval info: CONSTANT: RX_CRU_PRE_DIVIDE_BY NUMERIC "1"
-- Retrieval info: CONSTANT: RX_DATAPATH_PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: RX_DATA_RATE NUMERIC "2500"
-- Retrieval info: CONSTANT: RX_DATA_RATE_REMAINDER NUMERIC "0"
-- Retrieval info: CONSTANT: RX_DISABLE_AUTO_IDLE_INSERTION STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_BIT_REVERSAL STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_LOCK_TO_DATA_SIG STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_LOCK_TO_REFCLK_SIG STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_SELF_TEST_MODE STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_TRUE_COMPLEMENT_MATCH_IN_WORD_ALIGN STRING "false"
-- Retrieval info: CONSTANT: RX_FORCE_SIGNAL_DETECT STRING "false"
-- Retrieval info: CONSTANT: RX_PPMSELECT NUMERIC "8"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_BACK_TO_BACK STRING "false"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_FIFO_MODE STRING "normal"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_ORDERED_SET_BASED STRING "false"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN1 STRING "11010000111010000011"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN2 STRING "00101111000101111100"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN_SIZE NUMERIC "20"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_SKIP_SET_BASED STRING "false"
-- Retrieval info: CONSTANT: RX_RUN_LENGTH_ENABLE STRING "false"
-- Retrieval info: CONSTANT: RX_SIGNAL_DETECT_THRESHOLD NUMERIC "2"
-- Retrieval info: CONSTANT: RX_USE_ALIGN_STATE_MACHINE STRING "true"
-- Retrieval info: CONSTANT: RX_USE_CLKOUT STRING "false"
-- Retrieval info: CONSTANT: RX_USE_CORECLK STRING "true"
-- Retrieval info: CONSTANT: RX_USE_DESERIALIZER_DOUBLE_DATA_MODE STRING "false"
-- Retrieval info: CONSTANT: RX_USE_DESKEW_FIFO STRING "false"
-- Retrieval info: CONSTANT: RX_USE_DOUBLE_DATA_MODE STRING "true"
-- Retrieval info: CONSTANT: RX_USE_PIPE8B10BINVPOLARITY STRING "true"
-- Retrieval info: CONSTANT: RX_USE_RATE_MATCH_PATTERN1_ONLY STRING "false"
-- Retrieval info: CONSTANT: TRANSMITTER_TERMINATION STRING "oct_100_ohms"
-- Retrieval info: CONSTANT: TX_8B_10B_COMPATIBILITY_MODE STRING "true"
-- Retrieval info: CONSTANT: TX_8B_10B_MODE STRING "normal"
-- Retrieval info: CONSTANT: TX_ALLOW_POLARITY_INVERSION STRING "false"
-- Retrieval info: CONSTANT: TX_ANALOG_POWER STRING "1.2v"
-- Retrieval info: CONSTANT: TX_CHANNEL_BONDING STRING "indv"
-- Retrieval info: CONSTANT: TX_CHANNEL_WIDTH NUMERIC "16"
-- Retrieval info: CONSTANT: TX_COMMON_MODE STRING "0.6v"
-- Retrieval info: CONSTANT: TX_DATA_RATE NUMERIC "2500"
-- Retrieval info: CONSTANT: TX_DATA_RATE_REMAINDER NUMERIC "0"
-- Retrieval info: CONSTANT: TX_ENABLE_BIT_REVERSAL STRING "false"
-- Retrieval info: CONSTANT: TX_ENABLE_IDLE_SELECTION STRING "false"
-- Retrieval info: CONSTANT: TX_ENABLE_SELF_TEST_MODE STRING "false"
-- Retrieval info: CONSTANT: TX_REFCLK_DIVIDE_BY NUMERIC "1"
-- Retrieval info: CONSTANT: TX_TRANSMIT_PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: TX_USE_CORECLK STRING "true"
-- Retrieval info: CONSTANT: TX_USE_DOUBLE_DATA_MODE STRING "true"
-- Retrieval info: CONSTANT: TX_USE_SERIALIZER_DOUBLE_DATA_MODE STRING "false"
-- Retrieval info: CONSTANT: USE_CALIBRATION_BLOCK STRING "true"
-- Retrieval info: CONSTANT: VOD_CTRL_SETTING NUMERIC "4"
-- Retrieval info: USED_PORT: cal_blk_clk 0 0 0 0 INPUT NODEFVAL "cal_blk_clk"
-- Retrieval info: USED_PORT: fixedclk 0 0 0 0 INPUT NODEFVAL "fixedclk"
-- Retrieval info: USED_PORT: gxb_powerdown 0 0 1 0 INPUT NODEFVAL "gxb_powerdown[0..0]"
-- Retrieval info: USED_PORT: pipe8b10binvpolarity 0 0 1 0 INPUT NODEFVAL "pipe8b10binvpolarity[0..0]"
-- Retrieval info: USED_PORT: pipedatavalid 0 0 1 0 OUTPUT NODEFVAL "pipedatavalid[0..0]"
-- Retrieval info: USED_PORT: pipeelecidle 0 0 1 0 OUTPUT NODEFVAL "pipeelecidle[0..0]"
-- Retrieval info: USED_PORT: pipephydonestatus 0 0 1 0 OUTPUT NODEFVAL "pipephydonestatus[0..0]"
-- Retrieval info: USED_PORT: pipestatus 0 0 3 0 OUTPUT NODEFVAL "pipestatus[2..0]"
-- Retrieval info: USED_PORT: pll_inclk 0 0 0 0 INPUT NODEFVAL "pll_inclk"
-- Retrieval info: USED_PORT: pll_locked 0 0 1 0 OUTPUT NODEFVAL "pll_locked[0..0]"
-- Retrieval info: USED_PORT: powerdn 0 0 2 0 INPUT NODEFVAL "powerdn[1..0]"
-- Retrieval info: USED_PORT: rx_analogreset 0 0 1 0 INPUT NODEFVAL "rx_analogreset[0..0]"
-- Retrieval info: USED_PORT: rx_coreclk 0 0 1 0 INPUT NODEFVAL "rx_coreclk[0..0]"
-- Retrieval info: USED_PORT: rx_ctrldetect 0 0 2 0 OUTPUT NODEFVAL "rx_ctrldetect[1..0]"
-- Retrieval info: USED_PORT: rx_datain 0 0 1 0 INPUT NODEFVAL "rx_datain[0..0]"
-- Retrieval info: USED_PORT: rx_dataout 0 0 16 0 OUTPUT NODEFVAL "rx_dataout[15..0]"
-- Retrieval info: USED_PORT: rx_digitalreset 0 0 1 0 INPUT NODEFVAL "rx_digitalreset[0..0]"
-- Retrieval info: USED_PORT: rx_disperr 0 0 2 0 OUTPUT NODEFVAL "rx_disperr[1..0]"
-- Retrieval info: USED_PORT: rx_errdetect 0 0 2 0 OUTPUT NODEFVAL "rx_errdetect[1..0]"
-- Retrieval info: USED_PORT: rx_freqlocked 0 0 1 0 OUTPUT NODEFVAL "rx_freqlocked[0..0]"
-- Retrieval info: USED_PORT: rx_patterndetect 0 0 2 0 OUTPUT NODEFVAL "rx_patterndetect[1..0]"
-- Retrieval info: USED_PORT: rx_pll_locked 0 0 1 0 OUTPUT NODEFVAL "rx_pll_locked[0..0]"
-- Retrieval info: USED_PORT: rx_syncstatus 0 0 2 0 OUTPUT NODEFVAL "rx_syncstatus[1..0]"
-- Retrieval info: USED_PORT: tx_clkout 0 0 1 0 OUTPUT NODEFVAL "tx_clkout[0..0]"
-- Retrieval info: USED_PORT: tx_coreclk 0 0 1 0 INPUT NODEFVAL "tx_coreclk[0..0]"
-- Retrieval info: USED_PORT: tx_ctrlenable 0 0 2 0 INPUT NODEFVAL "tx_ctrlenable[1..0]"
-- Retrieval info: USED_PORT: tx_datain 0 0 16 0 INPUT NODEFVAL "tx_datain[15..0]"
-- Retrieval info: USED_PORT: tx_dataout 0 0 1 0 OUTPUT NODEFVAL "tx_dataout[0..0]"
-- Retrieval info: USED_PORT: tx_detectrxloop 0 0 1 0 INPUT NODEFVAL "tx_detectrxloop[0..0]"
-- Retrieval info: USED_PORT: tx_digitalreset 0 0 1 0 INPUT NODEFVAL "tx_digitalreset[0..0]"
-- Retrieval info: USED_PORT: tx_forcedispcompliance 0 0 1 0 INPUT NODEFVAL "tx_forcedispcompliance[0..0]"
-- Retrieval info: USED_PORT: tx_forceelecidle 0 0 1 0 INPUT NODEFVAL "tx_forceelecidle[0..0]"
-- Retrieval info: CONNECT: @fixedclk 0 0 0 0 fixedclk 0 0 0 0
-- Retrieval info: CONNECT: @tx_detectrxloop 0 0 1 0 tx_detectrxloop 0 0 1 0
-- Retrieval info: CONNECT: @tx_forcedispcompliance 0 0 1 0 tx_forcedispcompliance 0 0 1 0
-- Retrieval info: CONNECT: rx_patterndetect 0 0 2 0 @rx_patterndetect 0 0 2 0
-- Retrieval info: CONNECT: pipedatavalid 0 0 1 0 @pipedatavalid 0 0 1 0
-- Retrieval info: CONNECT: @rx_analogreset 0 0 1 0 rx_analogreset 0 0 1 0
-- Retrieval info: CONNECT: @powerdn 0 0 2 0 powerdn 0 0 2 0
-- Retrieval info: CONNECT: pipeelecidle 0 0 1 0 @pipeelecidle 0 0 1 0
-- Retrieval info: CONNECT: rx_ctrldetect 0 0 2 0 @rx_ctrldetect 0 0 2 0
-- Retrieval info: CONNECT: @gxb_powerdown 0 0 1 0 gxb_powerdown 0 0 1 0
-- Retrieval info: CONNECT: rx_dataout 0 0 16 0 @rx_dataout 0 0 16 0
-- Retrieval info: CONNECT: @cal_blk_clk 0 0 0 0 cal_blk_clk 0 0 0 0
-- Retrieval info: CONNECT: pipestatus 0 0 3 0 @pipestatus 0 0 3 0
-- Retrieval info: CONNECT: rx_errdetect 0 0 2 0 @rx_errdetect 0 0 2 0
-- Retrieval info: CONNECT: @tx_coreclk 0 0 1 0 tx_coreclk 0 0 1 0
-- Retrieval info: CONNECT: @tx_digitalreset 0 0 1 0 tx_digitalreset 0 0 1 0
-- Retrieval info: CONNECT: rx_pll_locked 0 0 1 0 @rx_pll_locked 0 0 1 0
-- Retrieval info: CONNECT: rx_syncstatus 0 0 2 0 @rx_syncstatus 0 0 2 0
-- Retrieval info: CONNECT: @pipe8b10binvpolarity 0 0 1 0 pipe8b10binvpolarity 0 0 1 0
-- Retrieval info: CONNECT: @rx_coreclk 0 0 1 0 rx_coreclk 0 0 1 0
-- Retrieval info: CONNECT: pll_locked 0 0 1 0 @pll_locked 0 0 1 0
-- Retrieval info: CONNECT: @rx_digitalreset 0 0 1 0 rx_digitalreset 0 0 1 0
-- Retrieval info: CONNECT: tx_clkout 0 0 1 0 @tx_clkout 0 0 1 0
-- Retrieval info: CONNECT: @pll_inclk 0 0 0 0 pll_inclk 0 0 0 0
-- Retrieval info: CONNECT: @tx_ctrlenable 0 0 2 0 tx_ctrlenable 0 0 2 0
-- Retrieval info: CONNECT: tx_dataout 0 0 1 0 @tx_dataout 0 0 1 0
-- Retrieval info: CONNECT: @tx_datain 0 0 16 0 tx_datain 0 0 16 0
-- Retrieval info: CONNECT: rx_freqlocked 0 0 1 0 @rx_freqlocked 0 0 1 0
-- Retrieval info: CONNECT: @rx_datain 0 0 1 0 rx_datain 0 0 1 0
-- Retrieval info: CONNECT: rx_disperr 0 0 2 0 @rx_disperr 0 0 2 0
-- Retrieval info: CONNECT: pipephydonestatus 0 0 1 0 @pipephydonestatus 0 0 1 0
-- Retrieval info: CONNECT: @tx_forceelecidle 0 0 1 0 tx_forceelecidle 0 0 1 0
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_agx.x1.vhd TRUE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_agx.x1.inc FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_agx.x1.cmp FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_agx.x1.bsf FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_agx.x1_inst.vhd FALSE FALSE
-- megafunction wizard: %ALT2GXB%
-- GENERATION: STANDARD
-- VERSION: WM1.0
-- MODULE: alt2gxb

-- ============================================================
-- File Name: alt2gxb_agx.vhd
-- Megafunction Name(s):
-- alt2gxb
--
-- Simulation Library Files(s):
--
-- ============================================================
-- ************************************************************
-- THIS IS A WIZARD-GENERATED FILE. DO NOT EDIT THIS FILE!
--
-- 7.1 Build 178 06/25/2007 SP 1 SJ Full Version
-- ************************************************************


--Copyright (C) 1991-2007 Altera Corporation
--Your use of Altera Corporation's design tools, logic functions 
--and other software and tools, and its AMPP partner logic
--functions, and any output files from any of the foregoing
--(including device programming or simulation files), and any
--associated documentation or information are expressly subject
--to the terms and conditions of the Altera Program License
--Subscription Agreement, Altera MegaCore Function License
--Agreement, or other applicable license agreement, including,
--without limitation, that your use is for the sole purpose of
--programming logic devices manufactured by Altera and sold by
--Altera or its authorized distributors. Please refer to the
--applicable agreement for further details.


LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY alt2gxb_agx_x4 IS
 PORT
 (
  cal_blk_clk : IN STD_LOGIC ;
  fixedclk : IN STD_LOGIC ;
  gxb_powerdown : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipe8b10binvpolarity : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  pll_inclk : IN STD_LOGIC ;
  powerdn : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_analogreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_coreclk : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  rx_datain : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  rx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_coreclk : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  tx_ctrlenable : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
  tx_datain : IN STD_LOGIC_VECTOR (63 DOWNTO 0);
  tx_detectrxloop : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  tx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_forcedispcompliance : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  tx_forceelecidle : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  coreclkout : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipedatavalid : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
  pipeelecidle : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
  pipephydonestatus : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
  pipestatus : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
  pll_locked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_ctrldetect : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_dataout : OUT STD_LOGIC_VECTOR (63 DOWNTO 0);
  rx_disperr : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_errdetect : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_freqlocked : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
  rx_patterndetect : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_pll_locked : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
  rx_syncstatus : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  tx_dataout : OUT STD_LOGIC_VECTOR (3 DOWNTO 0)
 );
END alt2gxb_agx_x4;


ARCHITECTURE SYN OF alt2gxb_agx_x4 IS

 SIGNAL sub_wire0 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire1 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire2 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire3 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire4 : STD_LOGIC_VECTOR (3 DOWNTO 0);
 SIGNAL sub_wire5 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire6 : STD_LOGIC_VECTOR (3 DOWNTO 0);
 SIGNAL sub_wire7 : STD_LOGIC_VECTOR (3 DOWNTO 0);
 SIGNAL sub_wire8 : STD_LOGIC_VECTOR (3 DOWNTO 0);
 SIGNAL sub_wire9 : STD_LOGIC_VECTOR (3 DOWNTO 0);
 SIGNAL sub_wire10 : STD_LOGIC_VECTOR (3 DOWNTO 0);
 SIGNAL sub_wire11 : STD_LOGIC_VECTOR (11 DOWNTO 0);
 SIGNAL sub_wire12 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire13 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire14 : STD_LOGIC_VECTOR (63 DOWNTO 0);



 COMPONENT alt2gxb
 GENERIC (
  cmu_pll_inclock_period : NATURAL;
  cmu_pll_loop_filter_resistor_control : NATURAL;
  digitalreset_port_width : NATURAL;
  enable_fast_recovery_pci_mode : STRING;
  enable_pll_inclk_alt_drive_rx_cru : STRING;
  enable_pll_inclk_drive_rx_cru : STRING;
  en_local_clk_div_ctrl : STRING;
  equalizer_ctrl_a_setting : NATURAL;
  equalizer_ctrl_b_setting : NATURAL;
  equalizer_ctrl_c_setting : NATURAL;
  equalizer_ctrl_d_setting : NATURAL;
  equalizer_ctrl_v_setting : NATURAL;
  equalizer_dcgain_setting : NATURAL;
  intended_device_family : STRING;
  loopback_mode : STRING;
  lpm_type : STRING;
  number_of_channels : NATURAL;
  operation_mode : STRING;
  preemphasis_ctrl_1stposttap_setting : NATURAL;
  preemphasis_ctrl_2ndposttap_inv_setting : STRING;
  preemphasis_ctrl_2ndposttap_setting : NATURAL;
  preemphasis_ctrl_pretap_inv_setting : STRING;
  preemphasis_ctrl_pretap_setting : NATURAL;
  protocol : STRING;
  receiver_termination : STRING;
  reconfig_dprio_mode : NATURAL;
  reverse_loopback_mode : STRING;
  rx_8b_10b_compatibility_mode : STRING;
  rx_8b_10b_mode : STRING;
  rx_align_pattern : STRING;
  rx_align_pattern_length : NATURAL;
  rx_allow_align_polarity_inversion : STRING;
  rx_allow_pipe_polarity_inversion : STRING;
  rx_bandwidth_mode : NATURAL;
  rx_bitslip_enable : STRING;
  rx_byte_ordering_mode : STRING;
  rx_channel_bonding : STRING;
  rx_channel_width : NATURAL;
  rx_common_mode : STRING;
  rx_cru_pre_divide_by : NATURAL;
  rx_datapath_protocol : STRING;
  rx_data_rate : NATURAL;
  rx_data_rate_remainder : NATURAL;
  rx_disable_auto_idle_insertion : STRING;
  rx_enable_bit_reversal : STRING;
  rx_enable_lock_to_data_sig : STRING;
  rx_enable_lock_to_refclk_sig : STRING;
  rx_enable_self_test_mode : STRING;
  rx_enable_true_complement_match_in_word_align : STRING;
  rx_force_signal_detect : STRING;
  rx_ppmselect : NATURAL;
  rx_rate_match_back_to_back : STRING;
  rx_rate_match_fifo_mode : STRING;
  rx_rate_match_ordered_set_based : STRING;
  rx_rate_match_pattern1 : STRING;
  rx_rate_match_pattern2 : STRING;
  rx_rate_match_pattern_size : NATURAL;
  rx_rate_match_skip_set_based : STRING;
  rx_run_length_enable : STRING;
  rx_signal_detect_threshold : NATURAL;
  rx_use_align_state_machine : STRING;
  rx_use_clkout : STRING;
  rx_use_coreclk : STRING;
  rx_use_deserializer_double_data_mode : STRING;
  rx_use_deskew_fifo : STRING;
  rx_use_double_data_mode : STRING;
  rx_use_pipe8b10binvpolarity : STRING;
  rx_use_rate_match_pattern1_only : STRING;
  transmitter_termination : STRING;
  tx_8b_10b_compatibility_mode : STRING;
  tx_8b_10b_mode : STRING;
  tx_allow_polarity_inversion : STRING;
  tx_analog_power : STRING;
  tx_channel_bonding : STRING;
  tx_channel_width : NATURAL;
  tx_common_mode : STRING;
  tx_data_rate : NATURAL;
  tx_data_rate_remainder : NATURAL;
  tx_enable_bit_reversal : STRING;
  tx_enable_idle_selection : STRING;
  tx_enable_self_test_mode : STRING;
  tx_refclk_divide_by : NATURAL;
  tx_transmit_protocol : STRING;
  tx_use_coreclk : STRING;
  tx_use_double_data_mode : STRING;
  tx_use_serializer_double_data_mode : STRING;
  use_calibration_block : STRING;
  vod_ctrl_setting : NATURAL
 );
 PORT (
   tx_forceelecidle : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
   rx_disperr : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   rx_patterndetect : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   rx_coreclk : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
   coreclkout : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_ctrldetect : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   pll_inclk : IN STD_LOGIC ;
   gxb_powerdown : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pipedatavalid : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
   tx_datain : IN STD_LOGIC_VECTOR (63 DOWNTO 0);
   rx_errdetect : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   pipephydonestatus : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
   rx_pll_locked : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
   rx_freqlocked : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
   cal_blk_clk : IN STD_LOGIC ;
   powerdn : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
   tx_dataout : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
   fixedclk : IN STD_LOGIC ;
   rx_datain : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
   pipeelecidle : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
   tx_coreclk : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
   tx_ctrlenable : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
   pipestatus : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
   rx_analogreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pipe8b10binvpolarity : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
   rx_syncstatus : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   rx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_forcedispcompliance : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
   pll_locked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_dataout : OUT STD_LOGIC_VECTOR (63 DOWNTO 0);
   tx_detectrxloop : IN STD_LOGIC_VECTOR (3 DOWNTO 0)
 );
 END COMPONENT;

BEGIN
 rx_disperr <= sub_wire0(7 DOWNTO 0);
 rx_patterndetect <= sub_wire1(7 DOWNTO 0);
 coreclkout <= sub_wire2(0 DOWNTO 0);
 rx_ctrldetect <= sub_wire3(7 DOWNTO 0);
 pipedatavalid <= sub_wire4(3 DOWNTO 0);
 rx_errdetect <= sub_wire5(7 DOWNTO 0);
 pipephydonestatus <= sub_wire6(3 DOWNTO 0);
 rx_pll_locked <= sub_wire7(3 DOWNTO 0);
 rx_freqlocked <= sub_wire8(3 DOWNTO 0);
 tx_dataout <= sub_wire9(3 DOWNTO 0);
 pipeelecidle <= sub_wire10(3 DOWNTO 0);
 pipestatus <= sub_wire11(11 DOWNTO 0);
 rx_syncstatus <= sub_wire12(7 DOWNTO 0);
 pll_locked <= sub_wire13(0 DOWNTO 0);
 rx_dataout <= sub_wire14(63 DOWNTO 0);

 alt2gxb_component : alt2gxb
 GENERIC MAP (
  cmu_pll_inclock_period => 10000,
  cmu_pll_loop_filter_resistor_control => 3,
  digitalreset_port_width => 1,
  enable_fast_recovery_pci_mode => "false",
  enable_pll_inclk_alt_drive_rx_cru => "true",
  enable_pll_inclk_drive_rx_cru => "true",
  en_local_clk_div_ctrl => "true",
  equalizer_ctrl_a_setting => 0,
  equalizer_ctrl_b_setting => 0,
  equalizer_ctrl_c_setting => 0,
  equalizer_ctrl_d_setting => 0,
  equalizer_ctrl_v_setting => 0,
  equalizer_dcgain_setting => 1,
  intended_device_family => "Arria GX",
  loopback_mode => "none",
  lpm_type => "alt2gxb",
  number_of_channels => 4,
  operation_mode => "duplex",
  preemphasis_ctrl_1stposttap_setting => 5,
  preemphasis_ctrl_2ndposttap_inv_setting => "false",
  preemphasis_ctrl_2ndposttap_setting => 0,
  preemphasis_ctrl_pretap_inv_setting => "false",
  preemphasis_ctrl_pretap_setting => 0,
  protocol => "pipe",
  receiver_termination => "oct_100_ohms",
  reconfig_dprio_mode => 0,
  reverse_loopback_mode => "rplb",
  rx_8b_10b_compatibility_mode => "true",
  rx_8b_10b_mode => "normal",
  rx_align_pattern => "0101111100",
  rx_align_pattern_length => 10,
  rx_allow_align_polarity_inversion => "false",
  rx_allow_pipe_polarity_inversion => "true",
  rx_bandwidth_mode => 1,
  rx_bitslip_enable => "false",
  rx_byte_ordering_mode => "none",
  rx_channel_bonding => "x4",
  rx_channel_width => 16,
  rx_common_mode => "0.9v",
  rx_cru_pre_divide_by => 1,
  rx_datapath_protocol => "pipe",
  rx_data_rate => 2500,
  rx_data_rate_remainder => 0,
  rx_disable_auto_idle_insertion => "false",
  rx_enable_bit_reversal => "false",
  rx_enable_lock_to_data_sig => "false",
  rx_enable_lock_to_refclk_sig => "false",
  rx_enable_self_test_mode => "false",
  rx_enable_true_complement_match_in_word_align => "false",
  rx_force_signal_detect => "false",
  rx_ppmselect => 8,
  rx_rate_match_back_to_back => "false",
  rx_rate_match_fifo_mode => "normal",
  rx_rate_match_ordered_set_based => "false",
  rx_rate_match_pattern1 => "11010000111010000011",
  rx_rate_match_pattern2 => "00101111000101111100",
  rx_rate_match_pattern_size => 20,
  rx_rate_match_skip_set_based => "false",
  rx_run_length_enable => "false",
  rx_signal_detect_threshold => 2,
  rx_use_align_state_machine => "true",
  rx_use_clkout => "false",
  rx_use_coreclk => "true",
  rx_use_deserializer_double_data_mode => "false",
  rx_use_deskew_fifo => "false",
  rx_use_double_data_mode => "true",
  rx_use_pipe8b10binvpolarity => "true",
  rx_use_rate_match_pattern1_only => "false",
  transmitter_termination => "oct_100_ohms",
  tx_8b_10b_compatibility_mode => "true",
  tx_8b_10b_mode => "normal",
  tx_allow_polarity_inversion => "false",
  tx_analog_power => "1.2v",
  tx_channel_bonding => "x4",
  tx_channel_width => 16,
  tx_common_mode => "0.6v",
  tx_data_rate => 2500,
  tx_data_rate_remainder => 0,
  tx_enable_bit_reversal => "false",
  tx_enable_idle_selection => "false",
  tx_enable_self_test_mode => "false",
  tx_refclk_divide_by => 1,
  tx_transmit_protocol => "pipe",
  tx_use_coreclk => "true",
  tx_use_double_data_mode => "true",
  tx_use_serializer_double_data_mode => "false",
  use_calibration_block => "true",
  vod_ctrl_setting => 4
 )
 PORT MAP (
  tx_forceelecidle => tx_forceelecidle,
  rx_coreclk => rx_coreclk,
  pll_inclk => pll_inclk,
  gxb_powerdown => gxb_powerdown,
  tx_datain => tx_datain,
  cal_blk_clk => cal_blk_clk,
  powerdn => powerdn,
  fixedclk => fixedclk,
  rx_datain => rx_datain,
  tx_coreclk => tx_coreclk,
  tx_ctrlenable => tx_ctrlenable,
  rx_analogreset => rx_analogreset,
  pipe8b10binvpolarity => pipe8b10binvpolarity,
  rx_digitalreset => rx_digitalreset,
  tx_digitalreset => tx_digitalreset,
  tx_forcedispcompliance => tx_forcedispcompliance,
  tx_detectrxloop => tx_detectrxloop,
  rx_disperr => sub_wire0,
  rx_patterndetect => sub_wire1,
  coreclkout => sub_wire2,
  rx_ctrldetect => sub_wire3,
  pipedatavalid => sub_wire4,
  rx_errdetect => sub_wire5,
  pipephydonestatus => sub_wire6,
  rx_pll_locked => sub_wire7,
  rx_freqlocked => sub_wire8,
  tx_dataout => sub_wire9,
  pipeelecidle => sub_wire10,
  pipestatus => sub_wire11,
  rx_syncstatus => sub_wire12,
  pll_locked => sub_wire13,
  rx_dataout => sub_wire14
 );



END SYN;

-- ============================================================
-- CNX file retrieval info
-- ============================================================
-- Retrieval info: PRIVATE: ALT_SIMLIB_GEN STRING "0"
-- Retrieval info: PRIVATE: INTENDED_DEVICE_FAMILY STRING "Arria GX"
-- Retrieval info: PRIVATE: RECONFIG_PROTOCOL STRING "BASIC"
-- Retrieval info: PRIVATE: RECONFIG_SUBPROTOCOL STRING "none"
-- Retrieval info: PRIVATE: RX_ENABLE_DC_COUPLING STRING "false"
-- Retrieval info: PRIVATE: RX_FORCE_SIGNAL_DETECT STRING "false"
-- Retrieval info: PRIVATE: RX_SIGNAL_DETECT_THRESHOLD NUMERIC "2"
-- Retrieval info: PRIVATE: SYNTH_WRAPPER_GEN_POSTFIX STRING "0"
-- Retrieval info: PRIVATE: VOD_CTRL_SETTING NUMERIC "800"
-- Retrieval info: PRIVATE: WIZ_DATA_RATE STRING "2500"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INCLK_FREQ_ARRAY STRING "100.0 125.0 200.0 250.0 400.0 500.0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_A STRING "2000"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_A_UNIT STRING "Mbps"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_B STRING "100.0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_B_UNIT STRING "MHz"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_SELECTION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK0_FREQ STRING "100"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK0_PROTOCOL STRING "PCI Express (PIPE)"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK1_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK1_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK2_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK2_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK3_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK3_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK4_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK4_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK5_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK5_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK6_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK6_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_ENABLE_EQUALIZER_CTRL NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EQUALIZER_CTRL_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EXTERNAL_RX_TERMINATION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EXTERNAL_TX_TERMINATION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_FORCE_DEFAULT_SETTINGS NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_INCLK_FREQ STRING "100"
-- Retrieval info: PRIVATE: WIZ_INCLK_FREQ_ARRAY STRING "100"
-- Retrieval info: PRIVATE: WIZ_INPUT_A STRING "2500"
-- Retrieval info: PRIVATE: WIZ_INPUT_A_UNIT STRING "Mbps"
-- Retrieval info: PRIVATE: WIZ_INPUT_B STRING "100"
-- Retrieval info: PRIVATE: WIZ_INPUT_B_UNIT STRING "MHz"
-- Retrieval info: PRIVATE: WIZ_INPUT_SELECTION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_1STPOSTTAP_SETTING NUMERIC "5"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_2NDPOSTTAP_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_PRETAP_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PROTOCOL STRING "PCI Express (PIPE)"
-- Retrieval info: PRIVATE: WIZ_RX_TERMINATION NUMERIC "100"
-- Retrieval info: PRIVATE: WIZ_RX_VCM STRING "0.85"
-- Retrieval info: PRIVATE: WIZ_SUBPROTOCOL STRING "x4"
-- Retrieval info: PRIVATE: WIZ_TX_TERMINATION NUMERIC "100"
-- Retrieval info: PRIVATE: WIZ_TX_VCM STRING "0.6"
-- Retrieval info: PRIVATE: WIZ_VCCHTX STRING "1.2"
-- Retrieval info: PRIVATE: WIZ_WORD_ALIGN_FLIP_PATTERN STRING "0"
-- Retrieval info: CONSTANT: CMU_PLL_INCLOCK_PERIOD NUMERIC "10000"
-- Retrieval info: CONSTANT: CMU_PLL_LOOP_FILTER_RESISTOR_CONTROL NUMERIC "3"
-- Retrieval info: CONSTANT: DIGITALRESET_PORT_WIDTH NUMERIC "1"
-- Retrieval info: CONSTANT: ENABLE_FAST_RECOVERY_PCI_MODE STRING "false"
-- Retrieval info: CONSTANT: ENABLE_PLL_INCLK_ALT_DRIVE_RX_CRU STRING "true"
-- Retrieval info: CONSTANT: ENABLE_PLL_INCLK_DRIVE_RX_CRU STRING "true"
-- Retrieval info: CONSTANT: EN_LOCAL_CLK_DIV_CTRL STRING "true"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_A_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_B_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_C_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_D_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_V_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_DCGAIN_SETTING NUMERIC "1"
-- Retrieval info: CONSTANT: INTENDED_DEVICE_FAMILY STRING "Arria GX"
-- Retrieval info: CONSTANT: LOOPBACK_MODE STRING "none"
-- Retrieval info: CONSTANT: LPM_TYPE STRING "alt2gxb"
-- Retrieval info: CONSTANT: NUMBER_OF_CHANNELS NUMERIC "4"
-- Retrieval info: CONSTANT: OPERATION_MODE STRING "duplex"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_1STPOSTTAP_SETTING NUMERIC "5"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_2NDPOSTTAP_INV_SETTING STRING "false"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_2NDPOSTTAP_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_PRETAP_INV_SETTING STRING "false"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_PRETAP_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: RECEIVER_TERMINATION STRING "oct_100_ohms"
-- Retrieval info: CONSTANT: RECONFIG_DPRIO_MODE NUMERIC "0"
-- Retrieval info: CONSTANT: REVERSE_LOOPBACK_MODE STRING "rplb"
-- Retrieval info: CONSTANT: RX_8B_10B_COMPATIBILITY_MODE STRING "true"
-- Retrieval info: CONSTANT: RX_8B_10B_MODE STRING "normal"
-- Retrieval info: CONSTANT: RX_ALIGN_PATTERN STRING "0101111100"
-- Retrieval info: CONSTANT: RX_ALIGN_PATTERN_LENGTH NUMERIC "10"
-- Retrieval info: CONSTANT: RX_ALLOW_ALIGN_POLARITY_INVERSION STRING "false"
-- Retrieval info: CONSTANT: RX_ALLOW_PIPE_POLARITY_INVERSION STRING "true"
-- Retrieval info: CONSTANT: RX_BANDWIDTH_MODE NUMERIC "1"
-- Retrieval info: CONSTANT: RX_BITSLIP_ENABLE STRING "false"
-- Retrieval info: CONSTANT: RX_BYTE_ORDERING_MODE STRING "none"
-- Retrieval info: CONSTANT: RX_CHANNEL_BONDING STRING "x4"
-- Retrieval info: CONSTANT: RX_CHANNEL_WIDTH NUMERIC "16"
-- Retrieval info: CONSTANT: RX_COMMON_MODE STRING "0.9v"
-- Retrieval info: CONSTANT: RX_CRU_PRE_DIVIDE_BY NUMERIC "1"
-- Retrieval info: CONSTANT: RX_DATAPATH_PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: RX_DATA_RATE NUMERIC "2500"
-- Retrieval info: CONSTANT: RX_DATA_RATE_REMAINDER NUMERIC "0"
-- Retrieval info: CONSTANT: RX_DISABLE_AUTO_IDLE_INSERTION STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_BIT_REVERSAL STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_LOCK_TO_DATA_SIG STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_LOCK_TO_REFCLK_SIG STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_SELF_TEST_MODE STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_TRUE_COMPLEMENT_MATCH_IN_WORD_ALIGN STRING "false"
-- Retrieval info: CONSTANT: RX_FORCE_SIGNAL_DETECT STRING "false"
-- Retrieval info: CONSTANT: RX_PPMSELECT NUMERIC "8"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_BACK_TO_BACK STRING "false"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_FIFO_MODE STRING "normal"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_ORDERED_SET_BASED STRING "false"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN1 STRING "11010000111010000011"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN2 STRING "00101111000101111100"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN_SIZE NUMERIC "20"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_SKIP_SET_BASED STRING "false"
-- Retrieval info: CONSTANT: RX_RUN_LENGTH_ENABLE STRING "false"
-- Retrieval info: CONSTANT: RX_SIGNAL_DETECT_THRESHOLD NUMERIC "2"
-- Retrieval info: CONSTANT: RX_USE_ALIGN_STATE_MACHINE STRING "true"
-- Retrieval info: CONSTANT: RX_USE_CLKOUT STRING "false"
-- Retrieval info: CONSTANT: RX_USE_CORECLK STRING "true"
-- Retrieval info: CONSTANT: RX_USE_DESERIALIZER_DOUBLE_DATA_MODE STRING "false"
-- Retrieval info: CONSTANT: RX_USE_DESKEW_FIFO STRING "false"
-- Retrieval info: CONSTANT: RX_USE_DOUBLE_DATA_MODE STRING "true"
-- Retrieval info: CONSTANT: RX_USE_PIPE8B10BINVPOLARITY STRING "true"
-- Retrieval info: CONSTANT: RX_USE_RATE_MATCH_PATTERN1_ONLY STRING "false"
-- Retrieval info: CONSTANT: TRANSMITTER_TERMINATION STRING "oct_100_ohms"
-- Retrieval info: CONSTANT: TX_8B_10B_COMPATIBILITY_MODE STRING "true"
-- Retrieval info: CONSTANT: TX_8B_10B_MODE STRING "normal"
-- Retrieval info: CONSTANT: TX_ALLOW_POLARITY_INVERSION STRING "false"
-- Retrieval info: CONSTANT: TX_ANALOG_POWER STRING "1.2v"
-- Retrieval info: CONSTANT: TX_CHANNEL_BONDING STRING "x4"
-- Retrieval info: CONSTANT: TX_CHANNEL_WIDTH NUMERIC "16"
-- Retrieval info: CONSTANT: TX_COMMON_MODE STRING "0.6v"
-- Retrieval info: CONSTANT: TX_DATA_RATE NUMERIC "2500"
-- Retrieval info: CONSTANT: TX_DATA_RATE_REMAINDER NUMERIC "0"
-- Retrieval info: CONSTANT: TX_ENABLE_BIT_REVERSAL STRING "false"
-- Retrieval info: CONSTANT: TX_ENABLE_IDLE_SELECTION STRING "false"
-- Retrieval info: CONSTANT: TX_ENABLE_SELF_TEST_MODE STRING "false"
-- Retrieval info: CONSTANT: TX_REFCLK_DIVIDE_BY NUMERIC "1"
-- Retrieval info: CONSTANT: TX_TRANSMIT_PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: TX_USE_CORECLK STRING "true"
-- Retrieval info: CONSTANT: TX_USE_DOUBLE_DATA_MODE STRING "true"
-- Retrieval info: CONSTANT: TX_USE_SERIALIZER_DOUBLE_DATA_MODE STRING "false"
-- Retrieval info: CONSTANT: USE_CALIBRATION_BLOCK STRING "true"
-- Retrieval info: CONSTANT: VOD_CTRL_SETTING NUMERIC "4"
-- Retrieval info: USED_PORT: cal_blk_clk 0 0 0 0 INPUT NODEFVAL "cal_blk_clk"
-- Retrieval info: USED_PORT: coreclkout 0 0 1 0 OUTPUT NODEFVAL "coreclkout[0..0]"
-- Retrieval info: USED_PORT: fixedclk 0 0 0 0 INPUT NODEFVAL "fixedclk"
-- Retrieval info: USED_PORT: gxb_powerdown 0 0 1 0 INPUT NODEFVAL "gxb_powerdown[0..0]"
-- Retrieval info: USED_PORT: pipe8b10binvpolarity 0 0 4 0 INPUT NODEFVAL "pipe8b10binvpolarity[3..0]"
-- Retrieval info: USED_PORT: pipedatavalid 0 0 4 0 OUTPUT NODEFVAL "pipedatavalid[3..0]"
-- Retrieval info: USED_PORT: pipeelecidle 0 0 4 0 OUTPUT NODEFVAL "pipeelecidle[3..0]"
-- Retrieval info: USED_PORT: pipephydonestatus 0 0 4 0 OUTPUT NODEFVAL "pipephydonestatus[3..0]"
-- Retrieval info: USED_PORT: pipestatus 0 0 12 0 OUTPUT NODEFVAL "pipestatus[11..0]"
-- Retrieval info: USED_PORT: pll_inclk 0 0 0 0 INPUT NODEFVAL "pll_inclk"
-- Retrieval info: USED_PORT: pll_locked 0 0 1 0 OUTPUT NODEFVAL "pll_locked[0..0]"
-- Retrieval info: USED_PORT: powerdn 0 0 8 0 INPUT NODEFVAL "powerdn[7..0]"
-- Retrieval info: USED_PORT: rx_analogreset 0 0 1 0 INPUT NODEFVAL "rx_analogreset[0..0]"
-- Retrieval info: USED_PORT: rx_coreclk 0 0 4 0 INPUT NODEFVAL "rx_coreclk[3..0]"
-- Retrieval info: USED_PORT: rx_ctrldetect 0 0 8 0 OUTPUT NODEFVAL "rx_ctrldetect[7..0]"
-- Retrieval info: USED_PORT: rx_datain 0 0 4 0 INPUT NODEFVAL "rx_datain[3..0]"
-- Retrieval info: USED_PORT: rx_dataout 0 0 64 0 OUTPUT NODEFVAL "rx_dataout[63..0]"
-- Retrieval info: USED_PORT: rx_digitalreset 0 0 1 0 INPUT NODEFVAL "rx_digitalreset[0..0]"
-- Retrieval info: USED_PORT: rx_disperr 0 0 8 0 OUTPUT NODEFVAL "rx_disperr[7..0]"
-- Retrieval info: USED_PORT: rx_errdetect 0 0 8 0 OUTPUT NODEFVAL "rx_errdetect[7..0]"
-- Retrieval info: USED_PORT: rx_freqlocked 0 0 4 0 OUTPUT NODEFVAL "rx_freqlocked[3..0]"
-- Retrieval info: USED_PORT: rx_patterndetect 0 0 8 0 OUTPUT NODEFVAL "rx_patterndetect[7..0]"
-- Retrieval info: USED_PORT: rx_pll_locked 0 0 4 0 OUTPUT NODEFVAL "rx_pll_locked[3..0]"
-- Retrieval info: USED_PORT: rx_syncstatus 0 0 8 0 OUTPUT NODEFVAL "rx_syncstatus[7..0]"
-- Retrieval info: USED_PORT: tx_coreclk 0 0 4 0 INPUT NODEFVAL "tx_coreclk[3..0]"
-- Retrieval info: USED_PORT: tx_ctrlenable 0 0 8 0 INPUT NODEFVAL "tx_ctrlenable[7..0]"
-- Retrieval info: USED_PORT: tx_datain 0 0 64 0 INPUT NODEFVAL "tx_datain[63..0]"
-- Retrieval info: USED_PORT: tx_dataout 0 0 4 0 OUTPUT NODEFVAL "tx_dataout[3..0]"
-- Retrieval info: USED_PORT: tx_detectrxloop 0 0 4 0 INPUT NODEFVAL "tx_detectrxloop[3..0]"
-- Retrieval info: USED_PORT: tx_digitalreset 0 0 1 0 INPUT NODEFVAL "tx_digitalreset[0..0]"
-- Retrieval info: USED_PORT: tx_forcedispcompliance 0 0 4 0 INPUT NODEFVAL "tx_forcedispcompliance[3..0]"
-- Retrieval info: USED_PORT: tx_forceelecidle 0 0 4 0 INPUT NODEFVAL "tx_forceelecidle[3..0]"
-- Retrieval info: CONNECT: @fixedclk 0 0 0 0 fixedclk 0 0 0 0
-- Retrieval info: CONNECT: @tx_detectrxloop 0 0 4 0 tx_detectrxloop 0 0 4 0
-- Retrieval info: CONNECT: @tx_forcedispcompliance 0 0 4 0 tx_forcedispcompliance 0 0 4 0
-- Retrieval info: CONNECT: rx_patterndetect 0 0 8 0 @rx_patterndetect 0 0 8 0
-- Retrieval info: CONNECT: pipedatavalid 0 0 4 0 @pipedatavalid 0 0 4 0
-- Retrieval info: CONNECT: @rx_analogreset 0 0 1 0 rx_analogreset 0 0 1 0
-- Retrieval info: CONNECT: @powerdn 0 0 8 0 powerdn 0 0 8 0
-- Retrieval info: CONNECT: pipeelecidle 0 0 4 0 @pipeelecidle 0 0 4 0
-- Retrieval info: CONNECT: rx_ctrldetect 0 0 8 0 @rx_ctrldetect 0 0 8 0
-- Retrieval info: CONNECT: @gxb_powerdown 0 0 1 0 gxb_powerdown 0 0 1 0
-- Retrieval info: CONNECT: rx_dataout 0 0 64 0 @rx_dataout 0 0 64 0
-- Retrieval info: CONNECT: @cal_blk_clk 0 0 0 0 cal_blk_clk 0 0 0 0
-- Retrieval info: CONNECT: pipestatus 0 0 12 0 @pipestatus 0 0 12 0
-- Retrieval info: CONNECT: rx_errdetect 0 0 8 0 @rx_errdetect 0 0 8 0
-- Retrieval info: CONNECT: @tx_coreclk 0 0 4 0 tx_coreclk 0 0 4 0
-- Retrieval info: CONNECT: @tx_digitalreset 0 0 1 0 tx_digitalreset 0 0 1 0
-- Retrieval info: CONNECT: rx_pll_locked 0 0 4 0 @rx_pll_locked 0 0 4 0
-- Retrieval info: CONNECT: coreclkout 0 0 1 0 @coreclkout 0 0 1 0
-- Retrieval info: CONNECT: rx_syncstatus 0 0 8 0 @rx_syncstatus 0 0 8 0
-- Retrieval info: CONNECT: @pipe8b10binvpolarity 0 0 4 0 pipe8b10binvpolarity 0 0 4 0
-- Retrieval info: CONNECT: @rx_coreclk 0 0 4 0 rx_coreclk 0 0 4 0
-- Retrieval info: CONNECT: pll_locked 0 0 1 0 @pll_locked 0 0 1 0
-- Retrieval info: CONNECT: @rx_digitalreset 0 0 1 0 rx_digitalreset 0 0 1 0
-- Retrieval info: CONNECT: @pll_inclk 0 0 0 0 pll_inclk 0 0 0 0
-- Retrieval info: CONNECT: @tx_ctrlenable 0 0 8 0 tx_ctrlenable 0 0 8 0
-- Retrieval info: CONNECT: tx_dataout 0 0 4 0 @tx_dataout 0 0 4 0
-- Retrieval info: CONNECT: @tx_datain 0 0 64 0 tx_datain 0 0 64 0
-- Retrieval info: CONNECT: rx_freqlocked 0 0 4 0 @rx_freqlocked 0 0 4 0
-- Retrieval info: CONNECT: @rx_datain 0 0 4 0 rx_datain 0 0 4 0
-- Retrieval info: CONNECT: rx_disperr 0 0 8 0 @rx_disperr 0 0 8 0
-- Retrieval info: CONNECT: pipephydonestatus 0 0 4 0 @pipephydonestatus 0 0 4 0
-- Retrieval info: CONNECT: @tx_forceelecidle 0 0 4 0 tx_forceelecidle 0 0 4 0
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_agx.x4.vhd TRUE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_agx.x4.inc FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_agx.x4.cmp FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_agx.x4.bsf FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_agx.x4_inst.vhd FALSE FALSE
library ieee;
use ieee.std_logic_1164.all;

entity alt2gxb_agx_2x4 is
  port (
    cal_blk_clk : in std_logic ;
    fixedclk : in std_logic ;
    gxb_powerdown : in std_logic_vector (0 downto 0);
    pipe8b10binvpolarity : in std_logic_vector (7 downto 0);
    pll_inclk : in std_logic ;
    powerdn : in std_logic_vector (15 downto 0);
    rx_analogreset : in std_logic_vector (0 downto 0);
    rx_coreclk : in std_logic_vector (7 downto 0);
    rx_datain : in std_logic_vector (7 downto 0);
    rx_digitalreset : in std_logic_vector (0 downto 0);
    tx_coreclk : in std_logic_vector (7 downto 0);
    tx_ctrlenable : in std_logic_vector (15 downto 0);
    tx_datain : in std_logic_vector (127 downto 0);
    tx_detectrxloop : in std_logic_vector (7 downto 0);
    tx_digitalreset : in std_logic_vector (0 downto 0);
    tx_forcedispcompliance : in std_logic_vector (7 downto 0);
    tx_forceelecidle : in std_logic_vector (7 downto 0);
    coreclkout : out std_logic_vector (1 downto 0);
    pipedatavalid : out std_logic_vector (7 downto 0);
    pipeelecidle : out std_logic_vector (7 downto 0);
    pipephydonestatus : out std_logic_vector (7 downto 0);
    pipestatus : out std_logic_vector (23 downto 0);
    pll_locked : out std_logic_vector (1 downto 0);
    rx_ctrldetect : out std_logic_vector (15 downto 0);
    rx_dataout : out std_logic_vector (127 downto 0);
    rx_disperr : out std_logic_vector (15 downto 0);
    rx_errdetect : out std_logic_vector (15 downto 0);
    rx_freqlocked : out std_logic_vector (7 downto 0);
    rx_patterndetect : out std_logic_vector (15 downto 0);
    rx_pll_locked : out std_logic_vector (7 downto 0);
    rx_syncstatus : out std_logic_vector (15 downto 0);
    tx_dataout : out std_logic_vector (7 downto 0)
  );
end alt2gxb_agx_2x4;


architecture syn of alt2gxb_agx_2x4 is

  component myalt2gxb_agx_x4
    port (
      cal_blk_clk : in std_logic;
      fixedclk : in std_logic;
      pll_inclk : in std_logic;
      powerdn : in std_logic_vector(4*2-1 downto 0);

      pll_locked : out std_logic_vector((4-1)/4 downto 0);
      gxb_powerdown : in std_logic_vector(0 downto 0);
      tx_digitalreset : in std_logic_vector(0 downto 0);
      rx_analogreset : in std_logic_vector(0 downto 0);
      rx_digitalreset : in std_logic_vector(0 downto 0);
      rx_pll_locked : out std_logic_vector(4-1 downto 0);
      rx_freqlocked : out std_logic_vector(4-1 downto 0);

      coreclkout : out std_logic_vector((4-1)/4 downto 0);
-- tx_clkout : out std_logic_vector(0 downto 0); -- x1 only

      tx_coreclk : in std_logic_vector(4-1 downto 0);
      rx_coreclk : in std_logic_vector(4-1 downto 0);

      tx_dataout : out std_logic_vector(4-1 downto 0);
      tx_datain : in std_logic_vector(4*16-1 downto 0);
      tx_ctrlenable : in std_logic_vector(4*2-1 downto 0);
      tx_forceelecidle : in std_logic_vector(4-1 downto 0);
      tx_detectrxloop : in std_logic_vector(4-1 downto 0);
      tx_forcedispcompliance : in std_logic_vector(4-1 downto 0);
      pipe8b10binvpolarity : in std_logic_vector(4-1 downto 0);

      rx_datain : in std_logic_vector(4-1 downto 0);
      rx_dataout : out std_logic_vector(4*16-1 downto 0);
      rx_ctrldetect : out std_logic_vector(4*2-1 downto 0);
      rx_patterndetect : out std_logic_vector(4*2-1 downto 0);
      rx_disperr : out std_logic_vector(4*2-1 downto 0);
      rx_errdetect : out std_logic_vector(4*2-1 downto 0);
      rx_syncstatus : out std_logic_vector(4*2-1 downto 0);

      pipedatavalid : out std_logic_vector(4-1 downto 0);
      pipeelecidle : out std_logic_vector(4-1 downto 0);
      pipephydonestatus : out std_logic_vector(4-1 downto 0);
      pipestatus : out std_logic_vector(4*3-1 downto 0)
    );
  end component;

begin

  --
  -- combine two alt2gxb (x4 PIPE, bonded mode) to form x8 PIPE.
  --
  x4gen : for i in 0 to 1 generate
    x4 : myalt2gxb_agx_x4
      port map (
        cal_blk_clk => cal_blk_clk,
        fixedclk => fixedclk,
        pll_inclk => pll_inclk,
        powerdn => powerdn(4*2*(i+1)-1 downto 4*2*i),

        pll_locked => pll_locked(i downto i),
        gxb_powerdown => gxb_powerdown,
        tx_digitalreset => tx_digitalreset,
        rx_analogreset => rx_analogreset,
        rx_digitalreset => rx_digitalreset,
        rx_pll_locked => rx_pll_locked(4*(i+1)-1 downto 4*i),
        rx_freqlocked => rx_freqlocked(4*(i+1)-1 downto 4*i),

        -- in bonded mode, one clk per GXB quad
        -- is provided by each central clock divider block.
        coreclkout => coreclkout(i downto i),
        rx_coreclk => rx_coreclk(4*(i+1)-1 downto 4*i),
        tx_coreclk => tx_coreclk(4*(i+1)-1 downto 4*i),
        tx_dataout => tx_dataout(4*(i+1)-1 downto 4*i),
        tx_datain => tx_datain(4*16*(i+1)-1 downto 4*16*i),
        tx_ctrlenable => tx_ctrlenable(4*2*(i+1)-1 downto 4*2*i),
        tx_forceelecidle => tx_forceelecidle(4*(i+1)-1 downto 4*i),
        tx_detectrxloop => tx_detectrxloop(4*(i+1)-1 downto 4*i),
        tx_forcedispcompliance => tx_forcedispcompliance(4*(i+1)-1 downto 4*i),

        rx_datain => rx_datain(4*(i+1)-1 downto 4*i),
        rx_dataout => rx_dataout(4*16*(i+1)-1 downto 4*16*i),
        rx_ctrldetect => rx_ctrldetect(4*2*(i+1)-1 downto 4*2*i),
        rx_patterndetect => rx_patterndetect(4*2*(i+1)-1 downto 4*2*i),
        rx_disperr => rx_disperr(4*2*(i+1)-1 downto 4*2*i),
        rx_errdetect => rx_errdetect(4*2*(i+1)-1 downto 4*2*i),
        rx_syncstatus => rx_syncstatus(4*2*(i+1)-1 downto 4*2*i),

        pipe8b10binvpolarity => pipe8b10binvpolarity(4*(i+1)-1 downto 4*i),
        pipedatavalid => pipedatavalid(4*(i+1)-1 downto 4*i),
        pipeelecidle => pipeelecidle(4*(i+1)-1 downto 4*i),
        pipephydonestatus => pipephydonestatus(4*(i+1)-1 downto 4*i),
        pipestatus => pipestatus(4*3*(i+1)-1 downto 4*3*i)
      );
  end generate x4gen;

end syn;


library ieee;
use ieee.std_logic_1164.all;

entity myalt2gxb_agx_x4 is
  port
    (
      cal_blk_clk : in std_logic ;
      fixedclk : in std_logic ;
      pll_inclk : in std_logic ;
      powerdn : in std_logic_vector(7 downto 0);

      pll_locked : out std_logic_vector(0 downto 0);
      gxb_powerdown : in std_logic_vector(0 downto 0);
      tx_digitalreset : in std_logic_vector(0 downto 0);
      rx_analogreset : in std_logic_vector(0 downto 0);
      rx_digitalreset : in std_logic_vector(0 downto 0);
      rx_pll_locked : out std_logic_vector(3 downto 0);
      rx_freqlocked : out std_logic_vector(3 downto 0);

      coreclkout : out std_logic_vector(0 downto 0);
      tx_coreclk : in std_logic_vector(3 downto 0);
      rx_coreclk : in std_logic_vector(3 downto 0);

      tx_dataout : out std_logic_vector(3 downto 0);
      tx_datain : in std_logic_vector(63 downto 0);
      tx_ctrlenable : in std_logic_vector(7 downto 0);
      tx_forceelecidle : in std_logic_vector(3 downto 0);
      tx_detectrxloop : in std_logic_vector(3 downto 0);
      tx_forcedispcompliance : in std_logic_vector(3 downto 0);
      pipe8b10binvpolarity : in std_logic_vector(3 downto 0);

      rx_datain : in std_logic_vector(3 downto 0);
      rx_dataout : out std_logic_vector(63 downto 0);
      rx_ctrldetect : out std_logic_vector(7 downto 0);
      rx_patterndetect : out std_logic_vector(7 downto 0);
      rx_disperr : out std_logic_vector(7 downto 0);
      rx_errdetect : out std_logic_vector(7 downto 0);
      rx_syncstatus : out std_logic_vector(7 downto 0);

      pipedatavalid : out std_logic_vector(3 downto 0);
      pipeelecidle : out std_logic_vector(3 downto 0);
      pipephydonestatus : out std_logic_vector(3 downto 0);
      pipestatus : out std_logic_vector(11 downto 0)
    );
end myalt2gxb_agx_x4;


architecture syn of myalt2gxb_agx_x4 is

  signal sub_wire0 : std_logic_vector (7 downto 0);
  signal sub_wire1 : std_logic_vector (7 downto 0);
  signal sub_wire2 : std_logic_vector (0 downto 0);
  signal sub_wire3 : std_logic_vector (7 downto 0);
  signal sub_wire4 : std_logic_vector (3 downto 0);
  signal sub_wire5 : std_logic_vector (7 downto 0);
  signal sub_wire6 : std_logic_vector (3 downto 0);
  signal sub_wire7 : std_logic_vector (3 downto 0);
  signal sub_wire8 : std_logic_vector (3 downto 0);
  signal sub_wire9 : std_logic_vector (3 downto 0);
  signal sub_wire10 : std_logic_vector (3 downto 0);
  signal sub_wire11 : std_logic_vector (11 downto 0);
  signal sub_wire12 : std_logic_vector (7 downto 0);
  signal sub_wire13 : std_logic_vector (0 downto 0);
  signal sub_wire14 : std_logic_vector (63 downto 0);



  component alt2gxb
    generic (
      cmu_pll_inclock_period : natural;
      cmu_pll_loop_filter_resistor_control : natural;
      digitalreset_port_width : natural;
      enable_fast_recovery_pci_mode : string;
      enable_pll_inclk_alt_drive_rx_cru : string;
      enable_pll_inclk_drive_rx_cru : string;
      en_local_clk_div_ctrl : string;
      equalizer_ctrl_a_setting : natural;
      equalizer_ctrl_b_setting : natural;
      equalizer_ctrl_c_setting : natural;
      equalizer_ctrl_d_setting : natural;
      equalizer_ctrl_v_setting : natural;
      equalizer_dcgain_setting : natural;
      intended_device_family : string;
      loopback_mode : string;
      lpm_type : string;
      number_of_channels : natural;
      operation_mode : string;
      preemphasis_ctrl_1stposttap_setting : natural;
      preemphasis_ctrl_2ndposttap_inv_setting : string;
      preemphasis_ctrl_2ndposttap_setting : natural;
      preemphasis_ctrl_pretap_inv_setting : string;
      preemphasis_ctrl_pretap_setting : natural;
      protocol : string;
      receiver_termination : string;
      reconfig_dprio_mode : natural;
      reverse_loopback_mode : string;
      rx_8b_10b_compatibility_mode : string;
      rx_8b_10b_mode : string;
      rx_align_pattern : string;
      rx_align_pattern_length : natural;
      rx_allow_align_polarity_inversion : string;
      rx_allow_pipe_polarity_inversion : string;
      rx_bandwidth_mode : natural;
      rx_bitslip_enable : string;
      rx_byte_ordering_mode : string;
      rx_channel_bonding : string;
      rx_channel_width : natural;
      rx_common_mode : string;
      rx_cru_pre_divide_by : natural;
      rx_datapath_protocol : string;
      rx_data_rate : natural;
      rx_data_rate_remainder : natural;
      rx_disable_auto_idle_insertion : string;
      rx_enable_bit_reversal : string;
      rx_enable_lock_to_data_sig : string;
      rx_enable_lock_to_refclk_sig : string;
      rx_enable_self_test_mode : string;
      rx_enable_true_complement_match_in_word_align : string;
      rx_force_signal_detect : string;
      rx_ppmselect : natural;
      rx_rate_match_back_to_back : string;
      rx_rate_match_fifo_mode : string;
      rx_rate_match_ordered_set_based : string;
      rx_rate_match_pattern1 : string;
      rx_rate_match_pattern2 : string;
      rx_rate_match_pattern_size : natural;
      rx_rate_match_skip_set_based : string;
      rx_run_length_enable : string;
      rx_signal_detect_threshold : natural;
      rx_use_align_state_machine : string;
      rx_use_clkout : string;
      rx_use_coreclk : string;
      rx_use_deserializer_double_data_mode : string;
      rx_use_deskew_fifo : string;
      rx_use_double_data_mode : string;
      rx_use_pipe8b10binvpolarity : string;
      rx_use_rate_match_pattern1_only : string;
      transmitter_termination : string;
      tx_8b_10b_compatibility_mode : string;
      tx_8b_10b_mode : string;
      tx_allow_polarity_inversion : string;
      tx_analog_power : string;
      tx_channel_bonding : string;
      tx_channel_width : natural;
      tx_common_mode : string;
      tx_data_rate : natural;
      tx_data_rate_remainder : natural;
      tx_enable_bit_reversal : string;
      tx_enable_idle_selection : string;
      tx_enable_self_test_mode : string;
      tx_refclk_divide_by : natural;
      tx_transmit_protocol : string;
      tx_use_coreclk : string;
      tx_use_double_data_mode : string;
      tx_use_serializer_double_data_mode : string;
      use_calibration_block : string;
      vod_ctrl_setting : natural
      );
    port (
      tx_forceelecidle : in std_logic_vector (3 downto 0);
      rx_disperr : out std_logic_vector (7 downto 0);
      rx_patterndetect : out std_logic_vector (7 downto 0);
      rx_coreclk : in std_logic_vector (3 downto 0);
      coreclkout : out std_logic_vector (0 downto 0);
      rx_ctrldetect : out std_logic_vector (7 downto 0);
      pll_inclk : in std_logic ;
      gxb_powerdown : in std_logic_vector (0 downto 0);
      pipedatavalid : out std_logic_vector (3 downto 0);
      tx_datain : in std_logic_vector (63 downto 0);
      rx_errdetect : out std_logic_vector (7 downto 0);
      pipephydonestatus : out std_logic_vector (3 downto 0);
      rx_pll_locked : out std_logic_vector (3 downto 0);
      rx_freqlocked : out std_logic_vector (3 downto 0);
      cal_blk_clk : in std_logic ;
      powerdn : in std_logic_vector (7 downto 0);
      tx_dataout : out std_logic_vector (3 downto 0);
      fixedclk : in std_logic ;
      rx_datain : in std_logic_vector (3 downto 0);
      pipeelecidle : out std_logic_vector (3 downto 0);
      tx_coreclk : in std_logic_vector (3 downto 0);
      tx_ctrlenable : in std_logic_vector (7 downto 0);
      pipestatus : out std_logic_vector (11 downto 0);
      rx_analogreset : in std_logic_vector (0 downto 0);
      pipe8b10binvpolarity : in std_logic_vector (3 downto 0);
      rx_syncstatus : out std_logic_vector (7 downto 0);
      rx_digitalreset : in std_logic_vector (0 downto 0);
      tx_digitalreset : in std_logic_vector (0 downto 0);
      tx_forcedispcompliance : in std_logic_vector (3 downto 0);
      pll_locked : out std_logic_vector (0 downto 0);
      rx_dataout : out std_logic_vector (63 downto 0);
      tx_detectrxloop : in std_logic_vector (3 downto 0)
      );
  end component;

begin
  rx_disperr <= sub_wire0(7 downto 0);
  rx_patterndetect <= sub_wire1(7 downto 0);
  coreclkout <= sub_wire2(0 downto 0);
  rx_ctrldetect <= sub_wire3(7 downto 0);
  pipedatavalid <= sub_wire4(3 downto 0);
  rx_errdetect <= sub_wire5(7 downto 0);
  pipephydonestatus <= sub_wire6(3 downto 0);
  rx_pll_locked <= sub_wire7(3 downto 0);
  rx_freqlocked <= sub_wire8(3 downto 0);
  tx_dataout <= sub_wire9(3 downto 0);
  pipeelecidle <= sub_wire10(3 downto 0);
  pipestatus <= sub_wire11(11 downto 0);
  rx_syncstatus <= sub_wire12(7 downto 0);
  pll_locked <= sub_wire13(0 downto 0);
  rx_dataout <= sub_wire14(63 downto 0);

  alt2gxb_component : alt2gxb
    generic map (
      cmu_pll_inclock_period => 10000,
      cmu_pll_loop_filter_resistor_control => 3,
      digitalreset_port_width => 1,
      enable_fast_recovery_pci_mode => "false",
      enable_pll_inclk_alt_drive_rx_cru => "true",
      enable_pll_inclk_drive_rx_cru => "true",
      en_local_clk_div_ctrl => "true",
      equalizer_ctrl_a_setting => 0,
      equalizer_ctrl_b_setting => 0,
      equalizer_ctrl_c_setting => 0,
      equalizer_ctrl_d_setting => 0,
      equalizer_ctrl_v_setting => 0,
      equalizer_dcgain_setting => 1,
      intended_device_family => "arria gx",
      loopback_mode => "none",
      lpm_type => "alt2gxb",
      number_of_channels => 4,
      operation_mode => "duplex",
      preemphasis_ctrl_1stposttap_setting => 5,
      preemphasis_ctrl_2ndposttap_inv_setting => "false",
      preemphasis_ctrl_2ndposttap_setting => 0,
      preemphasis_ctrl_pretap_inv_setting => "false",
      preemphasis_ctrl_pretap_setting => 0,
      protocol => "pipe",
      receiver_termination => "oct_100_ohms",
      reconfig_dprio_mode => 0,
      reverse_loopback_mode => "rplb",
      rx_8b_10b_compatibility_mode => "true",
      rx_8b_10b_mode => "normal",
      rx_align_pattern => "0101111100",
      rx_align_pattern_length => 10,
      rx_allow_align_polarity_inversion => "false",
      rx_allow_pipe_polarity_inversion => "true",
      rx_bandwidth_mode => 1,
      rx_bitslip_enable => "false",
      rx_byte_ordering_mode => "none",
      rx_channel_bonding => "x4",
      rx_channel_width => 16,
      rx_common_mode => "0.9v",
      rx_cru_pre_divide_by => 1,
      rx_datapath_protocol => "pipe",
      rx_data_rate => 2500,
      rx_data_rate_remainder => 0,
      rx_disable_auto_idle_insertion => "false",
      rx_enable_bit_reversal => "false",
      rx_enable_lock_to_data_sig => "false",
      rx_enable_lock_to_refclk_sig => "false",
      rx_enable_self_test_mode => "false",
      rx_enable_true_complement_match_in_word_align => "false",
      rx_force_signal_detect => "false",
      rx_ppmselect => 8,
      rx_rate_match_back_to_back => "false",
      rx_rate_match_fifo_mode => "normal",
      rx_rate_match_ordered_set_based => "false",
      rx_rate_match_pattern1 => "11010000111010000011",
      rx_rate_match_pattern2 => "00101111000101111100",
      rx_rate_match_pattern_size => 20,
      rx_rate_match_skip_set_based => "false",
      rx_run_length_enable => "false",
      rx_signal_detect_threshold => 2,
      rx_use_align_state_machine => "true",
      rx_use_clkout => "false",
      rx_use_coreclk => "true",
      rx_use_deserializer_double_data_mode => "false",
      rx_use_deskew_fifo => "false",
      rx_use_double_data_mode => "true",
      rx_use_pipe8b10binvpolarity => "true",
      rx_use_rate_match_pattern1_only => "false",
      transmitter_termination => "oct_100_ohms",
      tx_8b_10b_compatibility_mode => "true",
      tx_8b_10b_mode => "normal",
      tx_allow_polarity_inversion => "false",
      tx_analog_power => "1.2v",
      tx_channel_bonding => "x4",
      tx_channel_width => 16,
      tx_common_mode => "0.6v",
      tx_data_rate => 2500,
      tx_data_rate_remainder => 0,
      tx_enable_bit_reversal => "false",
      tx_enable_idle_selection => "false",
      tx_enable_self_test_mode => "false",
      tx_refclk_divide_by => 1,
      tx_transmit_protocol => "pipe",
      tx_use_coreclk => "true",
      tx_use_double_data_mode => "true",
      tx_use_serializer_double_data_mode => "false",
      use_calibration_block => "true",
      vod_ctrl_setting => 4
      )
    port map (
      tx_forceelecidle => tx_forceelecidle,
      rx_coreclk => rx_coreclk,
      pll_inclk => pll_inclk,
      gxb_powerdown => gxb_powerdown,
      tx_datain => tx_datain,
      cal_blk_clk => cal_blk_clk,
      powerdn => powerdn,
      fixedclk => fixedclk,
      rx_datain => rx_datain,
      tx_coreclk => tx_coreclk,
      tx_ctrlenable => tx_ctrlenable,
      rx_analogreset => rx_analogreset,
      pipe8b10binvpolarity => pipe8b10binvpolarity,
      rx_digitalreset => rx_digitalreset,
      tx_digitalreset => tx_digitalreset,
      tx_forcedispcompliance => tx_forcedispcompliance,
      tx_detectrxloop => tx_detectrxloop,
      rx_disperr => sub_wire0,
      rx_patterndetect => sub_wire1,
      coreclkout => sub_wire2,
      rx_ctrldetect => sub_wire3,
      pipedatavalid => sub_wire4,
      rx_errdetect => sub_wire5,
      pipephydonestatus => sub_wire6,
      rx_pll_locked => sub_wire7,
      rx_freqlocked => sub_wire8,
      tx_dataout => sub_wire9,
      pipeelecidle => sub_wire10,
      pipestatus => sub_wire11,
      rx_syncstatus => sub_wire12,
      pll_locked => sub_wire13,
      rx_dataout => sub_wire14
      );
end syn;
-- megafunction wizard: %ALT2GXB%
-- GENERATION: STANDARD
-- VERSION: WM1.0
-- MODULE: alt2gxb

-- ============================================================
-- File Name: alt2gxb_s2gx.vhd
-- Megafunction Name(s):
-- alt2gxb
--
-- Simulation Library Files(s):
--
-- ============================================================
-- ************************************************************
-- THIS IS A WIZARD-GENERATED FILE. DO NOT EDIT THIS FILE!
--
-- 7.1 Build 178 06/25/2007 SP 1 SJ Full Version
-- ************************************************************


--Copyright (C) 1991-2007 Altera Corporation
--Your use of Altera Corporation's design tools, logic functions 
--and other software and tools, and its AMPP partner logic
--functions, and any output files from any of the foregoing
--(including device programming or simulation files), and any
--associated documentation or information are expressly subject
--to the terms and conditions of the Altera Program License
--Subscription Agreement, Altera MegaCore Function License
--Agreement, or other applicable license agreement, including,
--without limitation, that your use is for the sole purpose of
--programming logic devices manufactured by Altera and sold by
--Altera or its authorized distributors. Please refer to the
--applicable agreement for further details.


LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY alt2gxb_s2gx_x1 IS
 PORT
 (
  cal_blk_clk : IN STD_LOGIC ;
  fixedclk : IN STD_LOGIC ;
  gxb_powerdown : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipe8b10binvpolarity : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  pll_inclk : IN STD_LOGIC ;
  powerdn : IN STD_LOGIC_VECTOR (1 DOWNTO 0);
  rx_analogreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_coreclk : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_datain : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_coreclk : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_ctrlenable : IN STD_LOGIC_VECTOR (1 DOWNTO 0);
  tx_datain : IN STD_LOGIC_VECTOR (15 DOWNTO 0);
  tx_detectrxloop : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_forcedispcompliance : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_forceelecidle : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipedatavalid : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipeelecidle : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipephydonestatus : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipestatus : OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
  pll_locked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_ctrldetect : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
  rx_dataout : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
  rx_disperr : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
  rx_errdetect : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
  rx_freqlocked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_patterndetect : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
  rx_pll_locked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_syncstatus : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
  coreclkout : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_dataout : OUT STD_LOGIC_VECTOR (0 DOWNTO 0)
 );
END alt2gxb_s2gx_x1;


ARCHITECTURE SYN OF alt2gxb_s2gx_x1 IS

 SIGNAL sub_wire0 : STD_LOGIC_VECTOR (1 DOWNTO 0);
 SIGNAL sub_wire1 : STD_LOGIC_VECTOR (1 DOWNTO 0);
 SIGNAL sub_wire2 : STD_LOGIC_VECTOR (1 DOWNTO 0);
 SIGNAL sub_wire3 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire4 : STD_LOGIC_VECTOR (1 DOWNTO 0);
 SIGNAL sub_wire5 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire6 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire7 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire8 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire9 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire10 : STD_LOGIC_VECTOR (2 DOWNTO 0);
 SIGNAL sub_wire11 : STD_LOGIC_VECTOR (1 DOWNTO 0);
 SIGNAL sub_wire12 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire13 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire14 : STD_LOGIC_VECTOR (15 DOWNTO 0);



 COMPONENT alt2gxb
 GENERIC (
  cmu_pll_inclock_period : NATURAL;
  cmu_pll_loop_filter_resistor_control : NATURAL;
  digitalreset_port_width : NATURAL;
  enable_fast_recovery_pci_mode : STRING;
  enable_pll_inclk_alt_drive_rx_cru : STRING;
  enable_pll_inclk_drive_rx_cru : STRING;
  en_local_clk_div_ctrl : STRING;
  equalizer_ctrl_a_setting : NATURAL;
  equalizer_ctrl_b_setting : NATURAL;
  equalizer_ctrl_c_setting : NATURAL;
  equalizer_ctrl_d_setting : NATURAL;
  equalizer_ctrl_v_setting : NATURAL;
  equalizer_dcgain_setting : NATURAL;
  intended_device_family : STRING;
  loopback_mode : STRING;
  lpm_type : STRING;
  number_of_channels : NATURAL;
  operation_mode : STRING;
  preemphasis_ctrl_1stposttap_setting : NATURAL;
  preemphasis_ctrl_2ndposttap_inv_setting : STRING;
  preemphasis_ctrl_2ndposttap_setting : NATURAL;
  preemphasis_ctrl_pretap_inv_setting : STRING;
  preemphasis_ctrl_pretap_setting : NATURAL;
  protocol : STRING;
  receiver_termination : STRING;
  reconfig_dprio_mode : NATURAL;
  reverse_loopback_mode : STRING;
  rx_8b_10b_compatibility_mode : STRING;
  rx_8b_10b_mode : STRING;
  rx_align_pattern : STRING;
  rx_align_pattern_length : NATURAL;
  rx_allow_align_polarity_inversion : STRING;
  rx_allow_pipe_polarity_inversion : STRING;
  rx_bandwidth_mode : NATURAL;
  rx_bitslip_enable : STRING;
  rx_byte_ordering_mode : STRING;
  rx_channel_bonding : STRING;
  rx_channel_width : NATURAL;
  rx_common_mode : STRING;
  rx_cru_pre_divide_by : NATURAL;
  rx_datapath_protocol : STRING;
  rx_data_rate : NATURAL;
  rx_data_rate_remainder : NATURAL;
  rx_disable_auto_idle_insertion : STRING;
  rx_enable_bit_reversal : STRING;
  rx_enable_lock_to_data_sig : STRING;
  rx_enable_lock_to_refclk_sig : STRING;
  rx_enable_self_test_mode : STRING;
  rx_enable_true_complement_match_in_word_align : STRING;
  rx_force_signal_detect : STRING;
  rx_ppmselect : NATURAL;
  rx_rate_match_back_to_back : STRING;
  rx_rate_match_fifo_mode : STRING;
  rx_rate_match_ordered_set_based : STRING;
  rx_rate_match_pattern1 : STRING;
  rx_rate_match_pattern2 : STRING;
  rx_rate_match_pattern_size : NATURAL;
  rx_rate_match_skip_set_based : STRING;
  rx_run_length_enable : STRING;
  rx_signal_detect_threshold : NATURAL;
  rx_use_align_state_machine : STRING;
  rx_use_clkout : STRING;
  rx_use_coreclk : STRING;
  rx_use_deserializer_double_data_mode : STRING;
  rx_use_deskew_fifo : STRING;
  rx_use_double_data_mode : STRING;
  rx_use_pipe8b10binvpolarity : STRING;
  rx_use_rate_match_pattern1_only : STRING;
  transmitter_termination : STRING;
  tx_8b_10b_compatibility_mode : STRING;
  tx_8b_10b_mode : STRING;
  tx_allow_polarity_inversion : STRING;
  tx_analog_power : STRING;
  tx_channel_bonding : STRING;
  tx_channel_width : NATURAL;
  tx_common_mode : STRING;
  tx_data_rate : NATURAL;
  tx_data_rate_remainder : NATURAL;
  tx_enable_bit_reversal : STRING;
  tx_enable_idle_selection : STRING;
  tx_enable_self_test_mode : STRING;
  tx_refclk_divide_by : NATURAL;
  tx_transmit_protocol : STRING;
  tx_use_coreclk : STRING;
  tx_use_double_data_mode : STRING;
  tx_use_serializer_double_data_mode : STRING;
  use_calibration_block : STRING;
  vod_ctrl_setting : NATURAL
 );
 PORT (
   tx_forceelecidle : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_disperr : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   rx_patterndetect : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   rx_coreclk : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_ctrldetect : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   pll_inclk : IN STD_LOGIC ;
   gxb_powerdown : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pipedatavalid : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_datain : IN STD_LOGIC_VECTOR (15 DOWNTO 0);
   rx_errdetect : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   pipephydonestatus : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_pll_locked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_freqlocked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   cal_blk_clk : IN STD_LOGIC ;
   powerdn : IN STD_LOGIC_VECTOR (1 DOWNTO 0);
   tx_dataout : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   fixedclk : IN STD_LOGIC ;
   rx_datain : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pipeelecidle : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_coreclk : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_ctrlenable : IN STD_LOGIC_VECTOR (1 DOWNTO 0);
   pipestatus : OUT STD_LOGIC_VECTOR (2 DOWNTO 0);
   rx_analogreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pipe8b10binvpolarity : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_syncstatus : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   rx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_clkout : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_forcedispcompliance : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pll_locked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_dataout : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
   tx_detectrxloop : IN STD_LOGIC_VECTOR (0 DOWNTO 0)
 );
 END COMPONENT;

BEGIN
 rx_disperr <= sub_wire0(1 DOWNTO 0);
 rx_patterndetect <= sub_wire1(1 DOWNTO 0);
 rx_ctrldetect <= sub_wire2(1 DOWNTO 0);
 pipedatavalid <= sub_wire3(0 DOWNTO 0);
 rx_errdetect <= sub_wire4(1 DOWNTO 0);
 pipephydonestatus <= sub_wire5(0 DOWNTO 0);
 rx_pll_locked <= sub_wire6(0 DOWNTO 0);
 rx_freqlocked <= sub_wire7(0 DOWNTO 0);
 tx_dataout <= sub_wire8(0 DOWNTO 0);
 pipeelecidle <= sub_wire9(0 DOWNTO 0);
 pipestatus <= sub_wire10(2 DOWNTO 0);
 rx_syncstatus <= sub_wire11(1 DOWNTO 0);
 coreclkout <= sub_wire12(0 DOWNTO 0);
 pll_locked <= sub_wire13(0 DOWNTO 0);
 rx_dataout <= sub_wire14(15 DOWNTO 0);

 alt2gxb_component : alt2gxb
 GENERIC MAP (
  cmu_pll_inclock_period => 10000,
  cmu_pll_loop_filter_resistor_control => 3,
  digitalreset_port_width => 1,
  enable_fast_recovery_pci_mode => "false",
  enable_pll_inclk_alt_drive_rx_cru => "true",
  enable_pll_inclk_drive_rx_cru => "true",
  en_local_clk_div_ctrl => "true",
  equalizer_ctrl_a_setting => 0,
  equalizer_ctrl_b_setting => 0,
  equalizer_ctrl_c_setting => 0,
  equalizer_ctrl_d_setting => 0,
  equalizer_ctrl_v_setting => 0,
  equalizer_dcgain_setting => 1,
  intended_device_family => "Stratix II GX",
  loopback_mode => "none",
  lpm_type => "alt2gxb",
  number_of_channels => 1,
  operation_mode => "duplex",
  preemphasis_ctrl_1stposttap_setting => 5,
  preemphasis_ctrl_2ndposttap_inv_setting => "false",
  preemphasis_ctrl_2ndposttap_setting => 0,
  preemphasis_ctrl_pretap_inv_setting => "false",
  preemphasis_ctrl_pretap_setting => 0,
  protocol => "pipe",
  receiver_termination => "oct_100_ohms",
  reconfig_dprio_mode => 0,
  reverse_loopback_mode => "rplb",
  rx_8b_10b_compatibility_mode => "true",
  rx_8b_10b_mode => "normal",
  rx_align_pattern => "0101111100",
  rx_align_pattern_length => 10,
  rx_allow_align_polarity_inversion => "false",
  rx_allow_pipe_polarity_inversion => "true",
  rx_bandwidth_mode => 1,
  rx_bitslip_enable => "false",
  rx_byte_ordering_mode => "none",
  rx_channel_bonding => "indv",
  rx_channel_width => 16,
  rx_common_mode => "0.9v",
  rx_cru_pre_divide_by => 1,
  rx_datapath_protocol => "pipe",
  rx_data_rate => 2500,
  rx_data_rate_remainder => 0,
  rx_disable_auto_idle_insertion => "false",
  rx_enable_bit_reversal => "false",
  rx_enable_lock_to_data_sig => "false",
  rx_enable_lock_to_refclk_sig => "false",
  rx_enable_self_test_mode => "false",
  rx_enable_true_complement_match_in_word_align => "false",
  rx_force_signal_detect => "false",
  rx_ppmselect => 8,
  rx_rate_match_back_to_back => "false",
  rx_rate_match_fifo_mode => "normal",
  rx_rate_match_ordered_set_based => "false",
  rx_rate_match_pattern1 => "11010000111010000011",
  rx_rate_match_pattern2 => "00101111000101111100",
  rx_rate_match_pattern_size => 20,
  rx_rate_match_skip_set_based => "false",
  rx_run_length_enable => "false",
  rx_signal_detect_threshold => 2,
  rx_use_align_state_machine => "true",
  rx_use_clkout => "false",
  rx_use_coreclk => "true",
  rx_use_deserializer_double_data_mode => "false",
  rx_use_deskew_fifo => "false",
  rx_use_double_data_mode => "true",
  rx_use_pipe8b10binvpolarity => "true",
  rx_use_rate_match_pattern1_only => "false",
  transmitter_termination => "oct_100_ohms",
  tx_8b_10b_compatibility_mode => "true",
  tx_8b_10b_mode => "normal",
  tx_allow_polarity_inversion => "false",
  tx_analog_power => "1.2v",
  tx_channel_bonding => "indv",
  tx_channel_width => 16,
  tx_common_mode => "0.6v",
  tx_data_rate => 2500,
  tx_data_rate_remainder => 0,
  tx_enable_bit_reversal => "false",
  tx_enable_idle_selection => "false",
  tx_enable_self_test_mode => "false",
  tx_refclk_divide_by => 1,
  tx_transmit_protocol => "pipe",
  tx_use_coreclk => "true",
  tx_use_double_data_mode => "true",
  tx_use_serializer_double_data_mode => "false",
  use_calibration_block => "true",
  vod_ctrl_setting => 4
 )
 PORT MAP (
  tx_forceelecidle => tx_forceelecidle,
  rx_coreclk => rx_coreclk,
  pll_inclk => pll_inclk,
  gxb_powerdown => gxb_powerdown,
  tx_datain => tx_datain,
  cal_blk_clk => cal_blk_clk,
  powerdn => powerdn,
  fixedclk => fixedclk,
  rx_datain => rx_datain,
  tx_coreclk => tx_coreclk,
  tx_ctrlenable => tx_ctrlenable,
  rx_analogreset => rx_analogreset,
  pipe8b10binvpolarity => pipe8b10binvpolarity,
  rx_digitalreset => rx_digitalreset,
  tx_digitalreset => tx_digitalreset,
  tx_forcedispcompliance => tx_forcedispcompliance,
  tx_detectrxloop => tx_detectrxloop,
  rx_disperr => sub_wire0,
  rx_patterndetect => sub_wire1,
  rx_ctrldetect => sub_wire2,
  pipedatavalid => sub_wire3,
  rx_errdetect => sub_wire4,
  pipephydonestatus => sub_wire5,
  rx_pll_locked => sub_wire6,
  rx_freqlocked => sub_wire7,
  tx_dataout => sub_wire8,
  pipeelecidle => sub_wire9,
  pipestatus => sub_wire10,
  rx_syncstatus => sub_wire11,
  tx_clkout => sub_wire12,
  pll_locked => sub_wire13,
  rx_dataout => sub_wire14
 );



END SYN;

-- ============================================================
-- CNX file retrieval info
-- ============================================================
-- Retrieval info: PRIVATE: ALT_SIMLIB_GEN STRING "0"
-- Retrieval info: PRIVATE: INTENDED_DEVICE_FAMILY STRING "Stratix II GX"
-- Retrieval info: PRIVATE: RECONFIG_PROTOCOL STRING "BASIC"
-- Retrieval info: PRIVATE: RECONFIG_SUBPROTOCOL STRING "none"
-- Retrieval info: PRIVATE: RX_ENABLE_DC_COUPLING STRING "false"
-- Retrieval info: PRIVATE: RX_FORCE_SIGNAL_DETECT STRING "false"
-- Retrieval info: PRIVATE: RX_SIGNAL_DETECT_THRESHOLD NUMERIC "2"
-- Retrieval info: PRIVATE: SYNTH_WRAPPER_GEN_POSTFIX STRING "0"
-- Retrieval info: PRIVATE: VOD_CTRL_SETTING NUMERIC "800"
-- Retrieval info: PRIVATE: WIZ_DATA_RATE STRING "2500"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INCLK_FREQ_ARRAY STRING "100.0 125.0 200.0 250.0 400.0 500.0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_A STRING "2000"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_A_UNIT STRING "Mbps"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_B STRING "100.0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_B_UNIT STRING "MHz"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_SELECTION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK0_FREQ STRING "100"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK0_PROTOCOL STRING "PCI Express (PIPE)"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK1_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK1_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK2_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK2_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK3_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK3_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK4_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK4_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK5_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK5_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK6_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK6_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_ENABLE_EQUALIZER_CTRL NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EQUALIZER_CTRL_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EXTERNAL_RX_TERMINATION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EXTERNAL_TX_TERMINATION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_FORCE_DEFAULT_SETTINGS NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_INCLK_FREQ STRING "100"
-- Retrieval info: PRIVATE: WIZ_INCLK_FREQ_ARRAY STRING "100"
-- Retrieval info: PRIVATE: WIZ_INPUT_A STRING "2500"
-- Retrieval info: PRIVATE: WIZ_INPUT_A_UNIT STRING "Mbps"
-- Retrieval info: PRIVATE: WIZ_INPUT_B STRING "100"
-- Retrieval info: PRIVATE: WIZ_INPUT_B_UNIT STRING "MHz"
-- Retrieval info: PRIVATE: WIZ_INPUT_SELECTION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_1STPOSTTAP_SETTING NUMERIC "5"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_2NDPOSTTAP_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_PRETAP_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PROTOCOL STRING "PCI Express (PIPE)"
-- Retrieval info: PRIVATE: WIZ_RX_TERMINATION NUMERIC "100"
-- Retrieval info: PRIVATE: WIZ_RX_VCM STRING "0.85"
-- Retrieval info: PRIVATE: WIZ_SUBPROTOCOL STRING "x1"
-- Retrieval info: PRIVATE: WIZ_TX_TERMINATION NUMERIC "100"
-- Retrieval info: PRIVATE: WIZ_TX_VCM STRING "0.6"
-- Retrieval info: PRIVATE: WIZ_VCCHTX STRING "1.2"
-- Retrieval info: PRIVATE: WIZ_WORD_ALIGN_FLIP_PATTERN STRING "0"
-- Retrieval info: CONSTANT: CMU_PLL_INCLOCK_PERIOD NUMERIC "10000"
-- Retrieval info: CONSTANT: CMU_PLL_LOOP_FILTER_RESISTOR_CONTROL NUMERIC "3"
-- Retrieval info: CONSTANT: DIGITALRESET_PORT_WIDTH NUMERIC "1"
-- Retrieval info: CONSTANT: ENABLE_FAST_RECOVERY_PCI_MODE STRING "false"
-- Retrieval info: CONSTANT: ENABLE_PLL_INCLK_ALT_DRIVE_RX_CRU STRING "true"
-- Retrieval info: CONSTANT: ENABLE_PLL_INCLK_DRIVE_RX_CRU STRING "true"
-- Retrieval info: CONSTANT: EN_LOCAL_CLK_DIV_CTRL STRING "true"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_A_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_B_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_C_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_D_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_V_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_DCGAIN_SETTING NUMERIC "1"
-- Retrieval info: CONSTANT: INTENDED_DEVICE_FAMILY STRING "Stratix II GX"
-- Retrieval info: CONSTANT: LOOPBACK_MODE STRING "none"
-- Retrieval info: CONSTANT: LPM_TYPE STRING "alt2gxb"
-- Retrieval info: CONSTANT: NUMBER_OF_CHANNELS NUMERIC "1"
-- Retrieval info: CONSTANT: OPERATION_MODE STRING "duplex"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_1STPOSTTAP_SETTING NUMERIC "5"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_2NDPOSTTAP_INV_SETTING STRING "false"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_2NDPOSTTAP_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_PRETAP_INV_SETTING STRING "false"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_PRETAP_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: RECEIVER_TERMINATION STRING "oct_100_ohms"
-- Retrieval info: CONSTANT: RECONFIG_DPRIO_MODE NUMERIC "0"
-- Retrieval info: CONSTANT: REVERSE_LOOPBACK_MODE STRING "rplb"
-- Retrieval info: CONSTANT: RX_8B_10B_COMPATIBILITY_MODE STRING "true"
-- Retrieval info: CONSTANT: RX_8B_10B_MODE STRING "normal"
-- Retrieval info: CONSTANT: RX_ALIGN_PATTERN STRING "0101111100"
-- Retrieval info: CONSTANT: RX_ALIGN_PATTERN_LENGTH NUMERIC "10"
-- Retrieval info: CONSTANT: RX_ALLOW_ALIGN_POLARITY_INVERSION STRING "false"
-- Retrieval info: CONSTANT: RX_ALLOW_PIPE_POLARITY_INVERSION STRING "true"
-- Retrieval info: CONSTANT: RX_BANDWIDTH_MODE NUMERIC "1"
-- Retrieval info: CONSTANT: RX_BITSLIP_ENABLE STRING "false"
-- Retrieval info: CONSTANT: RX_BYTE_ORDERING_MODE STRING "none"
-- Retrieval info: CONSTANT: RX_CHANNEL_BONDING STRING "indv"
-- Retrieval info: CONSTANT: RX_CHANNEL_WIDTH NUMERIC "16"
-- Retrieval info: CONSTANT: RX_COMMON_MODE STRING "0.9v"
-- Retrieval info: CONSTANT: RX_CRU_PRE_DIVIDE_BY NUMERIC "1"
-- Retrieval info: CONSTANT: RX_DATAPATH_PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: RX_DATA_RATE NUMERIC "2500"
-- Retrieval info: CONSTANT: RX_DATA_RATE_REMAINDER NUMERIC "0"
-- Retrieval info: CONSTANT: RX_DISABLE_AUTO_IDLE_INSERTION STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_BIT_REVERSAL STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_LOCK_TO_DATA_SIG STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_LOCK_TO_REFCLK_SIG STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_SELF_TEST_MODE STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_TRUE_COMPLEMENT_MATCH_IN_WORD_ALIGN STRING "false"
-- Retrieval info: CONSTANT: RX_FORCE_SIGNAL_DETECT STRING "false"
-- Retrieval info: CONSTANT: RX_PPMSELECT NUMERIC "8"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_BACK_TO_BACK STRING "false"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_FIFO_MODE STRING "normal"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_ORDERED_SET_BASED STRING "false"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN1 STRING "11010000111010000011"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN2 STRING "00101111000101111100"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN_SIZE NUMERIC "20"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_SKIP_SET_BASED STRING "false"
-- Retrieval info: CONSTANT: RX_RUN_LENGTH_ENABLE STRING "false"
-- Retrieval info: CONSTANT: RX_SIGNAL_DETECT_THRESHOLD NUMERIC "2"
-- Retrieval info: CONSTANT: RX_USE_ALIGN_STATE_MACHINE STRING "true"
-- Retrieval info: CONSTANT: RX_USE_CLKOUT STRING "false"
-- Retrieval info: CONSTANT: RX_USE_CORECLK STRING "true"
-- Retrieval info: CONSTANT: RX_USE_DESERIALIZER_DOUBLE_DATA_MODE STRING "false"
-- Retrieval info: CONSTANT: RX_USE_DESKEW_FIFO STRING "false"
-- Retrieval info: CONSTANT: RX_USE_DOUBLE_DATA_MODE STRING "true"
-- Retrieval info: CONSTANT: RX_USE_PIPE8B10BINVPOLARITY STRING "true"
-- Retrieval info: CONSTANT: RX_USE_RATE_MATCH_PATTERN1_ONLY STRING "false"
-- Retrieval info: CONSTANT: TRANSMITTER_TERMINATION STRING "oct_100_ohms"
-- Retrieval info: CONSTANT: TX_8B_10B_COMPATIBILITY_MODE STRING "true"
-- Retrieval info: CONSTANT: TX_8B_10B_MODE STRING "normal"
-- Retrieval info: CONSTANT: TX_ALLOW_POLARITY_INVERSION STRING "false"
-- Retrieval info: CONSTANT: TX_ANALOG_POWER STRING "1.2v"
-- Retrieval info: CONSTANT: TX_CHANNEL_BONDING STRING "indv"
-- Retrieval info: CONSTANT: TX_CHANNEL_WIDTH NUMERIC "16"
-- Retrieval info: CONSTANT: TX_COMMON_MODE STRING "0.6v"
-- Retrieval info: CONSTANT: TX_DATA_RATE NUMERIC "2500"
-- Retrieval info: CONSTANT: TX_DATA_RATE_REMAINDER NUMERIC "0"
-- Retrieval info: CONSTANT: TX_ENABLE_BIT_REVERSAL STRING "false"
-- Retrieval info: CONSTANT: TX_ENABLE_IDLE_SELECTION STRING "false"
-- Retrieval info: CONSTANT: TX_ENABLE_SELF_TEST_MODE STRING "false"
-- Retrieval info: CONSTANT: TX_REFCLK_DIVIDE_BY NUMERIC "1"
-- Retrieval info: CONSTANT: TX_TRANSMIT_PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: TX_USE_CORECLK STRING "true"
-- Retrieval info: CONSTANT: TX_USE_DOUBLE_DATA_MODE STRING "true"
-- Retrieval info: CONSTANT: TX_USE_SERIALIZER_DOUBLE_DATA_MODE STRING "false"
-- Retrieval info: CONSTANT: USE_CALIBRATION_BLOCK STRING "true"
-- Retrieval info: CONSTANT: VOD_CTRL_SETTING NUMERIC "4"
-- Retrieval info: USED_PORT: cal_blk_clk 0 0 0 0 INPUT NODEFVAL "cal_blk_clk"
-- Retrieval info: USED_PORT: fixedclk 0 0 0 0 INPUT NODEFVAL "fixedclk"
-- Retrieval info: USED_PORT: gxb_powerdown 0 0 1 0 INPUT NODEFVAL "gxb_powerdown[0..0]"
-- Retrieval info: USED_PORT: pipe8b10binvpolarity 0 0 1 0 INPUT NODEFVAL "pipe8b10binvpolarity[0..0]"
-- Retrieval info: USED_PORT: pipedatavalid 0 0 1 0 OUTPUT NODEFVAL "pipedatavalid[0..0]"
-- Retrieval info: USED_PORT: pipeelecidle 0 0 1 0 OUTPUT NODEFVAL "pipeelecidle[0..0]"
-- Retrieval info: USED_PORT: pipephydonestatus 0 0 1 0 OUTPUT NODEFVAL "pipephydonestatus[0..0]"
-- Retrieval info: USED_PORT: pipestatus 0 0 3 0 OUTPUT NODEFVAL "pipestatus[2..0]"
-- Retrieval info: USED_PORT: pll_inclk 0 0 0 0 INPUT NODEFVAL "pll_inclk"
-- Retrieval info: USED_PORT: pll_locked 0 0 1 0 OUTPUT NODEFVAL "pll_locked[0..0]"
-- Retrieval info: USED_PORT: powerdn 0 0 2 0 INPUT NODEFVAL "powerdn[1..0]"
-- Retrieval info: USED_PORT: rx_analogreset 0 0 1 0 INPUT NODEFVAL "rx_analogreset[0..0]"
-- Retrieval info: USED_PORT: rx_coreclk 0 0 1 0 INPUT NODEFVAL "rx_coreclk[0..0]"
-- Retrieval info: USED_PORT: rx_ctrldetect 0 0 2 0 OUTPUT NODEFVAL "rx_ctrldetect[1..0]"
-- Retrieval info: USED_PORT: rx_datain 0 0 1 0 INPUT NODEFVAL "rx_datain[0..0]"
-- Retrieval info: USED_PORT: rx_dataout 0 0 16 0 OUTPUT NODEFVAL "rx_dataout[15..0]"
-- Retrieval info: USED_PORT: rx_digitalreset 0 0 1 0 INPUT NODEFVAL "rx_digitalreset[0..0]"
-- Retrieval info: USED_PORT: rx_disperr 0 0 2 0 OUTPUT NODEFVAL "rx_disperr[1..0]"
-- Retrieval info: USED_PORT: rx_errdetect 0 0 2 0 OUTPUT NODEFVAL "rx_errdetect[1..0]"
-- Retrieval info: USED_PORT: rx_freqlocked 0 0 1 0 OUTPUT NODEFVAL "rx_freqlocked[0..0]"
-- Retrieval info: USED_PORT: rx_patterndetect 0 0 2 0 OUTPUT NODEFVAL "rx_patterndetect[1..0]"
-- Retrieval info: USED_PORT: rx_pll_locked 0 0 1 0 OUTPUT NODEFVAL "rx_pll_locked[0..0]"
-- Retrieval info: USED_PORT: rx_syncstatus 0 0 2 0 OUTPUT NODEFVAL "rx_syncstatus[1..0]"
-- Retrieval info: USED_PORT: tx_clkout 0 0 1 0 OUTPUT NODEFVAL "tx_clkout[0..0]"
-- Retrieval info: USED_PORT: tx_coreclk 0 0 1 0 INPUT NODEFVAL "tx_coreclk[0..0]"
-- Retrieval info: USED_PORT: tx_ctrlenable 0 0 2 0 INPUT NODEFVAL "tx_ctrlenable[1..0]"
-- Retrieval info: USED_PORT: tx_datain 0 0 16 0 INPUT NODEFVAL "tx_datain[15..0]"
-- Retrieval info: USED_PORT: tx_dataout 0 0 1 0 OUTPUT NODEFVAL "tx_dataout[0..0]"
-- Retrieval info: USED_PORT: tx_detectrxloop 0 0 1 0 INPUT NODEFVAL "tx_detectrxloop[0..0]"
-- Retrieval info: USED_PORT: tx_digitalreset 0 0 1 0 INPUT NODEFVAL "tx_digitalreset[0..0]"
-- Retrieval info: USED_PORT: tx_forcedispcompliance 0 0 1 0 INPUT NODEFVAL "tx_forcedispcompliance[0..0]"
-- Retrieval info: USED_PORT: tx_forceelecidle 0 0 1 0 INPUT NODEFVAL "tx_forceelecidle[0..0]"
-- Retrieval info: CONNECT: @fixedclk 0 0 0 0 fixedclk 0 0 0 0
-- Retrieval info: CONNECT: @tx_detectrxloop 0 0 1 0 tx_detectrxloop 0 0 1 0
-- Retrieval info: CONNECT: @tx_forcedispcompliance 0 0 1 0 tx_forcedispcompliance 0 0 1 0
-- Retrieval info: CONNECT: rx_patterndetect 0 0 2 0 @rx_patterndetect 0 0 2 0
-- Retrieval info: CONNECT: pipedatavalid 0 0 1 0 @pipedatavalid 0 0 1 0
-- Retrieval info: CONNECT: @rx_analogreset 0 0 1 0 rx_analogreset 0 0 1 0
-- Retrieval info: CONNECT: @powerdn 0 0 2 0 powerdn 0 0 2 0
-- Retrieval info: CONNECT: pipeelecidle 0 0 1 0 @pipeelecidle 0 0 1 0
-- Retrieval info: CONNECT: rx_ctrldetect 0 0 2 0 @rx_ctrldetect 0 0 2 0
-- Retrieval info: CONNECT: @gxb_powerdown 0 0 1 0 gxb_powerdown 0 0 1 0
-- Retrieval info: CONNECT: rx_dataout 0 0 16 0 @rx_dataout 0 0 16 0
-- Retrieval info: CONNECT: @cal_blk_clk 0 0 0 0 cal_blk_clk 0 0 0 0
-- Retrieval info: CONNECT: pipestatus 0 0 3 0 @pipestatus 0 0 3 0
-- Retrieval info: CONNECT: rx_errdetect 0 0 2 0 @rx_errdetect 0 0 2 0
-- Retrieval info: CONNECT: @tx_coreclk 0 0 1 0 tx_coreclk 0 0 1 0
-- Retrieval info: CONNECT: @tx_digitalreset 0 0 1 0 tx_digitalreset 0 0 1 0
-- Retrieval info: CONNECT: rx_pll_locked 0 0 1 0 @rx_pll_locked 0 0 1 0
-- Retrieval info: CONNECT: rx_syncstatus 0 0 2 0 @rx_syncstatus 0 0 2 0
-- Retrieval info: CONNECT: @pipe8b10binvpolarity 0 0 1 0 pipe8b10binvpolarity 0 0 1 0
-- Retrieval info: CONNECT: @rx_coreclk 0 0 1 0 rx_coreclk 0 0 1 0
-- Retrieval info: CONNECT: pll_locked 0 0 1 0 @pll_locked 0 0 1 0
-- Retrieval info: CONNECT: @rx_digitalreset 0 0 1 0 rx_digitalreset 0 0 1 0
-- Retrieval info: CONNECT: tx_clkout 0 0 1 0 @tx_clkout 0 0 1 0
-- Retrieval info: CONNECT: @pll_inclk 0 0 0 0 pll_inclk 0 0 0 0
-- Retrieval info: CONNECT: @tx_ctrlenable 0 0 2 0 tx_ctrlenable 0 0 2 0
-- Retrieval info: CONNECT: tx_dataout 0 0 1 0 @tx_dataout 0 0 1 0
-- Retrieval info: CONNECT: @tx_datain 0 0 16 0 tx_datain 0 0 16 0
-- Retrieval info: CONNECT: rx_freqlocked 0 0 1 0 @rx_freqlocked 0 0 1 0
-- Retrieval info: CONNECT: @rx_datain 0 0 1 0 rx_datain 0 0 1 0
-- Retrieval info: CONNECT: rx_disperr 0 0 2 0 @rx_disperr 0 0 2 0
-- Retrieval info: CONNECT: pipephydonestatus 0 0 1 0 @pipephydonestatus 0 0 1 0
-- Retrieval info: CONNECT: @tx_forceelecidle 0 0 1 0 tx_forceelecidle 0 0 1 0
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.x1.vhd TRUE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.x1.inc FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.x1.cmp FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.x1.bsf FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.x1_inst.vhd FALSE FALSE
-- megafunction wizard: %ALT2GXB%
-- GENERATION: STANDARD
-- VERSION: WM1.0
-- MODULE: alt2gxb

-- ============================================================
-- File Name: alt2gxb_s2gx.vhd
-- Megafunction Name(s):
-- alt2gxb
--
-- Simulation Library Files(s):
--
-- ============================================================
-- ************************************************************
-- THIS IS A WIZARD-GENERATED FILE. DO NOT EDIT THIS FILE!
--
-- 7.1 Build 178 06/25/2007 SP 1 SJ Full Version
-- ************************************************************


--Copyright (C) 1991-2007 Altera Corporation
--Your use of Altera Corporation's design tools, logic functions 
--and other software and tools, and its AMPP partner logic
--functions, and any output files from any of the foregoing
--(including device programming or simulation files), and any
--associated documentation or information are expressly subject
--to the terms and conditions of the Altera Program License
--Subscription Agreement, Altera MegaCore Function License
--Agreement, or other applicable license agreement, including,
--without limitation, that your use is for the sole purpose of
--programming logic devices manufactured by Altera and sold by
--Altera or its authorized distributors. Please refer to the
--applicable agreement for further details.


LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY alt2gxb_s2gx_x4 IS
 PORT
 (
  cal_blk_clk : IN STD_LOGIC ;
  fixedclk : IN STD_LOGIC ;
  gxb_powerdown : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipe8b10binvpolarity : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  pll_inclk : IN STD_LOGIC ;
  powerdn : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_analogreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_coreclk : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  rx_datain : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  rx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_coreclk : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  tx_ctrlenable : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
  tx_datain : IN STD_LOGIC_VECTOR (63 DOWNTO 0);
  tx_detectrxloop : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  tx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_forcedispcompliance : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  tx_forceelecidle : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
  coreclkout : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipedatavalid : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
  pipeelecidle : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
  pipephydonestatus : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
  pipestatus : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
  pll_locked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_ctrldetect : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_dataout : OUT STD_LOGIC_VECTOR (63 DOWNTO 0);
  rx_disperr : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_errdetect : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_freqlocked : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
  rx_patterndetect : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_pll_locked : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
  rx_syncstatus : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  tx_dataout : OUT STD_LOGIC_VECTOR (3 DOWNTO 0)
 );
END alt2gxb_s2gx_x4;


ARCHITECTURE SYN OF alt2gxb_s2gx_x4 IS

 SIGNAL sub_wire0 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire1 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire2 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire3 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire4 : STD_LOGIC_VECTOR (3 DOWNTO 0);
 SIGNAL sub_wire5 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire6 : STD_LOGIC_VECTOR (3 DOWNTO 0);
 SIGNAL sub_wire7 : STD_LOGIC_VECTOR (3 DOWNTO 0);
 SIGNAL sub_wire8 : STD_LOGIC_VECTOR (3 DOWNTO 0);
 SIGNAL sub_wire9 : STD_LOGIC_VECTOR (3 DOWNTO 0);
 SIGNAL sub_wire10 : STD_LOGIC_VECTOR (3 DOWNTO 0);
 SIGNAL sub_wire11 : STD_LOGIC_VECTOR (11 DOWNTO 0);
 SIGNAL sub_wire12 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire13 : STD_LOGIC_VECTOR (0 DOWNTO 0);
 SIGNAL sub_wire14 : STD_LOGIC_VECTOR (63 DOWNTO 0);



 COMPONENT alt2gxb
 GENERIC (
  cmu_pll_inclock_period : NATURAL;
  cmu_pll_loop_filter_resistor_control : NATURAL;
  digitalreset_port_width : NATURAL;
  enable_fast_recovery_pci_mode : STRING;
  enable_pll_inclk_alt_drive_rx_cru : STRING;
  enable_pll_inclk_drive_rx_cru : STRING;
  en_local_clk_div_ctrl : STRING;
  equalizer_ctrl_a_setting : NATURAL;
  equalizer_ctrl_b_setting : NATURAL;
  equalizer_ctrl_c_setting : NATURAL;
  equalizer_ctrl_d_setting : NATURAL;
  equalizer_ctrl_v_setting : NATURAL;
  equalizer_dcgain_setting : NATURAL;
  intended_device_family : STRING;
  loopback_mode : STRING;
  lpm_type : STRING;
  number_of_channels : NATURAL;
  operation_mode : STRING;
  preemphasis_ctrl_1stposttap_setting : NATURAL;
  preemphasis_ctrl_2ndposttap_inv_setting : STRING;
  preemphasis_ctrl_2ndposttap_setting : NATURAL;
  preemphasis_ctrl_pretap_inv_setting : STRING;
  preemphasis_ctrl_pretap_setting : NATURAL;
  protocol : STRING;
  receiver_termination : STRING;
  reconfig_dprio_mode : NATURAL;
  reverse_loopback_mode : STRING;
  rx_8b_10b_compatibility_mode : STRING;
  rx_8b_10b_mode : STRING;
  rx_align_pattern : STRING;
  rx_align_pattern_length : NATURAL;
  rx_allow_align_polarity_inversion : STRING;
  rx_allow_pipe_polarity_inversion : STRING;
  rx_bandwidth_mode : NATURAL;
  rx_bitslip_enable : STRING;
  rx_byte_ordering_mode : STRING;
  rx_channel_bonding : STRING;
  rx_channel_width : NATURAL;
  rx_common_mode : STRING;
  rx_cru_pre_divide_by : NATURAL;
  rx_datapath_protocol : STRING;
  rx_data_rate : NATURAL;
  rx_data_rate_remainder : NATURAL;
  rx_disable_auto_idle_insertion : STRING;
  rx_enable_bit_reversal : STRING;
  rx_enable_lock_to_data_sig : STRING;
  rx_enable_lock_to_refclk_sig : STRING;
  rx_enable_self_test_mode : STRING;
  rx_enable_true_complement_match_in_word_align : STRING;
  rx_force_signal_detect : STRING;
  rx_ppmselect : NATURAL;
  rx_rate_match_back_to_back : STRING;
  rx_rate_match_fifo_mode : STRING;
  rx_rate_match_ordered_set_based : STRING;
  rx_rate_match_pattern1 : STRING;
  rx_rate_match_pattern2 : STRING;
  rx_rate_match_pattern_size : NATURAL;
  rx_rate_match_skip_set_based : STRING;
  rx_run_length_enable : STRING;
  rx_signal_detect_threshold : NATURAL;
  rx_use_align_state_machine : STRING;
  rx_use_clkout : STRING;
  rx_use_coreclk : STRING;
  rx_use_deserializer_double_data_mode : STRING;
  rx_use_deskew_fifo : STRING;
  rx_use_double_data_mode : STRING;
  rx_use_pipe8b10binvpolarity : STRING;
  rx_use_rate_match_pattern1_only : STRING;
  transmitter_termination : STRING;
  tx_8b_10b_compatibility_mode : STRING;
  tx_8b_10b_mode : STRING;
  tx_allow_polarity_inversion : STRING;
  tx_analog_power : STRING;
  tx_channel_bonding : STRING;
  tx_channel_width : NATURAL;
  tx_common_mode : STRING;
  tx_data_rate : NATURAL;
  tx_data_rate_remainder : NATURAL;
  tx_enable_bit_reversal : STRING;
  tx_enable_idle_selection : STRING;
  tx_enable_self_test_mode : STRING;
  tx_refclk_divide_by : NATURAL;
  tx_transmit_protocol : STRING;
  tx_use_coreclk : STRING;
  tx_use_double_data_mode : STRING;
  tx_use_serializer_double_data_mode : STRING;
  use_calibration_block : STRING;
  vod_ctrl_setting : NATURAL
 );
 PORT (
   tx_forceelecidle : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
   rx_disperr : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   rx_patterndetect : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   rx_coreclk : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
   coreclkout : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_ctrldetect : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   pll_inclk : IN STD_LOGIC ;
   gxb_powerdown : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pipedatavalid : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
   tx_datain : IN STD_LOGIC_VECTOR (63 DOWNTO 0);
   rx_errdetect : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   pipephydonestatus : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
   rx_pll_locked : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
   rx_freqlocked : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
   cal_blk_clk : IN STD_LOGIC ;
   powerdn : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
   tx_dataout : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
   fixedclk : IN STD_LOGIC ;
   rx_datain : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
   pipeelecidle : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
   tx_coreclk : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
   tx_ctrlenable : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
   pipestatus : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
   rx_analogreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pipe8b10binvpolarity : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
   rx_syncstatus : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   rx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_forcedispcompliance : IN STD_LOGIC_VECTOR (3 DOWNTO 0);
   pll_locked : OUT STD_LOGIC_VECTOR (0 DOWNTO 0);
   rx_dataout : OUT STD_LOGIC_VECTOR (63 DOWNTO 0);
   tx_detectrxloop : IN STD_LOGIC_VECTOR (3 DOWNTO 0)
 );
 END COMPONENT;

BEGIN
 rx_disperr <= sub_wire0(7 DOWNTO 0);
 rx_patterndetect <= sub_wire1(7 DOWNTO 0);
 coreclkout <= sub_wire2(0 DOWNTO 0);
 rx_ctrldetect <= sub_wire3(7 DOWNTO 0);
 pipedatavalid <= sub_wire4(3 DOWNTO 0);
 rx_errdetect <= sub_wire5(7 DOWNTO 0);
 pipephydonestatus <= sub_wire6(3 DOWNTO 0);
 rx_pll_locked <= sub_wire7(3 DOWNTO 0);
 rx_freqlocked <= sub_wire8(3 DOWNTO 0);
 tx_dataout <= sub_wire9(3 DOWNTO 0);
 pipeelecidle <= sub_wire10(3 DOWNTO 0);
 pipestatus <= sub_wire11(11 DOWNTO 0);
 rx_syncstatus <= sub_wire12(7 DOWNTO 0);
 pll_locked <= sub_wire13(0 DOWNTO 0);
 rx_dataout <= sub_wire14(63 DOWNTO 0);

 alt2gxb_component : alt2gxb
 GENERIC MAP (
  cmu_pll_inclock_period => 10000,
  cmu_pll_loop_filter_resistor_control => 3,
  digitalreset_port_width => 1,
  enable_fast_recovery_pci_mode => "false",
  enable_pll_inclk_alt_drive_rx_cru => "true",
  enable_pll_inclk_drive_rx_cru => "true",
  en_local_clk_div_ctrl => "true",
  equalizer_ctrl_a_setting => 0,
  equalizer_ctrl_b_setting => 0,
  equalizer_ctrl_c_setting => 0,
  equalizer_ctrl_d_setting => 0,
  equalizer_ctrl_v_setting => 0,
  equalizer_dcgain_setting => 1,
  intended_device_family => "Stratix II GX",
  loopback_mode => "none",
  lpm_type => "alt2gxb",
  number_of_channels => 4,
  operation_mode => "duplex",
  preemphasis_ctrl_1stposttap_setting => 5,
  preemphasis_ctrl_2ndposttap_inv_setting => "false",
  preemphasis_ctrl_2ndposttap_setting => 0,
  preemphasis_ctrl_pretap_inv_setting => "false",
  preemphasis_ctrl_pretap_setting => 0,
  protocol => "pipe",
  receiver_termination => "oct_100_ohms",
  reconfig_dprio_mode => 0,
  reverse_loopback_mode => "rplb",
  rx_8b_10b_compatibility_mode => "true",
  rx_8b_10b_mode => "normal",
  rx_align_pattern => "0101111100",
  rx_align_pattern_length => 10,
  rx_allow_align_polarity_inversion => "false",
  rx_allow_pipe_polarity_inversion => "true",
  rx_bandwidth_mode => 1,
  rx_bitslip_enable => "false",
  rx_byte_ordering_mode => "none",
  rx_channel_bonding => "x4",
  rx_channel_width => 16,
  rx_common_mode => "0.9v",
  rx_cru_pre_divide_by => 1,
  rx_datapath_protocol => "pipe",
  rx_data_rate => 2500,
  rx_data_rate_remainder => 0,
  rx_disable_auto_idle_insertion => "false",
  rx_enable_bit_reversal => "false",
  rx_enable_lock_to_data_sig => "false",
  rx_enable_lock_to_refclk_sig => "false",
  rx_enable_self_test_mode => "false",
  rx_enable_true_complement_match_in_word_align => "false",
  rx_force_signal_detect => "false",
  rx_ppmselect => 8,
  rx_rate_match_back_to_back => "false",
  rx_rate_match_fifo_mode => "normal",
  rx_rate_match_ordered_set_based => "false",
  rx_rate_match_pattern1 => "11010000111010000011",
  rx_rate_match_pattern2 => "00101111000101111100",
  rx_rate_match_pattern_size => 20,
  rx_rate_match_skip_set_based => "false",
  rx_run_length_enable => "false",
  rx_signal_detect_threshold => 2,
  rx_use_align_state_machine => "true",
  rx_use_clkout => "false",
  rx_use_coreclk => "true",
  rx_use_deserializer_double_data_mode => "false",
  rx_use_deskew_fifo => "false",
  rx_use_double_data_mode => "true",
  rx_use_pipe8b10binvpolarity => "true",
  rx_use_rate_match_pattern1_only => "false",
  transmitter_termination => "oct_100_ohms",
  tx_8b_10b_compatibility_mode => "true",
  tx_8b_10b_mode => "normal",
  tx_allow_polarity_inversion => "false",
  tx_analog_power => "1.2v",
  tx_channel_bonding => "x4",
  tx_channel_width => 16,
  tx_common_mode => "0.6v",
  tx_data_rate => 2500,
  tx_data_rate_remainder => 0,
  tx_enable_bit_reversal => "false",
  tx_enable_idle_selection => "false",
  tx_enable_self_test_mode => "false",
  tx_refclk_divide_by => 1,
  tx_transmit_protocol => "pipe",
  tx_use_coreclk => "true",
  tx_use_double_data_mode => "true",
  tx_use_serializer_double_data_mode => "false",
  use_calibration_block => "true",
  vod_ctrl_setting => 4
 )
 PORT MAP (
  tx_forceelecidle => tx_forceelecidle,
  rx_coreclk => rx_coreclk,
  pll_inclk => pll_inclk,
  gxb_powerdown => gxb_powerdown,
  tx_datain => tx_datain,
  cal_blk_clk => cal_blk_clk,
  powerdn => powerdn,
  fixedclk => fixedclk,
  rx_datain => rx_datain,
  tx_coreclk => tx_coreclk,
  tx_ctrlenable => tx_ctrlenable,
  rx_analogreset => rx_analogreset,
  pipe8b10binvpolarity => pipe8b10binvpolarity,
  rx_digitalreset => rx_digitalreset,
  tx_digitalreset => tx_digitalreset,
  tx_forcedispcompliance => tx_forcedispcompliance,
  tx_detectrxloop => tx_detectrxloop,
  rx_disperr => sub_wire0,
  rx_patterndetect => sub_wire1,
  coreclkout => sub_wire2,
  rx_ctrldetect => sub_wire3,
  pipedatavalid => sub_wire4,
  rx_errdetect => sub_wire5,
  pipephydonestatus => sub_wire6,
  rx_pll_locked => sub_wire7,
  rx_freqlocked => sub_wire8,
  tx_dataout => sub_wire9,
  pipeelecidle => sub_wire10,
  pipestatus => sub_wire11,
  rx_syncstatus => sub_wire12,
  pll_locked => sub_wire13,
  rx_dataout => sub_wire14
 );



END SYN;

-- ============================================================
-- CNX file retrieval info
-- ============================================================
-- Retrieval info: PRIVATE: ALT_SIMLIB_GEN STRING "0"
-- Retrieval info: PRIVATE: INTENDED_DEVICE_FAMILY STRING "Stratix II GX"
-- Retrieval info: PRIVATE: RECONFIG_PROTOCOL STRING "BASIC"
-- Retrieval info: PRIVATE: RECONFIG_SUBPROTOCOL STRING "none"
-- Retrieval info: PRIVATE: RX_ENABLE_DC_COUPLING STRING "false"
-- Retrieval info: PRIVATE: RX_FORCE_SIGNAL_DETECT STRING "false"
-- Retrieval info: PRIVATE: RX_SIGNAL_DETECT_THRESHOLD NUMERIC "2"
-- Retrieval info: PRIVATE: SYNTH_WRAPPER_GEN_POSTFIX STRING "0"
-- Retrieval info: PRIVATE: VOD_CTRL_SETTING NUMERIC "800"
-- Retrieval info: PRIVATE: WIZ_DATA_RATE STRING "2500"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INCLK_FREQ_ARRAY STRING "100.0 125.0 200.0 250.0 400.0 500.0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_A STRING "2000"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_A_UNIT STRING "Mbps"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_B STRING "100.0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_B_UNIT STRING "MHz"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_SELECTION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK0_FREQ STRING "100"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK0_PROTOCOL STRING "PCI Express (PIPE)"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK1_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK1_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK2_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK2_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK3_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK3_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK4_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK4_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK5_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK5_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK6_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK6_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_ENABLE_EQUALIZER_CTRL NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EQUALIZER_CTRL_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EXTERNAL_RX_TERMINATION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EXTERNAL_TX_TERMINATION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_FORCE_DEFAULT_SETTINGS NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_INCLK_FREQ STRING "100"
-- Retrieval info: PRIVATE: WIZ_INCLK_FREQ_ARRAY STRING "100"
-- Retrieval info: PRIVATE: WIZ_INPUT_A STRING "2500"
-- Retrieval info: PRIVATE: WIZ_INPUT_A_UNIT STRING "Mbps"
-- Retrieval info: PRIVATE: WIZ_INPUT_B STRING "100"
-- Retrieval info: PRIVATE: WIZ_INPUT_B_UNIT STRING "MHz"
-- Retrieval info: PRIVATE: WIZ_INPUT_SELECTION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_1STPOSTTAP_SETTING NUMERIC "5"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_2NDPOSTTAP_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_PRETAP_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PROTOCOL STRING "PCI Express (PIPE)"
-- Retrieval info: PRIVATE: WIZ_RX_TERMINATION NUMERIC "100"
-- Retrieval info: PRIVATE: WIZ_RX_VCM STRING "0.85"
-- Retrieval info: PRIVATE: WIZ_SUBPROTOCOL STRING "x4"
-- Retrieval info: PRIVATE: WIZ_TX_TERMINATION NUMERIC "100"
-- Retrieval info: PRIVATE: WIZ_TX_VCM STRING "0.6"
-- Retrieval info: PRIVATE: WIZ_VCCHTX STRING "1.2"
-- Retrieval info: PRIVATE: WIZ_WORD_ALIGN_FLIP_PATTERN STRING "0"
-- Retrieval info: CONSTANT: CMU_PLL_INCLOCK_PERIOD NUMERIC "10000"
-- Retrieval info: CONSTANT: CMU_PLL_LOOP_FILTER_RESISTOR_CONTROL NUMERIC "3"
-- Retrieval info: CONSTANT: DIGITALRESET_PORT_WIDTH NUMERIC "1"
-- Retrieval info: CONSTANT: ENABLE_FAST_RECOVERY_PCI_MODE STRING "false"
-- Retrieval info: CONSTANT: ENABLE_PLL_INCLK_ALT_DRIVE_RX_CRU STRING "true"
-- Retrieval info: CONSTANT: ENABLE_PLL_INCLK_DRIVE_RX_CRU STRING "true"
-- Retrieval info: CONSTANT: EN_LOCAL_CLK_DIV_CTRL STRING "true"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_A_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_B_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_C_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_D_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_V_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_DCGAIN_SETTING NUMERIC "1"
-- Retrieval info: CONSTANT: INTENDED_DEVICE_FAMILY STRING "Stratix II GX"
-- Retrieval info: CONSTANT: LOOPBACK_MODE STRING "none"
-- Retrieval info: CONSTANT: LPM_TYPE STRING "alt2gxb"
-- Retrieval info: CONSTANT: NUMBER_OF_CHANNELS NUMERIC "4"
-- Retrieval info: CONSTANT: OPERATION_MODE STRING "duplex"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_1STPOSTTAP_SETTING NUMERIC "5"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_2NDPOSTTAP_INV_SETTING STRING "false"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_2NDPOSTTAP_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_PRETAP_INV_SETTING STRING "false"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_PRETAP_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: RECEIVER_TERMINATION STRING "oct_100_ohms"
-- Retrieval info: CONSTANT: RECONFIG_DPRIO_MODE NUMERIC "0"
-- Retrieval info: CONSTANT: REVERSE_LOOPBACK_MODE STRING "rplb"
-- Retrieval info: CONSTANT: RX_8B_10B_COMPATIBILITY_MODE STRING "true"
-- Retrieval info: CONSTANT: RX_8B_10B_MODE STRING "normal"
-- Retrieval info: CONSTANT: RX_ALIGN_PATTERN STRING "0101111100"
-- Retrieval info: CONSTANT: RX_ALIGN_PATTERN_LENGTH NUMERIC "10"
-- Retrieval info: CONSTANT: RX_ALLOW_ALIGN_POLARITY_INVERSION STRING "false"
-- Retrieval info: CONSTANT: RX_ALLOW_PIPE_POLARITY_INVERSION STRING "true"
-- Retrieval info: CONSTANT: RX_BANDWIDTH_MODE NUMERIC "1"
-- Retrieval info: CONSTANT: RX_BITSLIP_ENABLE STRING "false"
-- Retrieval info: CONSTANT: RX_BYTE_ORDERING_MODE STRING "none"
-- Retrieval info: CONSTANT: RX_CHANNEL_BONDING STRING "x4"
-- Retrieval info: CONSTANT: RX_CHANNEL_WIDTH NUMERIC "16"
-- Retrieval info: CONSTANT: RX_COMMON_MODE STRING "0.9v"
-- Retrieval info: CONSTANT: RX_CRU_PRE_DIVIDE_BY NUMERIC "1"
-- Retrieval info: CONSTANT: RX_DATAPATH_PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: RX_DATA_RATE NUMERIC "2500"
-- Retrieval info: CONSTANT: RX_DATA_RATE_REMAINDER NUMERIC "0"
-- Retrieval info: CONSTANT: RX_DISABLE_AUTO_IDLE_INSERTION STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_BIT_REVERSAL STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_LOCK_TO_DATA_SIG STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_LOCK_TO_REFCLK_SIG STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_SELF_TEST_MODE STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_TRUE_COMPLEMENT_MATCH_IN_WORD_ALIGN STRING "false"
-- Retrieval info: CONSTANT: RX_FORCE_SIGNAL_DETECT STRING "false"
-- Retrieval info: CONSTANT: RX_PPMSELECT NUMERIC "8"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_BACK_TO_BACK STRING "false"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_FIFO_MODE STRING "normal"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_ORDERED_SET_BASED STRING "false"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN1 STRING "11010000111010000011"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN2 STRING "00101111000101111100"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN_SIZE NUMERIC "20"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_SKIP_SET_BASED STRING "false"
-- Retrieval info: CONSTANT: RX_RUN_LENGTH_ENABLE STRING "false"
-- Retrieval info: CONSTANT: RX_SIGNAL_DETECT_THRESHOLD NUMERIC "2"
-- Retrieval info: CONSTANT: RX_USE_ALIGN_STATE_MACHINE STRING "true"
-- Retrieval info: CONSTANT: RX_USE_CLKOUT STRING "false"
-- Retrieval info: CONSTANT: RX_USE_CORECLK STRING "true"
-- Retrieval info: CONSTANT: RX_USE_DESERIALIZER_DOUBLE_DATA_MODE STRING "false"
-- Retrieval info: CONSTANT: RX_USE_DESKEW_FIFO STRING "false"
-- Retrieval info: CONSTANT: RX_USE_DOUBLE_DATA_MODE STRING "true"
-- Retrieval info: CONSTANT: RX_USE_PIPE8B10BINVPOLARITY STRING "true"
-- Retrieval info: CONSTANT: RX_USE_RATE_MATCH_PATTERN1_ONLY STRING "false"
-- Retrieval info: CONSTANT: TRANSMITTER_TERMINATION STRING "oct_100_ohms"
-- Retrieval info: CONSTANT: TX_8B_10B_COMPATIBILITY_MODE STRING "true"
-- Retrieval info: CONSTANT: TX_8B_10B_MODE STRING "normal"
-- Retrieval info: CONSTANT: TX_ALLOW_POLARITY_INVERSION STRING "false"
-- Retrieval info: CONSTANT: TX_ANALOG_POWER STRING "1.2v"
-- Retrieval info: CONSTANT: TX_CHANNEL_BONDING STRING "x4"
-- Retrieval info: CONSTANT: TX_CHANNEL_WIDTH NUMERIC "16"
-- Retrieval info: CONSTANT: TX_COMMON_MODE STRING "0.6v"
-- Retrieval info: CONSTANT: TX_DATA_RATE NUMERIC "2500"
-- Retrieval info: CONSTANT: TX_DATA_RATE_REMAINDER NUMERIC "0"
-- Retrieval info: CONSTANT: TX_ENABLE_BIT_REVERSAL STRING "false"
-- Retrieval info: CONSTANT: TX_ENABLE_IDLE_SELECTION STRING "false"
-- Retrieval info: CONSTANT: TX_ENABLE_SELF_TEST_MODE STRING "false"
-- Retrieval info: CONSTANT: TX_REFCLK_DIVIDE_BY NUMERIC "1"
-- Retrieval info: CONSTANT: TX_TRANSMIT_PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: TX_USE_CORECLK STRING "true"
-- Retrieval info: CONSTANT: TX_USE_DOUBLE_DATA_MODE STRING "true"
-- Retrieval info: CONSTANT: TX_USE_SERIALIZER_DOUBLE_DATA_MODE STRING "false"
-- Retrieval info: CONSTANT: USE_CALIBRATION_BLOCK STRING "true"
-- Retrieval info: CONSTANT: VOD_CTRL_SETTING NUMERIC "4"
-- Retrieval info: USED_PORT: cal_blk_clk 0 0 0 0 INPUT NODEFVAL "cal_blk_clk"
-- Retrieval info: USED_PORT: coreclkout 0 0 1 0 OUTPUT NODEFVAL "coreclkout[0..0]"
-- Retrieval info: USED_PORT: fixedclk 0 0 0 0 INPUT NODEFVAL "fixedclk"
-- Retrieval info: USED_PORT: gxb_powerdown 0 0 1 0 INPUT NODEFVAL "gxb_powerdown[0..0]"
-- Retrieval info: USED_PORT: pipe8b10binvpolarity 0 0 4 0 INPUT NODEFVAL "pipe8b10binvpolarity[3..0]"
-- Retrieval info: USED_PORT: pipedatavalid 0 0 4 0 OUTPUT NODEFVAL "pipedatavalid[3..0]"
-- Retrieval info: USED_PORT: pipeelecidle 0 0 4 0 OUTPUT NODEFVAL "pipeelecidle[3..0]"
-- Retrieval info: USED_PORT: pipephydonestatus 0 0 4 0 OUTPUT NODEFVAL "pipephydonestatus[3..0]"
-- Retrieval info: USED_PORT: pipestatus 0 0 12 0 OUTPUT NODEFVAL "pipestatus[11..0]"
-- Retrieval info: USED_PORT: pll_inclk 0 0 0 0 INPUT NODEFVAL "pll_inclk"
-- Retrieval info: USED_PORT: pll_locked 0 0 1 0 OUTPUT NODEFVAL "pll_locked[0..0]"
-- Retrieval info: USED_PORT: powerdn 0 0 8 0 INPUT NODEFVAL "powerdn[7..0]"
-- Retrieval info: USED_PORT: rx_analogreset 0 0 1 0 INPUT NODEFVAL "rx_analogreset[0..0]"
-- Retrieval info: USED_PORT: rx_coreclk 0 0 4 0 INPUT NODEFVAL "rx_coreclk[3..0]"
-- Retrieval info: USED_PORT: rx_ctrldetect 0 0 8 0 OUTPUT NODEFVAL "rx_ctrldetect[7..0]"
-- Retrieval info: USED_PORT: rx_datain 0 0 4 0 INPUT NODEFVAL "rx_datain[3..0]"
-- Retrieval info: USED_PORT: rx_dataout 0 0 64 0 OUTPUT NODEFVAL "rx_dataout[63..0]"
-- Retrieval info: USED_PORT: rx_digitalreset 0 0 1 0 INPUT NODEFVAL "rx_digitalreset[0..0]"
-- Retrieval info: USED_PORT: rx_disperr 0 0 8 0 OUTPUT NODEFVAL "rx_disperr[7..0]"
-- Retrieval info: USED_PORT: rx_errdetect 0 0 8 0 OUTPUT NODEFVAL "rx_errdetect[7..0]"
-- Retrieval info: USED_PORT: rx_freqlocked 0 0 4 0 OUTPUT NODEFVAL "rx_freqlocked[3..0]"
-- Retrieval info: USED_PORT: rx_patterndetect 0 0 8 0 OUTPUT NODEFVAL "rx_patterndetect[7..0]"
-- Retrieval info: USED_PORT: rx_pll_locked 0 0 4 0 OUTPUT NODEFVAL "rx_pll_locked[3..0]"
-- Retrieval info: USED_PORT: rx_syncstatus 0 0 8 0 OUTPUT NODEFVAL "rx_syncstatus[7..0]"
-- Retrieval info: USED_PORT: tx_coreclk 0 0 4 0 INPUT NODEFVAL "tx_coreclk[3..0]"
-- Retrieval info: USED_PORT: tx_ctrlenable 0 0 8 0 INPUT NODEFVAL "tx_ctrlenable[7..0]"
-- Retrieval info: USED_PORT: tx_datain 0 0 64 0 INPUT NODEFVAL "tx_datain[63..0]"
-- Retrieval info: USED_PORT: tx_dataout 0 0 4 0 OUTPUT NODEFVAL "tx_dataout[3..0]"
-- Retrieval info: USED_PORT: tx_detectrxloop 0 0 4 0 INPUT NODEFVAL "tx_detectrxloop[3..0]"
-- Retrieval info: USED_PORT: tx_digitalreset 0 0 1 0 INPUT NODEFVAL "tx_digitalreset[0..0]"
-- Retrieval info: USED_PORT: tx_forcedispcompliance 0 0 4 0 INPUT NODEFVAL "tx_forcedispcompliance[3..0]"
-- Retrieval info: USED_PORT: tx_forceelecidle 0 0 4 0 INPUT NODEFVAL "tx_forceelecidle[3..0]"
-- Retrieval info: CONNECT: @fixedclk 0 0 0 0 fixedclk 0 0 0 0
-- Retrieval info: CONNECT: @tx_detectrxloop 0 0 4 0 tx_detectrxloop 0 0 4 0
-- Retrieval info: CONNECT: @tx_forcedispcompliance 0 0 4 0 tx_forcedispcompliance 0 0 4 0
-- Retrieval info: CONNECT: rx_patterndetect 0 0 8 0 @rx_patterndetect 0 0 8 0
-- Retrieval info: CONNECT: pipedatavalid 0 0 4 0 @pipedatavalid 0 0 4 0
-- Retrieval info: CONNECT: @rx_analogreset 0 0 1 0 rx_analogreset 0 0 1 0
-- Retrieval info: CONNECT: @powerdn 0 0 8 0 powerdn 0 0 8 0
-- Retrieval info: CONNECT: pipeelecidle 0 0 4 0 @pipeelecidle 0 0 4 0
-- Retrieval info: CONNECT: rx_ctrldetect 0 0 8 0 @rx_ctrldetect 0 0 8 0
-- Retrieval info: CONNECT: @gxb_powerdown 0 0 1 0 gxb_powerdown 0 0 1 0
-- Retrieval info: CONNECT: rx_dataout 0 0 64 0 @rx_dataout 0 0 64 0
-- Retrieval info: CONNECT: @cal_blk_clk 0 0 0 0 cal_blk_clk 0 0 0 0
-- Retrieval info: CONNECT: pipestatus 0 0 12 0 @pipestatus 0 0 12 0
-- Retrieval info: CONNECT: rx_errdetect 0 0 8 0 @rx_errdetect 0 0 8 0
-- Retrieval info: CONNECT: @tx_coreclk 0 0 4 0 tx_coreclk 0 0 4 0
-- Retrieval info: CONNECT: @tx_digitalreset 0 0 1 0 tx_digitalreset 0 0 1 0
-- Retrieval info: CONNECT: rx_pll_locked 0 0 4 0 @rx_pll_locked 0 0 4 0
-- Retrieval info: CONNECT: coreclkout 0 0 1 0 @coreclkout 0 0 1 0
-- Retrieval info: CONNECT: rx_syncstatus 0 0 8 0 @rx_syncstatus 0 0 8 0
-- Retrieval info: CONNECT: @pipe8b10binvpolarity 0 0 4 0 pipe8b10binvpolarity 0 0 4 0
-- Retrieval info: CONNECT: @rx_coreclk 0 0 4 0 rx_coreclk 0 0 4 0
-- Retrieval info: CONNECT: pll_locked 0 0 1 0 @pll_locked 0 0 1 0
-- Retrieval info: CONNECT: @rx_digitalreset 0 0 1 0 rx_digitalreset 0 0 1 0
-- Retrieval info: CONNECT: @pll_inclk 0 0 0 0 pll_inclk 0 0 0 0
-- Retrieval info: CONNECT: @tx_ctrlenable 0 0 8 0 tx_ctrlenable 0 0 8 0
-- Retrieval info: CONNECT: tx_dataout 0 0 4 0 @tx_dataout 0 0 4 0
-- Retrieval info: CONNECT: @tx_datain 0 0 64 0 tx_datain 0 0 64 0
-- Retrieval info: CONNECT: rx_freqlocked 0 0 4 0 @rx_freqlocked 0 0 4 0
-- Retrieval info: CONNECT: @rx_datain 0 0 4 0 rx_datain 0 0 4 0
-- Retrieval info: CONNECT: rx_disperr 0 0 8 0 @rx_disperr 0 0 8 0
-- Retrieval info: CONNECT: pipephydonestatus 0 0 4 0 @pipephydonestatus 0 0 4 0
-- Retrieval info: CONNECT: @tx_forceelecidle 0 0 4 0 tx_forceelecidle 0 0 4 0
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.x4.vhd TRUE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.x4.inc FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.x4.cmp FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.x4.bsf FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.x4_inst.vhd FALSE FALSE
-- megafunction wizard: %ALT2GXB%
-- GENERATION: STANDARD
-- VERSION: WM1.0
-- MODULE: alt2gxb

-- ============================================================
-- File Name: alt2gxb_s2gx.vhd
-- Megafunction Name(s):
-- alt2gxb
--
-- Simulation Library Files(s):
--
-- ============================================================
-- ************************************************************
-- THIS IS A WIZARD-GENERATED FILE. DO NOT EDIT THIS FILE!
--
-- 7.1 Build 178 06/25/2007 SP 1 SJ Full Version
-- ************************************************************


--Copyright (C) 1991-2007 Altera Corporation
--Your use of Altera Corporation's design tools, logic functions 
--and other software and tools, and its AMPP partner logic
--functions, and any output files from any of the foregoing
--(including device programming or simulation files), and any
--associated documentation or information are expressly subject
--to the terms and conditions of the Altera Program License
--Subscription Agreement, Altera MegaCore Function License
--Agreement, or other applicable license agreement, including,
--without limitation, that your use is for the sole purpose of
--programming logic devices manufactured by Altera and sold by
--Altera or its authorized distributors. Please refer to the
--applicable agreement for further details.


LIBRARY ieee;
USE ieee.std_logic_1164.all;

ENTITY alt2gxb_s2gx_x8 IS
 PORT
 (
  cal_blk_clk : IN STD_LOGIC ;
  fixedclk : IN STD_LOGIC ;
  gxb_powerdown : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  pipe8b10binvpolarity : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
  pll_inclk : IN STD_LOGIC ;
  powerdn : IN STD_LOGIC_VECTOR (15 DOWNTO 0);
  rx_analogreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  rx_coreclk : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_datain : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_coreclk : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
  tx_ctrlenable : IN STD_LOGIC_VECTOR (15 DOWNTO 0);
  tx_datain : IN STD_LOGIC_VECTOR (127 DOWNTO 0);
  tx_detectrxloop : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
  tx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
  tx_forcedispcompliance : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
  tx_forceelecidle : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
  coreclkout : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
  pipedatavalid : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  pipeelecidle : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  pipephydonestatus : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  pipestatus : OUT STD_LOGIC_VECTOR (23 DOWNTO 0);
  pll_locked : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
  rx_ctrldetect : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
  rx_dataout : OUT STD_LOGIC_VECTOR (127 DOWNTO 0);
  rx_disperr : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
  rx_errdetect : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
  rx_freqlocked : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_patterndetect : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
  rx_pll_locked : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
  rx_syncstatus : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
  tx_dataout : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
 );
END alt2gxb_s2gx_x8;


ARCHITECTURE SYN OF alt2gxb_s2gx_x8 IS

 SIGNAL sub_wire0 : STD_LOGIC_VECTOR (15 DOWNTO 0);
 SIGNAL sub_wire1 : STD_LOGIC_VECTOR (15 DOWNTO 0);
 SIGNAL sub_wire2 : STD_LOGIC_VECTOR (1 DOWNTO 0);
 SIGNAL sub_wire3 : STD_LOGIC_VECTOR (15 DOWNTO 0);
 SIGNAL sub_wire4 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire5 : STD_LOGIC_VECTOR (15 DOWNTO 0);
 SIGNAL sub_wire6 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire7 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire8 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire9 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire10 : STD_LOGIC_VECTOR (7 DOWNTO 0);
 SIGNAL sub_wire11 : STD_LOGIC_VECTOR (23 DOWNTO 0);
 SIGNAL sub_wire12 : STD_LOGIC_VECTOR (15 DOWNTO 0);
 SIGNAL sub_wire13 : STD_LOGIC_VECTOR (1 DOWNTO 0);
 SIGNAL sub_wire14 : STD_LOGIC_VECTOR (127 DOWNTO 0);



 COMPONENT alt2gxb
 GENERIC (
  cmu_pll_inclock_period : NATURAL;
  cmu_pll_loop_filter_resistor_control : NATURAL;
  digitalreset_port_width : NATURAL;
  enable_fast_recovery_pci_mode : STRING;
  enable_pll_inclk_alt_drive_rx_cru : STRING;
  enable_pll_inclk_drive_rx_cru : STRING;
  en_local_clk_div_ctrl : STRING;
  equalizer_ctrl_a_setting : NATURAL;
  equalizer_ctrl_b_setting : NATURAL;
  equalizer_ctrl_c_setting : NATURAL;
  equalizer_ctrl_d_setting : NATURAL;
  equalizer_ctrl_v_setting : NATURAL;
  equalizer_dcgain_setting : NATURAL;
  intended_device_family : STRING;
  loopback_mode : STRING;
  lpm_type : STRING;
  number_of_channels : NATURAL;
  operation_mode : STRING;
  preemphasis_ctrl_1stposttap_setting : NATURAL;
  preemphasis_ctrl_2ndposttap_inv_setting : STRING;
  preemphasis_ctrl_2ndposttap_setting : NATURAL;
  preemphasis_ctrl_pretap_inv_setting : STRING;
  preemphasis_ctrl_pretap_setting : NATURAL;
  protocol : STRING;
  receiver_termination : STRING;
  reconfig_dprio_mode : NATURAL;
  reverse_loopback_mode : STRING;
  rx_8b_10b_compatibility_mode : STRING;
  rx_8b_10b_mode : STRING;
  rx_align_pattern : STRING;
  rx_align_pattern_length : NATURAL;
  rx_allow_align_polarity_inversion : STRING;
  rx_allow_pipe_polarity_inversion : STRING;
  rx_bandwidth_mode : NATURAL;
  rx_bitslip_enable : STRING;
  rx_byte_ordering_mode : STRING;
  rx_channel_bonding : STRING;
  rx_channel_width : NATURAL;
  rx_common_mode : STRING;
  rx_cru_pre_divide_by : NATURAL;
  rx_datapath_protocol : STRING;
  rx_data_rate : NATURAL;
  rx_data_rate_remainder : NATURAL;
  rx_disable_auto_idle_insertion : STRING;
  rx_enable_bit_reversal : STRING;
  rx_enable_lock_to_data_sig : STRING;
  rx_enable_lock_to_refclk_sig : STRING;
  rx_enable_self_test_mode : STRING;
  rx_enable_true_complement_match_in_word_align : STRING;
  rx_force_signal_detect : STRING;
  rx_ppmselect : NATURAL;
  rx_rate_match_back_to_back : STRING;
  rx_rate_match_fifo_mode : STRING;
  rx_rate_match_ordered_set_based : STRING;
  rx_rate_match_pattern1 : STRING;
  rx_rate_match_pattern2 : STRING;
  rx_rate_match_pattern_size : NATURAL;
  rx_rate_match_skip_set_based : STRING;
  rx_run_length_enable : STRING;
  rx_signal_detect_threshold : NATURAL;
  rx_use_align_state_machine : STRING;
  rx_use_clkout : STRING;
  rx_use_coreclk : STRING;
  rx_use_deserializer_double_data_mode : STRING;
  rx_use_deskew_fifo : STRING;
  rx_use_double_data_mode : STRING;
  rx_use_pipe8b10binvpolarity : STRING;
  rx_use_rate_match_pattern1_only : STRING;
  transmitter_termination : STRING;
  tx_8b_10b_compatibility_mode : STRING;
  tx_8b_10b_mode : STRING;
  tx_allow_polarity_inversion : STRING;
  tx_analog_power : STRING;
  tx_channel_bonding : STRING;
  tx_channel_width : NATURAL;
  tx_common_mode : STRING;
  tx_data_rate : NATURAL;
  tx_data_rate_remainder : NATURAL;
  tx_enable_bit_reversal : STRING;
  tx_enable_idle_selection : STRING;
  tx_enable_self_test_mode : STRING;
  tx_refclk_divide_by : NATURAL;
  tx_transmit_protocol : STRING;
  tx_use_coreclk : STRING;
  tx_use_double_data_mode : STRING;
  tx_use_serializer_double_data_mode : STRING;
  use_calibration_block : STRING;
  vod_ctrl_setting : NATURAL
 );
 PORT (
   tx_forceelecidle : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
   rx_disperr : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
   rx_patterndetect : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
   rx_coreclk : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
   coreclkout : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   rx_ctrldetect : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
   pll_inclk : IN STD_LOGIC ;
   gxb_powerdown : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pipedatavalid : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   tx_datain : IN STD_LOGIC_VECTOR (127 DOWNTO 0);
   rx_errdetect : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
   pipephydonestatus : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   rx_pll_locked : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   rx_freqlocked : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   cal_blk_clk : IN STD_LOGIC ;
   powerdn : IN STD_LOGIC_VECTOR (15 DOWNTO 0);
   tx_dataout : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   fixedclk : IN STD_LOGIC ;
   rx_datain : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
   pipeelecidle : OUT STD_LOGIC_VECTOR (7 DOWNTO 0);
   tx_coreclk : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
   tx_ctrlenable : IN STD_LOGIC_VECTOR (15 DOWNTO 0);
   pipestatus : OUT STD_LOGIC_VECTOR (23 DOWNTO 0);
   rx_analogreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   pipe8b10binvpolarity : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
   rx_syncstatus : OUT STD_LOGIC_VECTOR (15 DOWNTO 0);
   rx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_digitalreset : IN STD_LOGIC_VECTOR (0 DOWNTO 0);
   tx_forcedispcompliance : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
   pll_locked : OUT STD_LOGIC_VECTOR (1 DOWNTO 0);
   rx_dataout : OUT STD_LOGIC_VECTOR (127 DOWNTO 0);
   tx_detectrxloop : IN STD_LOGIC_VECTOR (7 DOWNTO 0)
 );
 END COMPONENT;

BEGIN
 rx_disperr <= sub_wire0(15 DOWNTO 0);
 rx_patterndetect <= sub_wire1(15 DOWNTO 0);
 coreclkout <= sub_wire2(1 DOWNTO 0);
 rx_ctrldetect <= sub_wire3(15 DOWNTO 0);
 pipedatavalid <= sub_wire4(7 DOWNTO 0);
 rx_errdetect <= sub_wire5(15 DOWNTO 0);
 pipephydonestatus <= sub_wire6(7 DOWNTO 0);
 rx_pll_locked <= sub_wire7(7 DOWNTO 0);
 rx_freqlocked <= sub_wire8(7 DOWNTO 0);
 tx_dataout <= sub_wire9(7 DOWNTO 0);
 pipeelecidle <= sub_wire10(7 DOWNTO 0);
 pipestatus <= sub_wire11(23 DOWNTO 0);
 rx_syncstatus <= sub_wire12(15 DOWNTO 0);
 pll_locked <= sub_wire13(1 DOWNTO 0);
 rx_dataout <= sub_wire14(127 DOWNTO 0);

 alt2gxb_component : alt2gxb
 GENERIC MAP (
  cmu_pll_inclock_period => 10000,
  cmu_pll_loop_filter_resistor_control => 3,
  digitalreset_port_width => 1,
  enable_fast_recovery_pci_mode => "false",
  enable_pll_inclk_alt_drive_rx_cru => "true",
  enable_pll_inclk_drive_rx_cru => "true",
  en_local_clk_div_ctrl => "true",
  equalizer_ctrl_a_setting => 0,
  equalizer_ctrl_b_setting => 0,
  equalizer_ctrl_c_setting => 0,
  equalizer_ctrl_d_setting => 0,
  equalizer_ctrl_v_setting => 0,
  equalizer_dcgain_setting => 1,
  intended_device_family => "Stratix II GX",
  loopback_mode => "none",
  lpm_type => "alt2gxb",
  number_of_channels => 8,
  operation_mode => "duplex",
  preemphasis_ctrl_1stposttap_setting => 5,
  preemphasis_ctrl_2ndposttap_inv_setting => "false",
  preemphasis_ctrl_2ndposttap_setting => 0,
  preemphasis_ctrl_pretap_inv_setting => "false",
  preemphasis_ctrl_pretap_setting => 0,
  protocol => "pipe",
  receiver_termination => "oct_100_ohms",
  reconfig_dprio_mode => 0,
  reverse_loopback_mode => "rplb",
  rx_8b_10b_compatibility_mode => "true",
  rx_8b_10b_mode => "normal",
  rx_align_pattern => "0101111100",
  rx_align_pattern_length => 10,
  rx_allow_align_polarity_inversion => "false",
  rx_allow_pipe_polarity_inversion => "true",
  rx_bandwidth_mode => 1,
  rx_bitslip_enable => "false",
  rx_byte_ordering_mode => "none",
  rx_channel_bonding => "x8",
  rx_channel_width => 16,
  rx_common_mode => "0.9v",
  rx_cru_pre_divide_by => 1,
  rx_datapath_protocol => "pipe",
  rx_data_rate => 2500,
  rx_data_rate_remainder => 0,
  rx_disable_auto_idle_insertion => "false",
  rx_enable_bit_reversal => "false",
  rx_enable_lock_to_data_sig => "false",
  rx_enable_lock_to_refclk_sig => "false",
  rx_enable_self_test_mode => "false",
  rx_enable_true_complement_match_in_word_align => "false",
  rx_force_signal_detect => "false",
  rx_ppmselect => 8,
  rx_rate_match_back_to_back => "false",
  rx_rate_match_fifo_mode => "normal",
  rx_rate_match_ordered_set_based => "false",
  rx_rate_match_pattern1 => "11010000111010000011",
  rx_rate_match_pattern2 => "00101111000101111100",
  rx_rate_match_pattern_size => 20,
  rx_rate_match_skip_set_based => "false",
  rx_run_length_enable => "false",
  rx_signal_detect_threshold => 2,
  rx_use_align_state_machine => "true",
  rx_use_clkout => "false",
  rx_use_coreclk => "true",
  rx_use_deserializer_double_data_mode => "false",
  rx_use_deskew_fifo => "false",
  rx_use_double_data_mode => "true",
  rx_use_pipe8b10binvpolarity => "true",
  rx_use_rate_match_pattern1_only => "false",
  transmitter_termination => "oct_100_ohms",
  tx_8b_10b_compatibility_mode => "true",
  tx_8b_10b_mode => "normal",
  tx_allow_polarity_inversion => "false",
  tx_analog_power => "1.2v",
  tx_channel_bonding => "x8",
  tx_channel_width => 16,
  tx_common_mode => "0.6v",
  tx_data_rate => 2500,
  tx_data_rate_remainder => 0,
  tx_enable_bit_reversal => "false",
  tx_enable_idle_selection => "false",
  tx_enable_self_test_mode => "false",
  tx_refclk_divide_by => 1,
  tx_transmit_protocol => "pipe",
  tx_use_coreclk => "true",
  tx_use_double_data_mode => "true",
  tx_use_serializer_double_data_mode => "false",
  use_calibration_block => "true",
  vod_ctrl_setting => 4
 )
 PORT MAP (
  tx_forceelecidle => tx_forceelecidle,
  rx_coreclk => rx_coreclk,
  pll_inclk => pll_inclk,
  gxb_powerdown => gxb_powerdown,
  tx_datain => tx_datain,
  cal_blk_clk => cal_blk_clk,
  powerdn => powerdn,
  fixedclk => fixedclk,
  rx_datain => rx_datain,
  tx_coreclk => tx_coreclk,
  tx_ctrlenable => tx_ctrlenable,
  rx_analogreset => rx_analogreset,
  pipe8b10binvpolarity => pipe8b10binvpolarity,
  rx_digitalreset => rx_digitalreset,
  tx_digitalreset => tx_digitalreset,
  tx_forcedispcompliance => tx_forcedispcompliance,
  tx_detectrxloop => tx_detectrxloop,
  rx_disperr => sub_wire0,
  rx_patterndetect => sub_wire1,
  coreclkout => sub_wire2,
  rx_ctrldetect => sub_wire3,
  pipedatavalid => sub_wire4,
  rx_errdetect => sub_wire5,
  pipephydonestatus => sub_wire6,
  rx_pll_locked => sub_wire7,
  rx_freqlocked => sub_wire8,
  tx_dataout => sub_wire9,
  pipeelecidle => sub_wire10,
  pipestatus => sub_wire11,
  rx_syncstatus => sub_wire12,
  pll_locked => sub_wire13,
  rx_dataout => sub_wire14
 );



END SYN;

-- ============================================================
-- CNX file retrieval info
-- ============================================================
-- Retrieval info: PRIVATE: ALT_SIMLIB_GEN STRING "0"
-- Retrieval info: PRIVATE: INTENDED_DEVICE_FAMILY STRING "Stratix II GX"
-- Retrieval info: PRIVATE: RECONFIG_PROTOCOL STRING "BASIC"
-- Retrieval info: PRIVATE: RECONFIG_SUBPROTOCOL STRING "none"
-- Retrieval info: PRIVATE: RX_ENABLE_DC_COUPLING STRING "false"
-- Retrieval info: PRIVATE: RX_FORCE_SIGNAL_DETECT STRING "false"
-- Retrieval info: PRIVATE: RX_SIGNAL_DETECT_THRESHOLD NUMERIC "2"
-- Retrieval info: PRIVATE: SYNTH_WRAPPER_GEN_POSTFIX STRING "0"
-- Retrieval info: PRIVATE: VOD_CTRL_SETTING NUMERIC "800"
-- Retrieval info: PRIVATE: WIZ_DATA_RATE STRING "2500"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INCLK_FREQ_ARRAY STRING "100.0 125.0 200.0 250.0 400.0 500.0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_A STRING "2000"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_A_UNIT STRING "Mbps"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_B STRING "100.0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_B_UNIT STRING "MHz"
-- Retrieval info: PRIVATE: WIZ_DPRIO_INPUT_SELECTION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK0_FREQ STRING "100"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK0_PROTOCOL STRING "PCI Express (PIPE)"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK1_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK1_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK2_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK2_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK3_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK3_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK4_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK4_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK5_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK5_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK6_FREQ STRING "250"
-- Retrieval info: PRIVATE: WIZ_DPRIO_REF_CLK6_PROTOCOL STRING "Basic"
-- Retrieval info: PRIVATE: WIZ_ENABLE_EQUALIZER_CTRL NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EQUALIZER_CTRL_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EXTERNAL_RX_TERMINATION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_EXTERNAL_TX_TERMINATION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_FORCE_DEFAULT_SETTINGS NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_INCLK_FREQ STRING "100"
-- Retrieval info: PRIVATE: WIZ_INCLK_FREQ_ARRAY STRING "100"
-- Retrieval info: PRIVATE: WIZ_INPUT_A STRING "2500"
-- Retrieval info: PRIVATE: WIZ_INPUT_A_UNIT STRING "Mbps"
-- Retrieval info: PRIVATE: WIZ_INPUT_B STRING "100"
-- Retrieval info: PRIVATE: WIZ_INPUT_B_UNIT STRING "MHz"
-- Retrieval info: PRIVATE: WIZ_INPUT_SELECTION NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_1STPOSTTAP_SETTING NUMERIC "5"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_2NDPOSTTAP_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PREEMPHASIS_CTRL_PRETAP_SETTING NUMERIC "0"
-- Retrieval info: PRIVATE: WIZ_PROTOCOL STRING "PCI Express (PIPE)"
-- Retrieval info: PRIVATE: WIZ_RX_TERMINATION NUMERIC "100"
-- Retrieval info: PRIVATE: WIZ_RX_VCM STRING "0.85"
-- Retrieval info: PRIVATE: WIZ_SUBPROTOCOL STRING "x8"
-- Retrieval info: PRIVATE: WIZ_TX_TERMINATION NUMERIC "100"
-- Retrieval info: PRIVATE: WIZ_TX_VCM STRING "0.6"
-- Retrieval info: PRIVATE: WIZ_VCCHTX STRING "1.2"
-- Retrieval info: PRIVATE: WIZ_WORD_ALIGN_FLIP_PATTERN STRING "0"
-- Retrieval info: CONSTANT: CMU_PLL_INCLOCK_PERIOD NUMERIC "10000"
-- Retrieval info: CONSTANT: CMU_PLL_LOOP_FILTER_RESISTOR_CONTROL NUMERIC "3"
-- Retrieval info: CONSTANT: DIGITALRESET_PORT_WIDTH NUMERIC "1"
-- Retrieval info: CONSTANT: ENABLE_FAST_RECOVERY_PCI_MODE STRING "false"
-- Retrieval info: CONSTANT: ENABLE_PLL_INCLK_ALT_DRIVE_RX_CRU STRING "true"
-- Retrieval info: CONSTANT: ENABLE_PLL_INCLK_DRIVE_RX_CRU STRING "true"
-- Retrieval info: CONSTANT: EN_LOCAL_CLK_DIV_CTRL STRING "true"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_A_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_B_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_C_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_D_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_CTRL_V_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: EQUALIZER_DCGAIN_SETTING NUMERIC "1"
-- Retrieval info: CONSTANT: INTENDED_DEVICE_FAMILY STRING "Stratix II GX"
-- Retrieval info: CONSTANT: LOOPBACK_MODE STRING "none"
-- Retrieval info: CONSTANT: LPM_TYPE STRING "alt2gxb"
-- Retrieval info: CONSTANT: NUMBER_OF_CHANNELS NUMERIC "8"
-- Retrieval info: CONSTANT: OPERATION_MODE STRING "duplex"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_1STPOSTTAP_SETTING NUMERIC "5"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_2NDPOSTTAP_INV_SETTING STRING "false"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_2NDPOSTTAP_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_PRETAP_INV_SETTING STRING "false"
-- Retrieval info: CONSTANT: PREEMPHASIS_CTRL_PRETAP_SETTING NUMERIC "0"
-- Retrieval info: CONSTANT: PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: RECEIVER_TERMINATION STRING "oct_100_ohms"
-- Retrieval info: CONSTANT: RECONFIG_DPRIO_MODE NUMERIC "0"
-- Retrieval info: CONSTANT: REVERSE_LOOPBACK_MODE STRING "rplb"
-- Retrieval info: CONSTANT: RX_8B_10B_COMPATIBILITY_MODE STRING "true"
-- Retrieval info: CONSTANT: RX_8B_10B_MODE STRING "normal"
-- Retrieval info: CONSTANT: RX_ALIGN_PATTERN STRING "0101111100"
-- Retrieval info: CONSTANT: RX_ALIGN_PATTERN_LENGTH NUMERIC "10"
-- Retrieval info: CONSTANT: RX_ALLOW_ALIGN_POLARITY_INVERSION STRING "false"
-- Retrieval info: CONSTANT: RX_ALLOW_PIPE_POLARITY_INVERSION STRING "true"
-- Retrieval info: CONSTANT: RX_BANDWIDTH_MODE NUMERIC "1"
-- Retrieval info: CONSTANT: RX_BITSLIP_ENABLE STRING "false"
-- Retrieval info: CONSTANT: RX_BYTE_ORDERING_MODE STRING "none"
-- Retrieval info: CONSTANT: RX_CHANNEL_BONDING STRING "x8"
-- Retrieval info: CONSTANT: RX_CHANNEL_WIDTH NUMERIC "16"
-- Retrieval info: CONSTANT: RX_COMMON_MODE STRING "0.9v"
-- Retrieval info: CONSTANT: RX_CRU_PRE_DIVIDE_BY NUMERIC "1"
-- Retrieval info: CONSTANT: RX_DATAPATH_PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: RX_DATA_RATE NUMERIC "2500"
-- Retrieval info: CONSTANT: RX_DATA_RATE_REMAINDER NUMERIC "0"
-- Retrieval info: CONSTANT: RX_DISABLE_AUTO_IDLE_INSERTION STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_BIT_REVERSAL STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_LOCK_TO_DATA_SIG STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_LOCK_TO_REFCLK_SIG STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_SELF_TEST_MODE STRING "false"
-- Retrieval info: CONSTANT: RX_ENABLE_TRUE_COMPLEMENT_MATCH_IN_WORD_ALIGN STRING "false"
-- Retrieval info: CONSTANT: RX_FORCE_SIGNAL_DETECT STRING "false"
-- Retrieval info: CONSTANT: RX_PPMSELECT NUMERIC "8"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_BACK_TO_BACK STRING "false"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_FIFO_MODE STRING "normal"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_ORDERED_SET_BASED STRING "false"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN1 STRING "11010000111010000011"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN2 STRING "00101111000101111100"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_PATTERN_SIZE NUMERIC "20"
-- Retrieval info: CONSTANT: RX_RATE_MATCH_SKIP_SET_BASED STRING "false"
-- Retrieval info: CONSTANT: RX_RUN_LENGTH_ENABLE STRING "false"
-- Retrieval info: CONSTANT: RX_SIGNAL_DETECT_THRESHOLD NUMERIC "2"
-- Retrieval info: CONSTANT: RX_USE_ALIGN_STATE_MACHINE STRING "true"
-- Retrieval info: CONSTANT: RX_USE_CLKOUT STRING "false"
-- Retrieval info: CONSTANT: RX_USE_CORECLK STRING "true"
-- Retrieval info: CONSTANT: RX_USE_DESERIALIZER_DOUBLE_DATA_MODE STRING "false"
-- Retrieval info: CONSTANT: RX_USE_DESKEW_FIFO STRING "false"
-- Retrieval info: CONSTANT: RX_USE_DOUBLE_DATA_MODE STRING "true"
-- Retrieval info: CONSTANT: RX_USE_PIPE8B10BINVPOLARITY STRING "true"
-- Retrieval info: CONSTANT: RX_USE_RATE_MATCH_PATTERN1_ONLY STRING "false"
-- Retrieval info: CONSTANT: TRANSMITTER_TERMINATION STRING "oct_100_ohms"
-- Retrieval info: CONSTANT: TX_8B_10B_COMPATIBILITY_MODE STRING "true"
-- Retrieval info: CONSTANT: TX_8B_10B_MODE STRING "normal"
-- Retrieval info: CONSTANT: TX_ALLOW_POLARITY_INVERSION STRING "false"
-- Retrieval info: CONSTANT: TX_ANALOG_POWER STRING "1.2v"
-- Retrieval info: CONSTANT: TX_CHANNEL_BONDING STRING "x8"
-- Retrieval info: CONSTANT: TX_CHANNEL_WIDTH NUMERIC "16"
-- Retrieval info: CONSTANT: TX_COMMON_MODE STRING "0.6v"
-- Retrieval info: CONSTANT: TX_DATA_RATE NUMERIC "2500"
-- Retrieval info: CONSTANT: TX_DATA_RATE_REMAINDER NUMERIC "0"
-- Retrieval info: CONSTANT: TX_ENABLE_BIT_REVERSAL STRING "false"
-- Retrieval info: CONSTANT: TX_ENABLE_IDLE_SELECTION STRING "false"
-- Retrieval info: CONSTANT: TX_ENABLE_SELF_TEST_MODE STRING "false"
-- Retrieval info: CONSTANT: TX_REFCLK_DIVIDE_BY NUMERIC "1"
-- Retrieval info: CONSTANT: TX_TRANSMIT_PROTOCOL STRING "pipe"
-- Retrieval info: CONSTANT: TX_USE_CORECLK STRING "true"
-- Retrieval info: CONSTANT: TX_USE_DOUBLE_DATA_MODE STRING "true"
-- Retrieval info: CONSTANT: TX_USE_SERIALIZER_DOUBLE_DATA_MODE STRING "false"
-- Retrieval info: CONSTANT: USE_CALIBRATION_BLOCK STRING "true"
-- Retrieval info: CONSTANT: VOD_CTRL_SETTING NUMERIC "4"
-- Retrieval info: USED_PORT: cal_blk_clk 0 0 0 0 INPUT NODEFVAL "cal_blk_clk"
-- Retrieval info: USED_PORT: coreclkout 0 0 2 0 OUTPUT NODEFVAL "coreclkout[1..0]"
-- Retrieval info: USED_PORT: fixedclk 0 0 0 0 INPUT NODEFVAL "fixedclk"
-- Retrieval info: USED_PORT: gxb_powerdown 0 0 1 0 INPUT NODEFVAL "gxb_powerdown[0..0]"
-- Retrieval info: USED_PORT: pipe8b10binvpolarity 0 0 8 0 INPUT NODEFVAL "pipe8b10binvpolarity[7..0]"
-- Retrieval info: USED_PORT: pipedatavalid 0 0 8 0 OUTPUT NODEFVAL "pipedatavalid[7..0]"
-- Retrieval info: USED_PORT: pipeelecidle 0 0 8 0 OUTPUT NODEFVAL "pipeelecidle[7..0]"
-- Retrieval info: USED_PORT: pipephydonestatus 0 0 8 0 OUTPUT NODEFVAL "pipephydonestatus[7..0]"
-- Retrieval info: USED_PORT: pipestatus 0 0 24 0 OUTPUT NODEFVAL "pipestatus[23..0]"
-- Retrieval info: USED_PORT: pll_inclk 0 0 0 0 INPUT NODEFVAL "pll_inclk"
-- Retrieval info: USED_PORT: pll_locked 0 0 2 0 OUTPUT NODEFVAL "pll_locked[1..0]"
-- Retrieval info: USED_PORT: powerdn 0 0 16 0 INPUT NODEFVAL "powerdn[15..0]"
-- Retrieval info: USED_PORT: rx_analogreset 0 0 1 0 INPUT NODEFVAL "rx_analogreset[0..0]"
-- Retrieval info: USED_PORT: rx_coreclk 0 0 8 0 INPUT NODEFVAL "rx_coreclk[7..0]"
-- Retrieval info: USED_PORT: rx_ctrldetect 0 0 16 0 OUTPUT NODEFVAL "rx_ctrldetect[15..0]"
-- Retrieval info: USED_PORT: rx_datain 0 0 8 0 INPUT NODEFVAL "rx_datain[7..0]"
-- Retrieval info: USED_PORT: rx_dataout 0 0 128 0 OUTPUT NODEFVAL "rx_dataout[127..0]"
-- Retrieval info: USED_PORT: rx_digitalreset 0 0 1 0 INPUT NODEFVAL "rx_digitalreset[0..0]"
-- Retrieval info: USED_PORT: rx_disperr 0 0 16 0 OUTPUT NODEFVAL "rx_disperr[15..0]"
-- Retrieval info: USED_PORT: rx_errdetect 0 0 16 0 OUTPUT NODEFVAL "rx_errdetect[15..0]"
-- Retrieval info: USED_PORT: rx_freqlocked 0 0 8 0 OUTPUT NODEFVAL "rx_freqlocked[7..0]"
-- Retrieval info: USED_PORT: rx_patterndetect 0 0 16 0 OUTPUT NODEFVAL "rx_patterndetect[15..0]"
-- Retrieval info: USED_PORT: rx_pll_locked 0 0 8 0 OUTPUT NODEFVAL "rx_pll_locked[7..0]"
-- Retrieval info: USED_PORT: rx_syncstatus 0 0 16 0 OUTPUT NODEFVAL "rx_syncstatus[15..0]"
-- Retrieval info: USED_PORT: tx_coreclk 0 0 8 0 INPUT NODEFVAL "tx_coreclk[7..0]"
-- Retrieval info: USED_PORT: tx_ctrlenable 0 0 16 0 INPUT NODEFVAL "tx_ctrlenable[15..0]"
-- Retrieval info: USED_PORT: tx_datain 0 0 128 0 INPUT NODEFVAL "tx_datain[127..0]"
-- Retrieval info: USED_PORT: tx_dataout 0 0 8 0 OUTPUT NODEFVAL "tx_dataout[7..0]"
-- Retrieval info: USED_PORT: tx_detectrxloop 0 0 8 0 INPUT NODEFVAL "tx_detectrxloop[7..0]"
-- Retrieval info: USED_PORT: tx_digitalreset 0 0 1 0 INPUT NODEFVAL "tx_digitalreset[0..0]"
-- Retrieval info: USED_PORT: tx_forcedispcompliance 0 0 8 0 INPUT NODEFVAL "tx_forcedispcompliance[7..0]"
-- Retrieval info: USED_PORT: tx_forceelecidle 0 0 8 0 INPUT NODEFVAL "tx_forceelecidle[7..0]"
-- Retrieval info: CONNECT: @fixedclk 0 0 0 0 fixedclk 0 0 0 0
-- Retrieval info: CONNECT: @tx_detectrxloop 0 0 8 0 tx_detectrxloop 0 0 8 0
-- Retrieval info: CONNECT: @tx_forcedispcompliance 0 0 8 0 tx_forcedispcompliance 0 0 8 0
-- Retrieval info: CONNECT: rx_patterndetect 0 0 16 0 @rx_patterndetect 0 0 16 0
-- Retrieval info: CONNECT: pipedatavalid 0 0 8 0 @pipedatavalid 0 0 8 0
-- Retrieval info: CONNECT: @rx_analogreset 0 0 1 0 rx_analogreset 0 0 1 0
-- Retrieval info: CONNECT: @powerdn 0 0 16 0 powerdn 0 0 16 0
-- Retrieval info: CONNECT: pipeelecidle 0 0 8 0 @pipeelecidle 0 0 8 0
-- Retrieval info: CONNECT: rx_ctrldetect 0 0 16 0 @rx_ctrldetect 0 0 16 0
-- Retrieval info: CONNECT: @gxb_powerdown 0 0 1 0 gxb_powerdown 0 0 1 0
-- Retrieval info: CONNECT: rx_dataout 0 0 128 0 @rx_dataout 0 0 128 0
-- Retrieval info: CONNECT: @cal_blk_clk 0 0 0 0 cal_blk_clk 0 0 0 0
-- Retrieval info: CONNECT: pipestatus 0 0 24 0 @pipestatus 0 0 24 0
-- Retrieval info: CONNECT: rx_errdetect 0 0 16 0 @rx_errdetect 0 0 16 0
-- Retrieval info: CONNECT: @tx_coreclk 0 0 8 0 tx_coreclk 0 0 8 0
-- Retrieval info: CONNECT: @tx_digitalreset 0 0 1 0 tx_digitalreset 0 0 1 0
-- Retrieval info: CONNECT: rx_pll_locked 0 0 8 0 @rx_pll_locked 0 0 8 0
-- Retrieval info: CONNECT: coreclkout 0 0 2 0 @coreclkout 0 0 2 0
-- Retrieval info: CONNECT: rx_syncstatus 0 0 16 0 @rx_syncstatus 0 0 16 0
-- Retrieval info: CONNECT: @pipe8b10binvpolarity 0 0 8 0 pipe8b10binvpolarity 0 0 8 0
-- Retrieval info: CONNECT: @rx_coreclk 0 0 8 0 rx_coreclk 0 0 8 0
-- Retrieval info: CONNECT: pll_locked 0 0 2 0 @pll_locked 0 0 2 0
-- Retrieval info: CONNECT: @rx_digitalreset 0 0 1 0 rx_digitalreset 0 0 1 0
-- Retrieval info: CONNECT: @pll_inclk 0 0 0 0 pll_inclk 0 0 0 0
-- Retrieval info: CONNECT: @tx_ctrlenable 0 0 16 0 tx_ctrlenable 0 0 16 0
-- Retrieval info: CONNECT: tx_dataout 0 0 8 0 @tx_dataout 0 0 8 0
-- Retrieval info: CONNECT: @tx_datain 0 0 128 0 tx_datain 0 0 128 0
-- Retrieval info: CONNECT: rx_freqlocked 0 0 8 0 @rx_freqlocked 0 0 8 0
-- Retrieval info: CONNECT: @rx_datain 0 0 8 0 rx_datain 0 0 8 0
-- Retrieval info: CONNECT: rx_disperr 0 0 16 0 @rx_disperr 0 0 16 0
-- Retrieval info: CONNECT: pipephydonestatus 0 0 8 0 @pipephydonestatus 0 0 8 0
-- Retrieval info: CONNECT: @tx_forceelecidle 0 0 8 0 tx_forceelecidle 0 0 8 0
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.vhd TRUE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.inc FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.cmp FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx.bsf FALSE FALSE
-- Retrieval info: GEN_FILE: TYPE_NORMAL alt2gxb_s2gx_inst.vhd FALSE FALSE
